{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Dialogs;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, System.SysUtils, Types, WEBLib.Controls, WEBLib.StdCtrls, WEBLib.WebCtrls,
  Vcl.Controls, System.UITypes, VCL.Graphics, VCL.Imaging.pngimage, VCL.Imaging.JPEG, VCL.Imaging.GIFImg;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Types, WEBLib.Controls, WEBLib.StdCtrls, WEBLib.WebCtrls,
  Controls, Graphics;
  {$ENDIF}

type
  TMsgDlgType = (mtWarning, mtError, mtInformation, mtConfirmation, mtCustom);
  TMsgDlgBtn = (mbYes, mbNo, mbOK, mbCancel, mbAbort, mbRetry, mbIgnore,
    mbAll, mbNoToAll, mbYesToAll, mbHelp, mbClose);
  TMsgDlgButtons = set of TMsgDlgBtn;

  {$IFDEF LCLTMSWEB}
  TModalResult = integer;
  {$ENDIF}

  TDialogResult = integer;
  TDialogResultProc = procedure(AValue: TModalResult);

  TCustomDialogButton = class(TCollectionItem)
  private
    FCaption: string;
    FTag: integer;
    FElementClassName: TElementClassName;
  published
    property Caption: string read FCaption write FCaption;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property Tag: integer read FTag write FTag default 0;
  end;

  TCustomDialogButtons = class(TCollection)
  private
    function GetItem(AIndex: integer): TCustomDialogButton; reintroduce;
    procedure SetItem(AIndex: integer; const Value: TCustomDialogButton); reintroduce;
  public
    function Add: TCustomDialogButton; reintroduce;
    function Insert(AIndex: integer): TCustomDialogButton; reintroduce;
    property Items[AIndex: integer]: TCustomDialogButton read GetItem write SetItem; default;
  end;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebMessageDlg = class(TWebNonVisualCustomControl)
  private
    FDialogResult: integer;
    FOnButtonClick: TNotifyEvent;
    FOnClose: TNotifyEvent;
    FOpacity: double;
    FElementButtonClassName: TElementClassName;
    FElementTitleClassName: TElementClassName;
    FElementContentClassName: TElementClassName;
    FDialogText: TStringList;
    FMessage: string;
    FTitle: string;
    FButtons: TMsgDlgButtons;
    FMsgDlgType: TMSgDlgType;
    FCustomButtons: TCustomDialogButtons;
    FElementDialogClassName: TElementClassName;
    procedure SetDialogText(const Value: TStringList);
    procedure SetCustomButtons(const Value: TCustomDialogButtons);
  protected
    procedure InitBootstrapStyle; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ShowDialog(Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; AProc: TDialogResultProc = nil): TModalResult; virtual;
    property DialogResult: TDialogResult read FDialogResult write FDialogResult;
  published
    property Buttons: TMsgDlgButtons read FButtons write FButtons;
    property CustomButtons: TCustomDialogButtons read FCustomButtons write SetCustomButtons;
    property DialogText: TStringList read FDialogText write SetDialogText;
    property DialogType: TMSgDlgType read FMsgDlgType write FMsgDlgType default mtCustom;
    property Height;
    property Left;
    property &Message: string read FMessage write FMessage;
    property Opacity: double read FOpacity write FOpacity;
    property Title: string read FTitle write FTitle;
    property Top;
    property Visible;
    property Width;
    property ElementButtonClassName: TElementClassName read FElementButtonClassName write FElementButtonClassName;
    property ElementDialogClassName: TElementClassName read FElementDialogClassName write FElementDialogClassName;
    property ElementTitleClassName: TElementClassName read FElementTitleClassName write FElementTitleClassName;
    property ElementContentClassName: TElementClassName read FElementContentClassName write FElementContentClassName;
    property OnButtonClick: TNotifyEvent read FOnButtonClick write FOnButtonClick;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebWaitMessage = class(TWebNonVisualCustomControl)
  private
    FPicture: TPicture;
    FOpacity: double;
    FOnShow: TNotifyEvent;
    procedure SetPicture(const Value: TPicture);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Show;
    procedure Hide;
  published
    property Opacity: double read FOpacity write FOpacity;
    property Picture: TPicture read FPicture write SetPicture;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebOpenDialog = class(TComponent)
  private
    FFiles: TFiles;
    FFileName: string;
    FOnChange: TNotifyEvent;
    FMultiFile: boolean;
    FAccept: string;
  public
    procedure Execute;
    property FileName: string read FFileName;
    property Files: TFiles read FFiles;
  published
    property Accept: string read FAccept write FAccept;
    property MultiFile: boolean read FMultiFile write FMultiFile default false;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;



procedure MessageDlg(const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; AResultProc: TDialogResultProc = nil);
procedure ShowMessage(Value: string);
function InputBox(const ACaption, APrompt, ADefault: string): string;
function InputQuery(const ACaption, APrompt: string; var Value: string): Boolean; overload;
function Confirm(const Value: string): boolean;

implementation

procedure ShowMessage(Value: string);
begin
//
end;

function InputBox(const ACaption, APrompt, ADefault: string): string;
begin
  Result := '';
end;

function InputQuery(const ACaption, APrompt: string; var Value: string): Boolean; overload;
begin
  Result := true;
end;

procedure MessageDlg(const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;  AResultProc: TDialogResultProc = nil);
begin
//
end;

function Confirm(const Value: string): boolean;
begin
  Result := true;
end;

{ TWebMessageDlg }

constructor TWebMessageDlg.Create(AOwner: TComponent);
begin
  inherited;
  FOpacity := 0.2;
  FDialogText := TStringList.Create;
  FDialogText.Add('Warning'); // 0
  FDialogText.Add('Error');   // 1
  FDialogText.Add('Information'); // 2
  FDialogText.Add('Confirm');     // 3
  FDialogText.Add('Custom');      // 4
  FDialogText.Add('OK');          // 5
  FDialogText.Add('Cancel');      // 6
  FDialogText.Add('Yes');         // 7
  FDialogText.Add('No');          // 8
  FDialogText.Add('Abort');       // 9
  FDialogText.Add('Retry');       // 10
  FDialogText.Add('Ignore');      // 11
  FDialogText.Add('All');         // 12
  FDialogText.Add('Yes to All');  // 13
  FDialogText.Add('No to All');   // 14
  FDialogText.Add('Help');        // 15
  FDialogText.Add('Close');       // 16
  FMsgDlgType := mtCustom;
  FCustomButtons := TCustomDialogButtons.Create(TCustomDialogButton);
end;

destructor TWebMessageDlg.Destroy;
begin
  FDialogText.Free;
  FCustomButtons.Free;
  inherited;
end;

procedure TWebMessageDlg.InitBootstrapStyle;
begin
  inherited;
  FElementButtonClassName := 'btn';
  FElementContentClassName := 'text-body';
  FElementTitleClassName := 'text-body';
  FElementDialogClassName := 'shadow-lg p-3 mb-5 bg-white rounded';
end;

procedure TWebMessageDlg.SetCustomButtons(const Value: TCustomDialogButtons);
begin
  FCustomButtons.Assign(Value);
end;

procedure TWebMessageDlg.SetDialogText(const Value: TStringList);
begin
  FDialogText.Assign(Value);
end;

function TWebMessageDlg.ShowDialog(Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; AProc: TDialogResultProc = nil): TModalResult;
begin
  Result := DialogResult;
  if Assigned(AProc) then
    AProc(Result);
end;


{ TWebWaitMessage }

constructor TWebWaitMessage.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
  Stream: TCustomMemoryStream;
  ResName: string;
  gif: TGIFImage;

begin
  inherited;

  FOpacity := 0.2;

  FPicture := TPicture.Create;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  ResName := 'DEFAULTWAITCURSOR';

  if FDesignTime and (FindResource(HInstance, PChar(ResName), RT_RCDATA) <> 0) then
  begin
    Stream := TResourceStream.Create(HInstance, ResName, RT_RCDATA);
    try
      gif := TGIFImage.Create;
      gif.LoadFromStream(Stream);
      FPicture.Assign(gif);
      gif.Free;
    finally
      Stream.Free;
    end;
  end;
end;

destructor TWebWaitMessage.Destroy;
begin
  FPicture.Free;
  inherited;
end;

procedure TWebWaitMessage.Hide;
begin
  //
end;

procedure TWebWaitMessage.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TWebWaitMessage.Show;
begin
  //
end;

{ TCustomDialogButtons }

function TCustomDialogButtons.Add: TCustomDialogButton;
begin
  Result := TCustomDialogButton(inherited Add);
end;

function TCustomDialogButtons.GetItem(AIndex: integer): TCustomDialogButton;
begin
  Result := TCustomDialogButton(inherited Items[AIndex]);
end;

function TCustomDialogButtons.Insert(AIndex: integer): TCustomDialogButton;
begin
  Result := TCustomDialogButton(inherited Insert(AIndex));
end;

procedure TCustomDialogButtons.SetItem(AIndex: integer;
  const Value: TCustomDialogButton);
begin
  inherited Items[AIndex] := Value;
end;


{ TWebOpenDialog }

procedure TWebOpenDialog.Execute;
begin
  //
end;

end.
