{***************************************************************************}
{ TMS Google Calendar access                                                }
{                                                                           }
{ written by TMS Software                                                   }
{            copyright  2018                                               }
{            Email : info@tmssoftware.com                                   }
{            Web : http://www.tmssoftware.com                               }
{                                                                           }
{ The source code is given as is. The author is not responsible             }
{ for any possible damage done due to the use of this code.                 }
{ The component can be freely used in any application. The complete         }
{ source code remains property of the author and may not be distributed,    }
{ published, given or sold in any form as such. No parts of the source      }
{ code can be included in any other component or application without        }
{ written authorization of the author.                                      }
{***************************************************************************}

unit WEBLib.GoogleCalendar;

interface

uses
  Classes, Graphics, WEBLib.REST, WEBLib.Controls, WEBLib.Google, SysUtils, StrUtils;

const
  MAJ_VER = 1; // Major version nr.
  MIN_VER = 0; // Minor version nr.
  REL_VER = 9; // Release nr.
  BLD_VER = 1; // Build nr.

  // version history
  // 1.0.0.0 : First release

var
  GColorNames: array[1..12] of string = ('Default', 'Blue', 'Green', 'Purple', 'Red', 'Yellow', 'Orange',
    'Turquoise', 'Gray', 'Boldblue', 'BoldGreen', 'BoldRed');

type
  TWebGoogleCalendar = class;
  TGCalendar = class;
  TGCalendars = class;
  TGCalendarItems = class;
  TGCalendarItem = class;
  TGAttendees = class;
  TGReminders = class;
  TGColors = class;

  TStatus = (stConfirmed, stTentative, stCancelled);
  TVisibility = (viDefault, viPublic, viPrivate);
  TResponseStatus = (rsNeedsAction, rsDeclined, rsTentative, rsAccepted);
  TReminderMethod = (rmPopup, rmEmail, rmSMS);
  TGItemColor = (icDefault, icBlue, icGreen, icPurple, icRed,
    icYellow, icOrange, icTurquoise, icGray, icBoldBlue,
    icBoldGreen, icBoldRed);
  TGCalendarColor = (ccBoldBrown, ccBrown, ccBoldRed, ccBoldOrange, ccBoldYellow,
    ccMediumGreen, ccDarkGreen, ccBoldGreen, ccLimeGreen, ccLightGreen, ccYellow,
    ccOrange, ccGreen, ccTurquoise, ccBlue, ccBoldBlue, ccBoldPurple, ccPurple,
    ccGray, ccLightGray, ccBoldGray, ccPink, ccBoldPing, ccDarkPurple);

  TGReminder = class(TCollectionItem)
  private
    FOwner: TGReminders;
    FMethod: TReminderMethod;
    FMinutes: integer;
    procedure SetMethod(const Value: TReminderMethod);
    procedure SetMinutes(const Value: integer);
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property Minutes: integer read FMinutes write SetMinutes default 10;
    property Method: TReminderMethod read FMethod write SetMethod default rmPopup;
  end;

  TGReminders = class(TCollection)
  private
    FOwner: TGCalendarItem;
    function GetItems(Index: Integer): TGReminder;
    procedure SetItems(Index: Integer; const Value: TGReminder);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGCalendarItem);
    function GetOwner: TPersistent; override;
    function Add: TGReminder;
    function Insert(Index: Integer):TGReminder;
    property Items[Index: Integer]: TGReminder read GetItems write SetItems; default;
  end;

  TGDefaultReminders = class(TCollection)
  private
    FOwner: TGCalendar;
    function GetItems(Index: Integer): TGReminder;
    procedure SetItems(Index: Integer; const Value: TGReminder);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGCalendar);
    function GetOwner: TPersistent; override;
    function Add: TGReminder;
    function Insert(Index: Integer):TGReminder;
    property Items[Index: Integer]: TGReminder read GetItems write SetItems; default;
  end;

  TGAttendee = class(TCollectionItem)
  private
    FOwner: TGAttendees;
    FSummary: string;
    FEmail: string;
    FStatus: TResponseStatus;
    procedure SetEmail(const Value: string);
    procedure SetStatus(const Value: TResponseStatus);
    procedure SetSummary(const Value: string);
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property Summary: string read FSummary write SetSummary;
    property Email: string read FEmail write SetEmail;
    property Status: TResponseStatus read FStatus write SetStatus default rsNeedsAction;
  end;

  TGAttendees = class(TCollection)
  private
    FOwner: TGCalendarItem;
    function GetItems(Index: Integer): TGAttendee;
    procedure SetItems(Index: Integer; const Value: TGAttendee);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGCalendarItem);
    function GetOwner: TPersistent; override;
    function Add: TGAttendee;
    function Insert(Index: Integer):TGAttendee;
    property Items[Index: Integer]: TGAttendee read GetItems write SetItems; default;
  end;

  TGCalendar = class(TCollectionItem)
  private
    FOwner: TGCalendars;
    FSummary: string;
    FDescription: string;
    FID: string;
    FEtag: string;
    FLocation: string;
    FTimeZone: string;
    FPrimary: boolean;
    FColor: TGCalendarColor;
    FForegroundColor: TColor;
    FBackgroundColor: TColor;
    FDefaultReminders: TGDefaultReminders;
    procedure SetDescription(const Value: string);
    procedure SetID(const Value: string);
    procedure SetSummary(const Value: string);
    procedure SetLocation(const Value: string);
    procedure SetTimeZone(const Value: string);
    procedure SetPrimary(const Value: boolean);
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property ID: string read FID write SetID;
    property Description: string read FDescription write SetDescription;
    property ETag: string read FEtag write FEtag;
    property Location: string read FLocation write SetLocation;
    property Primary: boolean read FPrimary write SetPrimary default false;
    property Summary: string read FSummary write SetSummary;
    property TimeZone: string read FTimeZone write SetTimeZone;
    property Color: TGCalendarColor read FColor write FColor default ccGray;
    property BackgroundColor: TColor read FBackgroundColor write FBackgroundColor default clNone;
    property ForeGroundColor: TColor read FForegroundColor write FForegroundColor default clNone;
    property DefaultReminders: TGDefaultReminders read FDefaultReminders write FDefaultReminders;
  end;

  TGCalendars = class(TCollection)
  private
    FOwner: TWebGoogleCalendar;
    function GetItems(Index: Integer): TGCalendar;
    procedure SetItems(Index: Integer; const Value: TGCalendar);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TWebGoogleCalendar);
    function GetOwner: TPersistent; override;
    function Add: TGCalendar;
    function Insert(Index: Integer):TGCalendar;
    property Items[Index: Integer]: TGCalendar read GetItems write SetItems; default;
  end;

  TGCalendarUser = class(TPersistent)
  private
    FDisplayName: string;
    FEmail: string;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
  published
    property Email: string read FEmail write FEmail;
    property DisplayName: string read FDisplayName write FDisplayName;
  end;


  TGCalendarItem = class(TCollectionItem)
  private
    FOwner: TGCalendarItems;
    FLocation: string;
    FSummary: string;
    FStartTime: TDateTime;
    FEndTime: TDateTime;
    FDescription: string;
    FID: string;
    FVisibility: TVisibility;
    FStatus: TStatus;
    FRecurrence: string;
    FCalendarID: string;
//    FOverlap: boolean;
    FAttendees: TGAttendees;
    FIsAllDay: boolean;
    FRecurringID: string;
    FETag: string;
    FSequence: integer;
    FUpdated: TDateTime;
    FCreated: TDateTime;
    FTimeZone: String;
    FUseDefaultReminders: boolean;
    FReminders: TGReminders;
    FColor: TGItemColor;
    FSendNotifications: boolean;
    FOrganizer: TGCalendarUser;
    FCreator: TGCalendarUser;
    procedure SetDescription(const Value: string);
    procedure SetEndTime(const Value: TDateTime);
    procedure SetLocation(const Value: string);
    procedure SetSummary(const Value: string);
    procedure SetStartTime(const Value: TDateTime);
    procedure SetID(const Value: string);
    procedure SetVisibility(const Value: TVisibility);
    procedure SetCalendarID(const Value: string);
    procedure SetIsAllDay(const Value: boolean);
//    procedure SetOverlap(const Value: boolean);
    procedure SetRecurrence(const Value: string);
    procedure SetRecurringID(const Value: string);
    procedure SetSequence(const Value: integer);
    procedure SetCreated(const Value: TDateTime);
    procedure SetUpdated(const Value: TDateTime);
    procedure SetCreator(const Value: TGCalendarUser);
    procedure SetOrganizer(const Value: TGCalendarUser);

  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property ID: string read FID write SetID;
    property ETag: string read FETag write FETag;
    property CalendarID: string read FCalendarID write SetCalendarID;
    property Summary: string read FSummary write SetSummary;
    property Description: string read FDescription write SetDescription;
    property StartTime: TDateTime read FStartTime write SetStartTime;
    property EndTime: TDateTime read FEndTime write SetEndTime;
    property Created: TDateTime read FCreated write SetCreated;
    property Updated: TDateTime read FUpdated write SetUpdated;
    property IsAllDay: boolean read FIsAllDay write SetIsAllDay default false;
    property Location: string read FLocation write SetLocation;
    property Status: TStatus read FStatus default stConfirmed;
    property Visibility: TVisibility read FVisibility write SetVisibility default viPublic;
    property Recurrence: string read FRecurrence write SetRecurrence;
    property RecurringID: string read FRecurringID write SetRecurringID;
//    property Overlap: boolean read FOverlap write SetOverlap default false;
    property Sequence: integer read FSequence write SetSequence;
    property Attendees: TGAttendees read FAttendees write FAttendees;
    property Reminders: TGReminders read FReminders write FReminders;
    property UseDefaultReminders: boolean read FUseDefaultReminders write FUseDefaultReminders default true;
    property TimeZone: String read FTimeZone write FTimeZone;
    property Color: TGItemColor read FColor write FColor default icDefault;
    property Creator: TGCalendarUser read FCreator write SetCreator;
    property Organizer: TGCalendarUser read FOrganizer write SetOrganizer;
    property SendNotifications: boolean read FSendNotifications write FSendNotifications default false;
  end;

  TGCalendarItems = class(TCollection)
  private
    FOwner: TWebGoogleCalendar;
    function GetItems(Index: Integer): TGCalendarItem;
    procedure SetItems(Index: Integer; const Value: TGCalendarItem);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TWebGoogleCalendar);
    function GetOwner: TPersistent; override;
    function Add: TGCalendarItem;
    function Find(const ID: integer): TGCalendarItem; overload;
    function Find(const ID: string): TGCalendarItem; overload;
    function Insert(Index: Integer): TGCalendarItem;
    function GetItemByID(const ID: string): TGCalendarItem;
    property Items[Index: Integer]: TGCalendarItem read GetItems write SetItems; default;
  end;

  TGColor = class(TCollectionItem)
  private
    FOwner: TGColors;
    FForegroundColor: TColor;
    FBackgroundColor: TColor;
    FID: integer;
    procedure SetBackGroundColor(const Value: TColor);
    procedure SetForeGroundColor(const Value: TColor);
    procedure SetID(const Value: integer);
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property ID: integer read FID write SetID default -1;
    property BackgroundColor: TColor read FBackgroundColor write SetBackGroundColor default clNone;
    property ForegroundColor: TColor read FForegroundColor write SetForeGroundColor default clNone;
  end;

  TGColors = class(TCollection)
  private
    FOwner: TWebGoogleCalendar;
    function GetItems(Index: Integer): TGColor;
    procedure SetItems(Index: Integer; const Value: TGColor);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TWebGoogleCalendar);
    function GetOwner: TPersistent; override;
    function Add: TGColor;
    function Insert(Index: Integer): TGColor;
    property Items[Index: Integer]: TGColor read GetItems write SetItems; default;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebGoogleCalendar = class(TWebGoogleClient)
  private
    APIBase: string;
    FItems: TGCalendarItems;
    FResult: TGCalendarItems;
    FCalendars: TGCalendars;
    FItemColors: TGColors;
    FCalendarColors: TGColors;
    FOnCalendars: TNotifyEvent;
    FOnCalendar: TNotifyEvent;
  protected
    procedure GetCalendarInt(ID: string; FromDate, ToDate: TDate; ChangedSince: TDateTime; MaxResults: integer = 250);
    function GetCalendarData(Item: TGCalendarItem): string;
    procedure ParseCalendar(jo: TJSONObject; Item: TGCalendar);
    procedure ParseCalendarItem(CalendarID: string; jo: TJSONObject; Item: TGCalendaritem);
    procedure ParseColor(jo: TJSONObject; ColorType: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddCalendar(Calendar: TGCalendar);
    procedure DeleteCalendar(Calendar: TGCalendar);
    procedure UpdateCalendar(Calendar: TGCalendar);
    procedure GetCalendars;
    procedure GetCalendar(ID: string; FromDate, ToDate: TDate; MaxResults: integer = 250); overload;
    procedure GetCalendar(FromDate, ToDate: TDate; MaxResults: integer = 250); overload;
    procedure GetCalendar(ID: string; ChangedSince: TDateTime; MaxResults: integer = 250); overload;
    procedure GetCalendar(ID: string; MaxResults: integer = 250); overload;
    procedure GetColors;
    function GetItemByID(CalendarID, ItemID: string): TGCalendarItem;
    procedure Add(Item: TGCalendarItem);
    procedure Update(Item: TGCalendarItem);
    procedure Delete(Item: TGCalendarItem);
    property ItemColors: TGColors read FItemColors write FItemColors;
    property CalendarColors: TGColors read FCalendarColors write FCalendarColors;
    function AccessTokenLifeTime: integer;
  published
    property Calendars: TGCalendars read FCalendars write FCalendars;
    property Items: TGCalendarItems read FItems write FItems;
    property OnCalendars: TNotifyEvent read FOnCalendars write FOnCalendars;
    property OnCalendar: TNotifyEvent read FOnCalendar write FOnCalendar;
  end;

implementation

function RGBToColor(const RGBString: string): TColor;
var
  RGBStringToConvert: string;
  RBGStringChecked  : string;
begin
  if LeftStr(RGBString, 1) = '#' then
    RBGStringChecked := RightStr(RGBString, Length(RGBString) -1)
  else
    RBGStringChecked := RGBString;

  //Put in proper order for the StrToInt conversion
  //(expects as B, G, R and NOT IN RGB order).
  RGBStringToConvert    := '$'
    + Copy(RBGStringChecked, 5, 2)
    + Copy(RBGStringChecked, 3, 2)
    + Copy(RBGStringChecked, 1, 2);

  Result := TColor(StrToInt(string(RGBStringToConvert)));
end;

{ TWebGoogleCalendar }

constructor TWebGoogleCalendar.Create(AOwner: TComponent);
begin
  inherited;
  APIBase := 'https://www.googleapis.com';

  FCalendars := TGCalendars.Create(Self);
  FItems := TGCalendarItems.Create(self);
  FResult := TGCalendarItems.Create(self);
  FItemColors := TGColors.Create(Self);
  FCalendarColors := TGColors.Create(Self);

  //required scopes
  Scopes.Add('https://www.googleapis.com/auth/calendar');

//  App.Service := 'Google Calendar';
//  App.DevURL := 'https://code.google.com/apis/console/';
end;

destructor TWebGoogleCalendar.Destroy;
begin
  FResult.Free;
  FCalendars.Free;
  FItems.Free;
  FItemColors.Free;
  FCalendarColors.Free;
  inherited;
end;

procedure ParseJSON(Data: string; var JSONObject: TJSONObject);
begin
end;

procedure TWebGoogleCalendar.GetCalendar(ID: string; FromDate, ToDate: TDate; MaxResults: integer = 250);
begin
  GetCalendarInt(ID,FromDate,ToDate,0,MaxResults);
end;

procedure TWebGoogleCalendar.GetCalendar(ID: string; ChangedSince: TDateTime; MaxResults: integer = 250);
begin
  GetCalendarInt(ID,0,0,ChangedSince,MaxResults);
end;

procedure TWebGoogleCalendar.GetCalendarInt(ID: string; FromDate, ToDate: TDate; ChangedSince: TDateTime; MaxResults: integer = 250);
var
  url: string;
//  o, fo: TJSONObject;
//  ja: TJSONArray;
//  i: integer;
//  item: TGCalendarItem;
//  resdat: ansistring;
//  strRes: string;
  headers: TCoreCloudHeaders;
//  jv: TJSONValue;
begin
  url := APIBase + '/calendar/v3/calendars/' + URLEncode(ID) + '/events';

  //load items in chronological order
  //get instances of recurrent items
//  url := url + '?key=' + App.Key + '&singleEvents=True&orderBy=startTime&maxResults=' + IntToStr(MaxResults);
  url := url + '?singleEvents=True&orderBy=startTime&maxResults=' + IntToStr(MaxResults);

//  include deleted events
//  url := url + '?key=' + App.Key + '&singleEvents=True&orderBy=startTime&showDeleted=true';

  //load items in order of creation
  //get parent recurrent item only
  //url := url + '?key=' + App.Key;

  if (FromDate <> 0) and (ToDate <> 0)  then
  begin
    url := url + '&timeMin=' + URLEncode(DateTimeToWL(FromDate))
      + '&timeMax=' + URLEncode(DateTimeToWL(ToDate));
  end;

  if (ChangedSince <> 0) then
  begin
    url := url + '&updatedMin=' + URLEncode(DateTimeToWL(ChangedSince));
  end;

  AddHeader(headers, 'Authorization', 'Bearer ' + Token_Access);
  AddHeader(headers, 'Content-Type', 'application/json');

//  OnResponse := HandleGetCalendars;

//  HttpsGet(url, headers);

  (*
  strRes := StringReplace(strRes, #10, '', [rfReplaceAll]);
  strRes := StringReplace(strRes, #13, '', [rfReplaceAll]);

  jv := TJSONObject.ParseJSONValue(strres);

  Items.Clear;

  if Assigned(jv) and (jv is TJSONObject) then
  begin
    try
      o := jv as TJSONObject;
      ja := GetJSONValue(o,'items') as TJSONArray;

      if Assigned(ja) then
      begin
        for i := 0 to GetArraySize(ja) - 1 do
        begin
          fo := GetArrayItem(ja,i) as TJSONObject;
          if Assigned(fo) then
          begin
            item := Items.Add;
            ParseCalendarItem(ID, fo, item);
          end;
        end;
      end;
    finally
      jv.Free;
    end;
  end;
  *)
end;

procedure TWebGoogleCalendar.GetCalendar(ID: string; MaxResults: integer = 250);
begin
  GetCalendar(ID, 0, 0, MaxResults);
end;

procedure TWebGoogleCalendar.GetCalendar(FromDate, ToDate: TDate; MaxResults: integer = 250);
begin
  GetCalendar(EmptyStr, FromDate, ToDate, 250);
end;

function TWebGoogleCalendar.GetCalendarData(Item: TGCalendarItem): string;
begin
end;

procedure TWebGoogleCalendar.GetCalendars;
begin
end;

procedure TWebGoogleCalendar.ParseColor(jo: TJSONObject; ColorType: string);
begin
end;

procedure TWebGoogleCalendar.GetColors;
begin
end;

function TWebGoogleCalendar.GetItemByID(CalendarID, ItemID: string): TGCalendarItem;
begin
  Result := nil;
end;

procedure TWebGoogleCalendar.ParseCalendar(jo: TJSONObject; Item: TGCalendar);
begin
end;

procedure TWebGoogleCalendar.ParseCalendarItem(CalendarID:string; jo: TJSONObject; Item: TGCalendaritem);
begin
end;

function TWebGoogleCalendar.AccessTokenLifeTime: integer;
begin
  Result := 0;
end;

procedure TWebGoogleCalendar.Add(Item: TGCalendarItem);
begin
end;

procedure TWebGoogleCalendar.AddCalendar(Calendar: TGCalendar);
begin
end;

procedure TWebGoogleCalendar.Delete(Item: TGCalendarItem);
begin
end;

procedure TWebGoogleCalendar.DeleteCalendar(Calendar: TGCalendar);
begin
end;

procedure TWebGoogleCalendar.Update(Item: TGCalendarItem);
begin
end;


procedure TWebGoogleCalendar.UpdateCalendar(Calendar: TGCalendar);
begin
end;

{ TGCalendarItem }

procedure TGCalendarItem.Assign(Source: TPersistent);
begin
end;

constructor TGCalendarItem.Create(ACollection: TCollection);
begin
  inherited;
  FID := EmptyStr;
  FCalendarID := EmptyStr;
  FSummary := 'Item ' + IntToStr(ACollection.Count - 1);
  FDescription := EmptyStr;
  FStartTime := Date;
  FEndTime := Date;
  FCreated := 0;
  FUpdated := 0;
  FLocation := EmptyStr;
  FStatus := stConfirmed;
  FVisibility := viPublic;
  FRecurrence := EmptyStr;
  FRecurringID := EmptyStr;
  FIsAllDay := false;
  FAttendees := TGAttendees.Create(Self);
  FSequence := -1;
  FTimeZone := EmptyStr;
  FUseDefaultReminders := true;
  FReminders := TGReminders.Create(Self);
  FColor := icDefault;
  FSendNotifications := false;

  if ACollection is TGCalendarItems then
  begin
    FOwner := ACollection as TGCalendarItems;
  end;

  FCreator := TGCalendarUser.Create;
  FOrganizer := TGCalendarUser.Create;
end;

destructor TGCalendarItem.Destroy;
begin
  FAttendees.Free;
  FReminders.Free;
  FCreator.Free;
  FOrganizer.Free;

  inherited;
end;

function TGCalendarItem.GetDisplayName: string;
begin
  if Summary <> '' then
    Result := Summary
  else
    Result := 'Item '+ IntToStr(Index);
end;

function TGCalendarItem.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

procedure TGCalendarItem.SetCalendarID(const Value: string);
begin
  FCalendarID := Value;
end;

procedure TGCalendarItem.SetCreated(const Value: TDateTime);
begin
  FCreated := Value;
end;

procedure TGCalendarItem.SetCreator(const Value: TGCalendarUser);
begin
  FCreator.Assign(Value);
end;

procedure TGCalendarItem.SetDescription(const Value: string);
begin
  FDescription := Value;
end;

procedure TGCalendarItem.SetEndTime(const Value: TDateTime);
begin
  FEndTime := Value;
end;

procedure TGCalendarItem.SetID(const Value: string);
begin
  FID := Value;
end;

procedure TGCalendarItem.SetIsAllDay(const Value: boolean);
begin
  FIsAllDay := Value;
end;

procedure TGCalendarItem.SetLocation(const Value: string);
begin
  FLocation := Value;
end;

procedure TGCalendarItem.SetOrganizer(const Value: TGCalendarUser);
begin
  FOrganizer.Assign(Value);
end;

procedure TGCalendarItem.SetSummary(const Value: string);
begin
  FSummary := Value;
end;

procedure TGCalendarItem.SetUpdated(const Value: TDateTime);
begin
  FUpdated := Value;
end;

//procedure TGCalendarItem.SetOverlap(const Value: boolean);
//begin
//  FOverlap := Value;
//end;

procedure TGCalendarItem.SetRecurrence(const Value: string);
begin
  FRecurrence := Value;
end;

procedure TGCalendarItem.SetRecurringID(const Value: string);
begin
  FRecurringID := Value;
end;

procedure TGCalendarItem.SetSequence(const Value: integer);
begin
  FSequence := Value;
end;

procedure TGCalendarItem.SetStartTime(const Value: TDateTime);
begin
  FStartTime := Value;
end;

procedure TGCalendarItem.SetVisibility(const Value: TVisibility);
begin
  FVisibility := Value;
end;

{ TGCalendarItems }

function TGCalendarItems.Add: TGCalendarItem;
begin
  Result := TGCalendarItem(inherited Add);
end;

constructor TGCalendarItems.Create(AOwner: TWebGoogleCalendar);
begin
  inherited Create(TGCalendarItem);
  FOwner := AOwner;
end;

function TGCalendarItems.Find(const ID: integer): TGCalendarItem;
begin
  Result := Find(IntToStr(ID));
end;

function TGCalendarItems.Find(const ID: string): TGCalendarItem;
var
  i: integer;
begin
  Result := nil;

  for i := 0 to Count - 1 do
  begin
    if Items[i].ID = ID then
    begin
      Result := Items[i];
      break;
    end;
  end;
end;

function TGCalendarItems.GetItemByID(const ID: string): TGCalendarItem;
var
  i: integer;
begin
  Result := nil;

  for i := 0 to Count - 1 do
  begin
    if Items[i].ID = ID then
    begin
      Result := Items[i];
      break;
    end;
  end;
end;

function TGCalendarItems.GetItems(Index: Integer): TGCalendarItem;
begin
  Result := TGCalendarItem(inherited Items[Index]);
end;

function TGCalendarItems.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TGCalendarItems.Insert(Index: Integer): TGCalendarItem;
begin
  Result := TGCalendarItem(inherited Insert(Index));
end;

procedure TGCalendarItems.SetItems(Index: Integer;
  const Value: TGCalendarItem);
begin
  inherited Items[Index] := Value;
end;

procedure TGCalendarItems.Update(Item: TCollectionItem);
begin
  inherited;
end;


{ TGCalendar }

procedure TGCalendar.Assign(Source: TPersistent);
begin
  if (Source is TGCalendar) then
  begin
    FID := (Source as TGCalendar).ID;
    FEtag := (Source as TGCalendar).ETag;
    FSummary := (Source as TGCalendar).Summary;
    FDescription := (Source as TGCalendar).Description;
    FTimeZone := (Source as TGCalendar).TimeZone;
    FLocation := (Source as TGCalendar).Location;
    FPrimary := (Source as TGCalendar).Primary;
    FColor := (Source as TGCalendar).Color;
  end;
end;

constructor TGCalendar.Create(ACollection: TCollection);
begin
  inherited;
  FID := EmptyStr;
  FSummary := 'Item ' + IntToStr(ACollection.Count - 1);
  FDescription := EmptyStr;
  FTimeZone := EmptyStr;
  FLocation := EmptyStr;
  FPrimary := false;
  FColor := ccGray;
  FDefaultReminders := TGDefaultReminders.Create(Self);

  if ACollection is TGCalendars then
  begin
    FOwner := ACollection as TGCalendars;
  end;
end;

destructor TGCalendar.Destroy;
begin
  FDefaultReminders.Free;
  inherited;
end;

function TGCalendar.GetDisplayName: string;
begin
  if Summary <> '' then
    Result := Summary
  else
    Result := 'Calendar '+ IntToStr(Index);
end;

function TGCalendar.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

procedure TGCalendar.SetDescription(const Value: string);
begin
  FDescription := Value;
end;

procedure TGCalendar.SetID(const Value: string);
begin
  FID := Value;
end;

procedure TGCalendar.SetLocation(const Value: string);
begin
  FLocation := Value;
end;

procedure TGCalendar.SetPrimary(const Value: boolean);
begin
  FPrimary := Value;
end;

procedure TGCalendar.SetSummary(const Value: string);
begin
  FSummary := Value;
end;

procedure TGCalendar.SetTimeZone(const Value: string);
begin
  FTimeZone := Value;
end;

{ TGCalendars }

function TGCalendars.Add: TGCalendar;
begin
  Result := TGCalendar(inherited Add);
end;

constructor TGCalendars.Create(AOwner: TWebGoogleCalendar);
begin
  inherited Create(TGCalendar);
  FOwner := AOwner;
end;

function TGCalendars.GetItems(Index: Integer): TGCalendar;
begin
  Result := TGCalendar(inherited Items[Index]);
end;

function TGCalendars.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TGCalendars.Insert(Index: Integer): TGCalendar;
begin
  Result := TGCalendar(inherited Insert(Index));
end;

procedure TGCalendars.SetItems(Index: Integer; const Value: TGCalendar);
begin
  inherited Items[Index] := Value;
end;

procedure TGCalendars.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TGAttendee }

procedure TGAttendee.Assign(Source: TPersistent);
begin
  if (Source is TGAttendee) then
  begin
    FEmail := (Source as TGAttendee).Email;
    FSummary := (Source as TGAttendee).Summary;
    FStatus := (Source as TGAttendee).Status
  end;
end;

constructor TGAttendee.Create(ACollection: TCollection);
begin
  inherited;
  FEmail := EmptyStr;
  FSummary := 'Attendee ' + IntToStr(ACollection.Count - 1);
  FStatus := rsNeedsAction;

  if ACollection is TGAttendees then
  begin
    FOwner := ACollection as TGAttendees;
  end;
end;

destructor TGAttendee.Destroy;
begin
  inherited;
end;

function TGAttendee.GetDisplayName: string;
begin
  if Summary <> '' then
    Result := Summary
  else
    Result := 'Attendee '+ IntToStr(Index);
end;

function TGAttendee.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

procedure TGAttendee.SetEmail(const Value: string);
begin
  FEmail := Value;
end;

procedure TGAttendee.SetStatus(const Value: TResponseStatus);
begin
  FStatus := Value;
end;

procedure TGAttendee.SetSummary(const Value: string);
begin
  FSummary := Value;
end;

{ TGAttendees }

function TGAttendees.Add: TGAttendee;
begin
  Result := TGAttendee(inherited Add);
end;

constructor TGAttendees.Create(AOwner: TGCalendarItem);
begin
  inherited Create(TGAttendee);
  FOwner := AOwner;
end;

function TGAttendees.GetItems(Index: Integer): TGAttendee;
begin
  Result := TGAttendee(inherited Items[Index]);
end;

function TGAttendees.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TGAttendees.Insert(Index: Integer): TGAttendee;
begin
  Result := TGAttendee(inherited Insert(Index));
end;

procedure TGAttendees.SetItems(Index: Integer; const Value: TGAttendee);
begin
  inherited Items[Index] := Value;
end;

procedure TGAttendees.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TGReminder }

procedure TGReminder.Assign(Source: TPersistent);
begin
  if (Source is TGReminder) then
  begin
    FMinutes := (Source as TGReminder).Minutes;
    FMethod := (Source as TGReminder).Method;
  end;
end;

constructor TGReminder.Create(ACollection: TCollection);
begin
  inherited;
  FMinutes := 10;
  FMethod := rmPopup;

  if ACollection is TGReminders then
  begin
    FOwner := ACollection as TGReminders;
  end;
end;

destructor TGReminder.Destroy;
begin
  inherited;
end;

function TGReminder.GetDisplayName: string;
begin
  Result := 'Reminder '+ IntToStr(Index);
end;

function TGReminder.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

procedure TGReminder.SetMethod(const Value: TReminderMethod);
begin
  FMethod := Value;
end;

procedure TGReminder.SetMinutes(const Value: integer);
begin
  FMinutes := Value;
end;

{ TGReminders }

function TGReminders.Add: TGReminder;
begin
  Result := TGReminder(inherited Add);
end;

constructor TGReminders.Create(AOwner: TGCalendarItem);
begin
  inherited Create(TGReminder);
  FOwner := AOwner;
end;

function TGReminders.GetItems(Index: Integer): TGReminder;
begin
  Result := TGReminder(inherited Items[Index]);
end;

function TGReminders.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TGReminders.Insert(Index: Integer): TGReminder;
begin
  Result := TGReminder(inherited Insert(Index));
end;

procedure TGReminders.SetItems(Index: Integer; const Value: TGReminder);
begin
  inherited Items[Index] := Value;
end;

procedure TGReminders.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TGDefaultReminders }

function TGDefaultReminders.Add: TGReminder;
begin
  Result := TGReminder(inherited Add);
end;

constructor TGDefaultReminders.Create(AOwner: TGCalendar);
begin
  inherited Create(TGReminder);
  FOwner := AOwner;
end;

function TGDefaultReminders.GetItems(Index: Integer): TGReminder;
begin
  Result := TGReminder(inherited Items[Index]);
end;

function TGDefaultReminders.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TGDefaultReminders.Insert(Index: Integer): TGReminder;
begin
  Result := TGReminder(inherited Insert(Index));
end;

procedure TGDefaultReminders.SetItems(Index: Integer; const Value: TGReminder);
begin
  inherited Items[Index] := Value;
end;

procedure TGDefaultReminders.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TGColor }

procedure TGColor.Assign(Source: TPersistent);
begin
  if (Source is TGColor) then
  begin
    FID := (Source as TGColor).ID;
    FBackgroundColor := (Source as TGColor).BackgroundColor;
    FForegroundColor := (Source as TGColor).ForegroundColor;
  end;
end;

constructor TGColor.Create(ACollection: TCollection);
begin
  inherited;
  FID := -1;
  FBackgroundColor := clNone;
  FForeGroundColor := clNone;

  if ACollection is TGColors then
  begin
    FOwner := ACollection as TGColors;
  end;
end;

destructor TGColor.Destroy;
begin

  inherited;
end;

function TGColor.GetDisplayName: string;
begin
  Result := 'Color '+ IntToStr(ID)
end;

function TGColor.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

procedure TGColor.SetBackGroundColor(const Value: TColor);
begin
  FBackgroundColor := Value;
end;

procedure TGColor.SetForeGroundColor(const Value: TColor);
begin
  FForegroundColor := Value;
end;

procedure TGColor.SetID(const Value: integer);
begin
  FID := Value;
end;

{ TGColors }

function TGColors.Add: TGColor;
begin
  Result := TGColor(inherited Add);
end;

constructor TGColors.Create(AOwner: TWebGoogleCalendar);
begin
  inherited Create(TGColor);
  FOwner := AOwner;
end;

function TGColors.GetItems(Index: Integer): TGColor;
begin
  Result := TGColor(inherited Items[Index]);
end;

function TGColors.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TGColors.Insert(Index: Integer): TGColor;
begin
  Result := TGColor(inherited Insert(Index));
end;

procedure TGColors.SetItems(Index: Integer; const Value: TGColor);
begin
  inherited Items[Index] := Value;
end;

procedure TGColors.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TGCalendarUser }

procedure TGCalendarUser.Assign(Source: TPersistent);
begin
  if (Source is TGCalendarUser) then
  begin
    FEmail := (Source as TGCalendarUser).Email;
    FDisplayName := (Source as TGCalendarUser).DisplayName;
  end;
end;

constructor TGCalendarUser.Create;
begin
  inherited;
  FDisplayName := '';
  FEmail := '';
end;

end.
