{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}
unit WEBLib.GoogleChart;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, WinApi.Windows, System.SysUtils, Vcl.Controls, Vcl.Graphics,
  Types, WEBLib.Controls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Types, LCLType, LCLIntf, Controls, Graphics, WEBLib.Controls;
  {$ENDIF}

type
  TGoogleChartType = (gctArea, gctBar, gctBubble, gctBubbleColor, gctCandlestick,
    gctColumn, gctLine, gctPie, gctScatter, gctTimeLine);
  TGoogleChartEasing = (gceLinear, gceIn, gceOut, gceInAndOut);
  TGoogleChartStacked = (gcsDisabled, gcsAbsolute, gcsPercent, gcsRelative);
  TGoogleChartPosition = (gcpBottom, gcpLeft, gcpIn, gcpNone, gcpRight, gcpTop);
  TGoogleChartAlignment = (gcaAuto, gcaStart, gcaCenter, gcaEnd);
  TGoogleChartTooltip = (gcttHover, gcttNone, gcttSelect);
  TGoogleChartPieText = (gcptPercentage, gcptValue, gcptLabel, gcptNone);
  TGoogleChartPointShape = (gcpsCircle, gcpsTriangle, gcpsSquare, gcpsDiamond,
    gcpsStar, gcpsPolygon);
  TGoogleChartCurveType = (gcctNone, gcctFunction);

  TGoogleChartDataPoint = record
    Title: string;
    X: Double;
    Y: Double;
    Minimum: Double;
    Maximum: Double;
    StartTime: TDateTime;
    EndTime: TDateTime;
    Series: string;
    Size: Double;
    Value: Double;
    Country: string;
    Offset: Double;
    Color: TColor;
  end;

  TGoogleChartAnimationRecord = record
    Duration: string;
    Easing: string;
    Startup: string;
  end;

  TGoogleChartAnnotations = record
    AlwaysOutside: string;
    BoxStyle: string;
    Style: string;
    TextStyle: string;
  end;

  TGoogleChartColor = record
    Stroke: string;
    StrokeWidth: string;
    Fill: string;
  end;

  TGoogleChartViewWindow = record
    Max: string;
    Min: string;
  end;

  TGoogleChartAxisRecord = record
    Baseline: string;
    BaselineColor: string;
    Direction: string;
    Format: string;
    GridLines: string;
    MinorGridLines: string;
    TextPosition: string;
    TextStyle: string;
    Ticks: string;
    Title: string;
    TitleTextStyle: string;
    SlantedText: string;
    SlantedTextAngle: string;
    MaxValue: string;
    MinValue: string;
    ViewWindowMode: string;
    ViewWindow: TGoogleChartViewWindow;
  end;

  TGoogleChartLegendRecord = record
    Position: string;
    Alignment: string;
    TextStyle: string;
  end;

  TGoogleChartTooltipRecord = record
    IgnoreBounds: string;
    ShowColorCode: string;
    TextStyle: string;
    Trigger: string;
  end;

  TGoogleChartBubble = record
    Opacity: string;
    Stroke: string;
    TextStyle: string;
  end;

  TGoogleChartColorAxisLegend = record
    Position: string;
    TextStyle: string;
    NumberFormat: string;
  end;

  TGoogleChartColorAxis = record
    MinValue: string;
    MaxValue: string;
    Values: string;
    Colors: string;
    Legend: TGoogleChartColorAxisLegend;
  end;

  TGoogleChartSizeAxis = record
    MaxSize: string;
    MaxValue: string;
    MinSize: string;
    MinValue: string;
  end;

  TGoogleChartCandleStick = record
    HollowIsRising: string;
    FallingColor: TGoogleChartColor;
    RisingColor: TGoogleChartColor;
  end;

  TGoogleChartOptions = record
    //General (not Gauge / Pie)
    Animation: TGoogleChartAnimationRecord;
    Annotations: TGoogleChartAnnotations;
    AxisTitlesPosition: string;
    BackgroundColor: TGoogleChartColor;
    Colors: string;
    DataOpacity: string;
    EnableInteractivity: string;
    FocusTarget: string;
    FontSize: string;
    FontName: string;
    HAxis: TGoogleChartAxisRecord;
    IsStacked: string;
    Legend: TGoogleChartLegendRecord;
    Orientation: string;
    ReverseCategories: string;
    Series: string;
    Title: string;
    TitlePosition: string;
    TitleTextStyle: string;
    Tooltip: TGoogleChartTooltipRecord;
    VAxis: TGoogleChartAxisRecord;
    //Column / Bar / Area / CandleStick / Line
    VAxes: string;
    //Column / Bar / Scatter
    Trendlines: string;
    //Area / Bubble / CandleStick / Line
    SelectionMode: string;
    //Area / CandleStick / Line / Pie / Scatter
    AggregationTarget: string;
    //Area
    AreaOpacity: string;
    InterpolateNulls: string;
    //Bubble
    Bubble: TGoogleChartBubble;
    ColorAxis: TGoogleChartColorAxis;
    SizeAxis: TGoogleChartSizeAxis;
    SortBubbleBySize: string;
    //CandleStick
    CandleStick: TGoogleChartCandleStick;
    //Gauge
    GreenColor: string;
    GreenFrom: string;
    GreenTo: string;
    MajorTicks: string;
    Max: string;
    Min: string;
    MinorTicks: string;
    RedColor: string;
    RedFrom: string;
    RedTo: string;
    YellowColor: string;
    YelloFrom: string;
    YellowTo: string;
    //Line
    CurveType: string;
    //Pie
    Is3D: string;
    PieHole: string;
    PieSliceBorderColor: string;
    PieSliceText: string;
    PieSliceTextStyle: string;
    PieStartAngle: string;
    PieResidueSliceColor: string;
    PieResidueSliceLabel: string;
    Slices: string;
    SliceVisibilityTreshold: string;
  end;

  TWebGoogleChart = class;
  TGoogleChartSeries = class;
  TGoogleChartSeriesItem = class;

  TGoogleChartSelectEventArgs = class(TPersistent)
  private
    FPointIndex: Integer;
    FSeriesIndex: Integer;
  published
    property PointIndex: Integer read FPointIndex write FPointIndex;
    property SeriesIndex: Integer read FSeriesIndex write FSeriesIndex;
  end;

  TGoogleChartSelectEvent = procedure(Sender: TObject; Event: TGoogleChartSelectEventArgs) of object;
  TGoogleChartCustomizeChartEvent = procedure(Sender: TObject; var Options: TGoogleChartOptions) of object;
  TGoogleChartCustomizeChartJSON = procedure(Sender: TObject; var Options: string) of object;

  TGoogleChartSeriesValueItem = class(TCollectionItem)
  private
    FDataPoint: TGoogleChartDataPoint;
  public
    property DataPoint: TGoogleChartDataPoint read FDataPoint write FDataPoint;
  end;

  TGoogleChartSeriesValues = class(TCollection)
  private
    FOwner: TGoogleChartSeries;
    function GetItems(Index: integer): TGoogleChartSeriesValueItem;
    procedure SetItems(Index: integer; const Value: TGoogleChartSeriesValueItem);
  public
    constructor Create(AOwner: TGoogleChartSeries); reintroduce;
    procedure AddSinglePoint(AValue: Double; ALabel: string = '');
    procedure AddPiePoint(AValue: Double; ALabel: string = ''; Offset: Double = 0);
    procedure AddXYPoint(X, Y: Double);
    procedure AddCandlestickPoint(X, Y, Minimum, Maximum: Double; ALabel: string = '');
    procedure AddTimelinePoint(StartTime, EndTime: TDateTime; ALabel: string = '');
    procedure AddBubblePoint(X, Y: Double; Series: string; Size: Double; ALabel: string = '');
    procedure AddBubbleColorPoint(X, Y: Double; Value: Double; ALabel: string = '');
    property Items[Index: integer]: TGoogleChartSeriesValueItem read GetItems write SetItems; default;
  end;

  TGoogleChartLineOptions = class(TPersistent)
  private
    FOwner: TGoogleChartSeriesItem;
    FPointSize: Integer;
    FPointShape: TGoogleChartPointShape;
    FLineWidth: Integer;
  public
    constructor Create(AOwner: TGoogleChartSeriesItem); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property PointSize: Integer read FPointSize write FPointSize default 7;
    property PointShape: TGoogleChartPointShape read FPointShape write FPointShape default gcpsCircle;
    property LineWidth: Integer read FLineWidth write FLineWidth default 2;
  end;

  TGoogleChartSeriesItem = class(TCollectionItem)
  private
    FOwner: TGoogleChartSeries;
    FChartType: TGoogleChartType;
    FTitle: string;
    FValues: TGoogleChartSeriesValues;
    FColor: TColor;
    FLine: TGoogleChartLineOptions;
    FTag: integer;
    procedure SetTitle(const Value: string);
    procedure SetColor(const Value: TColor);
  public
    constructor Create(ACollection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property Values: TGoogleChartSeriesValues read FValues;
  published
    property Title: string read FTitle write SetTitle;
    property ChartType: TGoogleChartType read FChartType write FChartType default gctBar;
    property Color: TColor read FColor write SetColor default clNone;
    property Line: TGoogleChartLineOptions read FLine write FLine;
    property Tag: integer read FTag write FTag default 0;
  end;

  TGoogleChartSeries = class(TCollection)
  private
    FOwner: TWebGoogleChart;
    function GetItems(Index: integer): TGoogleChartSeriesItem;
    procedure SetItems(Index: integer; const Value: TGoogleChartSeriesItem);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TWebGoogleChart); overload;
    function GetOwner: TPersistent; override;
    function Add: TGoogleChartSeriesItem; reintroduce;
    function Insert(Index: Integer):TGoogleChartSeriesItem; reintroduce;
    property Items[Index: integer]: TGoogleChartSeriesItem read GetItems write SetItems; default;
  end;

  TGoogleChartAppearance = class;

  TGoogleChartAxis = class(TPersistent)
  private
    FOwner: TGoogleChartAppearance;
    FMinValue: Integer;
    FAutoMaxMinValue: Boolean;
    FMaxValue: Integer;
  public
    constructor Create(AOwner: TGoogleChartAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property MaxValue: Integer read FMaxValue write FMaxValue default 0;
    property MinValue: Integer read FMinValue write FMinValue default 0;
    property AutoMaxMinValue: Boolean read FAutoMaxMinValue write FAutoMaxMinValue default True;
  end;

  TGoogleChartAnimation = class(TPersistent)
  private
    FOwner: TGoogleChartAppearance;
    FDuration: Integer;
    FEasing: TGoogleChartEasing;
    FStartup: Boolean;
  public
    constructor Create(AOwner: TGoogleChartAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Duration: Integer read FDuration write FDuration default 1000;
    property Easing: TGoogleChartEasing read FEasing write FEasing default gceOut;
    property Startup: Boolean read FStartup write FStartup default true;
  end;

  TGoogleChartBackground = class(TPersistent)
  private
    FOwner: TGoogleChartAppearance;
    FBorderColor: TColor;
    FColor: TColor;
    FBorderWidth: Integer;
  public
    constructor Create(AOwner: TGoogleChartAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Color: TColor read FColor write FColor default clNone;
    property BorderColor: TColor read FBorderColor write FBorderColor default clNone;
    property BorderWidth: Integer read FBorderWidth write FBorderWidth default 0;
  end;

  TGoogleChartLegend = class(TPersistent)
  private
    FOwner: TGoogleChartAppearance;
    FAlignment: TGoogleChartAlignment;
    FPosition: TGoogleChartPosition;
  public
    constructor Create(AOwner: TGoogleChartAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Position: TGoogleChartPosition read FPosition write FPosition default gcpRight;
    property Alignment: TGoogleChartAlignment read FAlignment write FAlignment default gcaAuto;
  end;

  TGoogleChartPie = class(TPersistent)
  private
    FOwner: TGoogleChartAppearance;
    FEnable3D: Boolean;
    FPieHole: Double;
    FPieSliceText: TGoogleChartPieText;
  public
    constructor Create(AOwner: TGoogleChartAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Enable3D: Boolean read FEnable3D write FEnable3D default false;
    property PieHole: Double read FPieHole write FPieHole;
    property PieSliceText: TGoogleChartPieText read FPieSliceText write FPieSliceText default gcptPercentage;
  end;

  TGoogleChartLine = class(TPersistent)
  private
    FOwner: TGoogleChartAppearance;
    FCurveType: TGoogleChartCurveType;
  public
    constructor Create(AOwner: TGoogleChartAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property CurveType: TGoogleChartCurveType read FCurveType write FCurveType default gcctNone;
  end;

  TGoogleChartAppearance = class(TPersistent)
  private
    FOwner: TWebGoogleChart;
    FStacked: TGoogleChartStacked;
    FAnimation: TGoogleChartAnimation;
    FBackground: TGoogleChartBackground;
    FLegend: TGoogleChartLegend;
    FReverseCategories: Boolean;
    FTooltip: TGoogleChartTooltip;
    FPieChart: TGoogleChartPie;
    FVAxis: TGoogleChartAxis;
    FHAxis: TGoogleChartAxis;
    FLineChart: TGoogleChartLine;
    procedure SetAnimation(const Value: TGoogleChartAnimation);
    procedure SetBackground(const Value: TGoogleChartBackground);
    procedure SetLegend(const Value: TGoogleChartLegend);
    procedure SetPieChart(const Value: TGoogleChartPie);
    procedure SetVAxis(const Value: TGoogleChartAxis);
    procedure SetHAxis(const Value: TGoogleChartAxis);
    procedure SetLineChart(const Value: TGoogleChartLine);
  public
    constructor Create(AOwner: TWebGoogleChart); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Animation: TGoogleChartAnimation read FAnimation write SetAnimation;
    property Background: TGoogleChartBackground read FBackground write SetBackground;
    property Legend: TGoogleChartLegend read FLegend write SetLegend;
    property ReverseCategories: Boolean read FReverseCategories write FReverseCategories default False;
    property Stacked: TGoogleChartStacked read FStacked write FStacked default gcsDisabled;
    property Tooltip: TGoogleChartTooltip read FTooltip write FTooltip default gcttHover;
    property PieChart: TGoogleChartPie read FPieChart write SetPieChart;
    property LineChart: TGoogleChartLine read FLineChart write SetLineChart;
    property VAxis: TGoogleChartAxis read FVAxis write SetVAxis;
    property HAxis: TGoogleChartAxis read FHAxis write SetHAxis;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  [JSLibReference('https://www.gstatic.com/charts/loader.js')]
  {$ENDIF}
  TWebGoogleChart = class(TWebCustomControl)
  private
    FSeries: TGoogleChartSeries;
    FTitle: string;
    FOnLoad: TNotifyEvent;
    FOnSelect: TGoogleChartSelectEvent;
    FOnCustomizeChartJSON: TGoogleChartCustomizeChartJSON;
    FOnCustomizeChart: TGoogleChartCustomizeChartEvent;
    FAppearance: TGoogleChartAppearance;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Font;
    property Height;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property Top;
    property Visible;
    property Width;
    property Appearance: TGoogleChartAppearance read FAppearance write FAppearance;
    property Series: TGoogleChartSeries read FSeries write FSeries;
    property Title: string read FTitle write FTitle;

    property OnLoaded: TNotifyEvent read FOnLoad write FOnLoad;
    property OnSelect: TGoogleChartSelectEvent read FOnSelect write FOnSelect;
    property OnCustomizeChart: TGoogleChartCustomizeChartEvent read FOnCustomizeChart write FOnCustomizeChart;
    property OnCustomizeChartJSON: TGoogleChartCustomizeChartJSON read FOnCustomizeChartJSON write FOnCustomizeChartJSON;
  end;


implementation

{ TGoogleChartAppearance }

procedure TGoogleChartAppearance.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartAppearance) then
  begin
    FStacked := (Source as TGoogleChartAppearance).Stacked;
    FAnimation.Assign((Source as TGoogleChartAppearance).Animation);
    FBackground.Assign((Source as TGoogleChartAppearance).Background);
    FLegend.Assign((Source as TGoogleChartAppearance).Legend);
    FReverseCategories := (Source as TGoogleChartAppearance).ReverseCategories;
    FTooltip := (Source as TGoogleChartAppearance).Tooltip;
    FPieChart.Assign((Source as TGoogleChartAppearance).PieChart);
    FVAxis.Assign((Source as TGoogleChartAxis));
    FHAxis.Assign((Source as TGoogleChartAxis));
  end;
end;

constructor TGoogleChartAppearance.Create(AOwner: TWebGoogleChart);
begin
  inherited Create;
  FStacked := gcsDisabled;
  FAnimation := TGoogleChartAnimation.Create(Self);
  FBackground := TGoogleChartBackground.Create(Self);
  FLegend := TGoogleChartLegend.Create(Self);
  FReverseCategories := False;
  FToolTip := gcttHover;
  FPieChart := TGoogleChartPie.Create(Self);
  FLineChart := TGoogleChartLine.Create(Self);
  FVAxis := TGoogleChartAxis.Create(Self);
  FHAxis := TGoogleChartAxis.Create(Self);
  FOwner := AOwner;
end;

destructor TGoogleChartAppearance.Destroy;
begin
  FAnimation.Free;
  FBackground.Free;
  FLegend.Free;
  FPieChart.Free;
  FLineChart.Free;
  FVAxis.Free;
  FHAxis.Free;
  inherited;
end;

function TGoogleChartAppearance.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

procedure TGoogleChartAppearance.SetAnimation(const Value: TGoogleChartAnimation);
begin
  FAnimation.Assign(Value);
end;

procedure TGoogleChartAppearance.SetBackground(
  const Value: TGoogleChartBackground);
begin
  FBackground.Assign(Value);
end;

procedure TGoogleChartAppearance.SetHAxis(const Value: TGoogleChartAxis);
begin
  FHAxis.Assign(Value);
end;

procedure TGoogleChartAppearance.SetLegend(const Value: TGoogleChartLegend);
begin
  FLegend.Assign(Value);
end;

procedure TGoogleChartAppearance.SetLineChart(const Value: TGoogleChartLine);
begin
  FLineChart.Assign(Value);
end;

procedure TGoogleChartAppearance.SetPieChart(const Value: TGoogleChartPie);
begin
  FPieChart.Assign(Value);
end;

procedure TGoogleChartAppearance.SetVAxis(const Value: TGoogleChartAxis);
begin
  FVAxis.Assign(Value);
end;

{ TGoogleChartAnimation }

procedure TGoogleChartAnimation.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartAnimation) then
  begin
    FDuration := (Source as TGoogleChartAnimation).Duration;
  end;
end;

constructor TGoogleChartAnimation.Create(AOwner: TGoogleChartAppearance);
begin
  inherited Create;
  FDuration := 1000;
  FEasing := gceOut;
  FStartup := True;
  FOwner := AOwner;
end;

destructor TGoogleChartAnimation.Destroy;
begin
  inherited;
end;

function TGoogleChartAnimation.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TGoogleChartBackground }

procedure TGoogleChartBackground.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartBackground) then
  begin
    Color := (Source as TGoogleChartBackground).Color;
    BorderColor := (Source as TGoogleChartBackground).BorderColor;
    BorderWidth := (Source as TGoogleChartBackground).BorderWidth;
  end;
end;

constructor TGoogleChartBackground.Create(AOwner: TGoogleChartAppearance);
begin
  inherited Create;
  FColor := clNone;
  FBorderColor := clNone;
  FBorderWidth := 0;
  FOwner := AOwner;
end;

destructor TGoogleChartBackground.Destroy;
begin
  inherited;
end;

function TGoogleChartBackground.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TGoogleChartLegend }

procedure TGoogleChartLegend.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartLegend) then
  begin
    FPosition := (Source as TGoogleChartLegend).Position;
    FAlignment := (Source as TGoogleChartLegend).Alignment;
  end;
end;

constructor TGoogleChartLegend.Create(AOwner: TGoogleChartAppearance);
begin
  inherited Create;
  FPosition := gcpRight;
  FAlignment := gcaAuto;
  FOwner := AOwner;
end;

destructor TGoogleChartLegend.Destroy;
begin
  inherited;
end;

function TGoogleChartLegend.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TGoogleChartPie }

procedure TGoogleChartPie.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartPie) then
  begin
    FEnable3D := (Source as TGoogleChartPie).Enable3D;
    FPieHole := (Source as TGoogleChartPie).PieHole;
  end;
end;

constructor TGoogleChartPie.Create(AOwner: TGoogleChartAppearance);
begin
  inherited Create;
  FEnable3D := False;
  FPieHole := 0;
  FPieSliceText := gcptPercentage;
  FOwner := AOwner;
end;

destructor TGoogleChartPie.Destroy;
begin
  inherited;
end;

function TGoogleChartPie.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TGoogleChartLine }

procedure TGoogleChartLine.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartLine) then
  begin
    CurveType := (Source as TGoogleChartLine).FCurveType;
  end;
end;

constructor TGoogleChartLine.Create(AOwner: TGoogleChartAppearance);
begin
  FCurveType := gcctNone;
  FOwner := AOwner;
end;

destructor TGoogleChartLine.Destroy;
begin
  inherited;
end;

function TGoogleChartLine.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TGoogleChartAxis }

procedure TGoogleChartAxis.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartAxis) then
  begin
    FMaxValue := (Source as TGoogleChartAxis).MaxValue;
    FMinValue := (Source as TGoogleChartAxis).MinValue;
    FAutoMaxMinValue := (Source as TGoogleChartAxis).AutoMaxMinValue;
  end;
end;

constructor TGoogleChartAxis.Create(AOwner: TGoogleChartAppearance);
begin
  inherited Create;
  FMaxValue := 0;
  FMinValue := 0;
  FAutoMaxMinValue := True;
  FOwner := AOwner;
end;

destructor TGoogleChartAxis.Destroy;
begin
  inherited;
end;

function TGoogleChartAxis.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TGoogleChartLineOptions }

procedure TGoogleChartLineOptions.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TGoogleChartLineOptions) then
  begin
    FPointSize := (Source as TGoogleChartLineOptions).PointSize;
    FPointShape := (Source as TGoogleChartLineOptions).PointShape;
    FLineWidth := (Source as TGoogleChartLineOptions).LineWidth;
  end;
end;

constructor TGoogleChartLineOptions.Create(AOwner: TGoogleChartSeriesItem);
begin
  inherited Create;
  FPointSize := 7;
  FPointShape := gcpsCircle;
  FLineWidth := 2;
  FOwner := AOwner;
end;

destructor TGoogleChartLineOptions.Destroy;
begin
  inherited;
end;

function TGoogleChartLineOptions.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TGoogleChartSeriesValues }

procedure TGoogleChartSeriesValues.AddBubbleColorPoint(X, Y, Value: Double;
  ALabel: string);
begin
end;

procedure TGoogleChartSeriesValues.AddBubblePoint(X, Y: Double; Series: string;
  Size: Double; ALabel: string);
begin
end;

procedure TGoogleChartSeriesValues.AddCandlestickPoint(X, Y, Minimum, Maximum: Double; ALabel: string);
begin
end;

procedure TGoogleChartSeriesValues.AddPiePoint(AValue: Double; ALabel: string;
  Offset: Double);
begin
end;

procedure TGoogleChartSeriesValues.AddSinglePoint(AValue: Double;
  ALabel: string);
begin
end;

procedure TGoogleChartSeriesValues.AddTimelinePoint(StartTime,
  EndTime: TDateTime; ALabel: string);
begin
end;

procedure TGoogleChartSeriesValues.AddXYPoint(X, Y: Double);
begin
end;

constructor TGoogleChartSeriesValues.Create(AOwner: TGoogleChartSeries);
begin
  inherited Create(TGoogleChartSeriesValueItem);

  if AOwner is TGoogleChartSeries then
  begin
    FOwner := AOwner as TGoogleChartSeries;
  end;
end;

function TGoogleChartSeriesValues.GetItems(
  Index: integer): TGoogleChartSeriesValueItem;
begin
  Result := TGoogleChartSeriesValueItem(inherited Items[Index]);
end;

procedure TGoogleChartSeriesValues.SetItems(Index: integer;
  const Value: TGoogleChartSeriesValueItem);
begin
  inherited Items[Index] := Value;
end;

{ TGoogleChartSeries }

function TGoogleChartSeries.Add: TGoogleChartSeriesItem;
begin
  Result := TGoogleChartSeriesItem(inherited Add);
end;

constructor TGoogleChartSeries.Create(AOwner: TWebGoogleChart);
begin
  inherited Create(TGoogleChartSeriesItem);
  FOwner := AOwner;
end;

function TGoogleChartSeries.GetItems(Index: integer): TGoogleChartSeriesItem;
begin
  Result := TGoogleChartSeriesItem(inherited Items[Index]);
end;

function TGoogleChartSeries.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TGoogleChartSeries.Insert(Index: Integer): TGoogleChartSeriesItem;
begin
  Result := TGoogleChartSeriesItem(inherited Insert(Index));
end;

procedure TGoogleChartSeries.SetItems(Index: integer;
  const Value: TGoogleChartSeriesItem);
begin
  inherited Items[Index] := Value;
end;

procedure TGoogleChartSeries.Update(Item: TCollectionItem);
begin
  inherited;
  FOwner.Invalidate;
end;

{ TWebGoogleChart }

constructor TWebGoogleChart.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
begin
  inherited;
  FAppearance := TGoogleChartAppearance.Create(Self);
  FTitle := '';
  FSeries := TGoogleChartSeries.Create(Self);
  Width := 300;
  Height := 300;

  FDesignTime := (csDesigning in ComponentState) and not
    ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  if FDesignTime then
  begin
    Appearance.Animation.Startup := false;
    FSeries.Clear;
    FSeries.Add;
    FSeries.Items[0].ChartType := gctLine;
    FSeries.Items[0].Color := clRed;
    FSeries.Items[0].Tag := -1;
  end;

end;

destructor TWebGoogleChart.Destroy;
begin
  FAppearance.Free;
  FSeries.Free;
  inherited;
end;

procedure TWebGoogleChart.Paint;
var
  i: integer;
  y: integer;
  s: string;
begin
  inherited;

  y := 10;

  for i := 0 to Series.Count - 1 do
  begin
    s := 'Series ' + IntToStr(i+1)+ ':';
    case Series[i].ChartType of
    gctArea: s := s + 'Area';
    gctLine: s := s+ 'Line';
    gctBar: s := s + 'Bar';
    gctBubble: s := s + 'Bubble';
    gctBubbleColor: s := s + 'BubbleColor';
    gctCandlestick: s := s + 'Candlestick';
    gctColumn: s := s + 'Column';
    gctPie: s := s + 'Pie';
    gctScatter: s := s + 'Scatter';
    gctTimeLine: s:= s + 'Timeline';
    end;

    s := s + ':' + Title;

    Canvas.Brush.Style := bsClear;
    Canvas.Font.Name := 'Arial';
    Canvas.Font.Size := 10;
    Canvas.Font.Color := Series[i].Color;
    Canvas.TextOut(10,y,s);

    y := y + 20;
  end;

  Canvas.Font.Color := clWindowText;
  Canvas.Brush.Style := bsSolid;
end;

{ TGoogleChartSeriesItem }

procedure TGoogleChartSeriesItem.Assign(Source: TPersistent);
begin
  if (Source is TGoogleChartSeriesItem) then
  begin
    FTitle := (Source as TGoogleChartSeriesItem).Title;
    FChartType := (Source as TGoogleChartSeriesItem).ChartType;
    FColor := (Source as TGoogleChartSeriesItem).Color;
    FLine := (Source as TGoogleChartSeriesItem).Line;
    FTag := (Source as TGoogleChartSeriesItem).Tag;
  end;
end;

constructor TGoogleChartSeriesItem.Create(ACollection: TCollection);
begin
  inherited;
  FChartType := gctBar;
  FTitle := '';
  FColor := clNone;

  if ACollection is TGoogleChartSeries then
  begin
    FOwner := ACollection as TGoogleChartSeries;
    FValues := TGoogleChartSeriesValues.Create(FOwner);
    FLine := TGoogleChartLineOptions.Create(Self);
  end;
end;

destructor TGoogleChartSeriesItem.Destroy;
begin
  FValues.Free;
  FLine.Free;
  inherited;
end;

procedure TGoogleChartSeriesItem.SetColor(const Value: TColor);
begin
  if (FColor <> Value) then
  begin
    FColor := Value;
    (Collection as TGoogleChartSeries).Update(Self);
  end;
end;

procedure TGoogleChartSeriesItem.SetTitle(const Value: string);
begin
  if FTitle <> Value then
  begin
    FTitle := Value;
    (Collection as TGoogleChartSeries).Update(Self);
  end;
end;

end.
