{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.JQCtrls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, RTTI, VCL.Controls, VCL.Forms, VCL.StdCtrls, System.SysUtils, VCL.Graphics, Types,
  Vcl.Themes, Vcl.Menus, WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Menus, WEBLib.ComCtrls, js;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Types, LCLType, LCLIntf, Controls, Forms, StdCtrls, SysUtils, Graphics,
  Menus, WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Menus, WEBLib.ComCtrls, js;
  {$ENDIF}

type
  TJQXDayOfWeek = (dowMonday, dowTuesday, dowWednesday, dowThursday,
    dowFriday, dowSaturday, dowSunday, dowLocaleDefault);
  TJQXMenuMode = (mmHorizontal, mmVertical, mmPopup);
  TJQXColorMode = (cmHue, cmSaturation);
  TJQXSymbolPosition = (spLeft, spRight);
  TJQXInputMode = (imAdvanced, imSimple);
//  TJQXRatingPrecision = (rpFull, rpHalf);
  TJQXProgressOrientation = (poHorizontal, poVertical);
  TJQXProgressLayout = (plNormal, plReverse);
  TJQXTemplate = (ttDefault, ttPrimary, ttSuccess, ttWarning, ttDanger, ttInfo);
  TJQXHorizontalPosition = (hpLeft, hpRight, hpBoth);
  TJQXVerticalPosition = (vpTop, vpBottom, vpBoth);
  TJQXSliderMode = (smDefault, smFixed);
  TJQXKnobMarkType = (mtCircle, mtLine);
  TJQXKnobPointerType = (ptArrow, ptCircle, ptLine);
  TJQXPosition = (xpLeft, xpTop, xpCenter, xpBottom, xpRight, xpTopLeft,
    xpBottomLeft, xpTopRight, xpBottomRight);
  TJQXImagePosition = (ipAfterImage, ipAboveImage, ipBelowImage, ipBeforeImage, ipOverlay);
  TJQXGroupMode = (gmDefault, gmCheckBox, gmRadioButton);
  TJQXTagCloudSort = (tcsNone, tcsLabel, tcsValue);
  TJQXSortOrder = (soAscending, soDescending);
  TJQXTextCase = (tcOriginal, tcUpperCase, tcLowerCase, tcFirstUpper, tcCamelCase);
  TJQXAnimationType = (atNone, atFade, atSlide);
  TJQXTabPosition = (tpTop, tpBottom);
  TJQXToggleMode = (tmClick, tmDoubleClick, tmMouseEnter);

  TJQXCalendarEventArgs = class(TPersistent)
  private
    FAllow: boolean;
    FDate: TDateTime;
  published
    property Allow: boolean read FAllow write FAllow default true;
    property Date: TDateTime read FDate write FDate;
  end;

  TJQXMenuEventArgs = class(TPersistent)
  private
    FSource: TMenuItem;
  published
    property Source: TMenuItem read FSource write FSource;
  end;

  TJQXSliderEventArgs = class(TPersistent)
  private
    FValue: Double;
    FMaximumValue: Double;
    FMinimumValue: Double;
  published
    property Value: Double read FValue write FValue;
    property MinimumValue: Double read FMinimumValue write FMinimumValue;
    property MaximumValue: Double read FMaximumValue write FMaximumValue;
  end;

  TJQXButtonEventArgs = class(TPersistent)
  private
    FButtonIndex: Integer;
  published
    property ButtonIndex: Integer read FButtonIndex write FButtonIndex;
  end;

  TJQXTagEventArgs = class(TPersistent)
  private
    FTagIndex: Integer;
    FTagValue: Integer;
    FDisplayIndex: Integer;
    FTagLabel: String;
  published
    property TagIndex: Integer read FTagIndex write FTagIndex;
    property TagLabel: String read FTagLabel write FTagLabel;
    property TagValue: Integer read FTagValue write FTagValue;
    property DisplayIndex: Integer read FDisplayIndex write FDisplayIndex;
  end;

  TJQXTabEventArgs = class(TPersistent)
  private
    FTabIndex: Integer;
  published
    property TabIndex: Integer read FTabIndex write FTabIndex;
  end;

  TJQXTabEvent = procedure(Sender: TObject; Event: TJQXTabEventArgs) of object;
  TJQXTagEvent = procedure(Sender: TObject; Event: TJQXTagEventArgs) of object;
  TDateEvent = procedure(Sender: TObject; Event: TJQXCalendarEventArgs) of object;
  TItemEvent = procedure(Sender: TObject; Event: TJQXMenuEventArgs) of object;
  TJQXSliderEvent = procedure(Sender: TObject; Event: TJQXSliderEventArgs) of object;
  TJQXButtonEvent = procedure(Sender: TObject; Event: TJQXButtonEventArgs) of object;

  TWebJQXCustomControl = class(TWebCustomControl)
  private
    FTheme: string;
  public
    property Theme: string read FTheme write FTheme;
  published
    property Align;
    property Anchors;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property ElementPosition;
    property Visible;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXCalendar = class(TWebJQXCustomControl)
  private
    FFirstDayOfWeek: TJQXDayOfWeek;
    FDate: TDateTime;
    FEndDate: TDateTime;
    FMaxDate: TDateTime;
    FMinDate: TDateTime;
    FMultiSelect: boolean;
    FOtherMonthDays: boolean;
    FShowToday: boolean;
    FWeekNumbers: boolean;
    FOnDateClick: TDateEvent;
    FOnNavigateClick: TDateEvent;
    procedure SetDate(const Value: TDateTime);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled;
    property FirstDayOfWeek: TJQXDayOfWeek read FFirstDayOfWeek write FFirstDayOfWeek;
    property Date: TDateTime read FDate write SetDate;
    property EndDate: TDateTime read FEndDate write FEndDate;
    property MaxDate: TDateTime read FMaxDate write FMaxDate;
    property MinDate: TDateTime read FMinDate write FMinDate;
    property MultiSelect: boolean read FMultiSelect write FMultiSelect default false;
    property OtherMonthDays: boolean read FOtherMonthDays write FOtherMonthDays default true;
    property ShowToday: boolean read FShowToday write FShowToday default true;
    property WeekNumbers: boolean read FWeekNumbers write FWeekNumbers default false;
    property Theme;
    property OnDateClick: TDateEvent read FOnDateClick write FOnDateClick;
    property OnNavigateClick: TDateEvent read FOnNavigateClick write FOnNavigateClick;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXDateTimeInput = class(TWebJQXCustomControl)
  private
    FFirstDayOfWeek: TJQXDayOfWeek;
    FDate: TDateTime;
    FEndDate: TDateTime;
    FMaxDate: TDateTime;
    FMinDate: TDateTime;
    FMultiSelect: boolean;
    FShowToday: boolean;
    FWeekNumbers: boolean;
    FOnDateClick: TDateEvent;
    procedure SetDate(const Value: TDateTime);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled;
    property FirstDayOfWeek: TJQXDayOfWeek read FFirstDayOfWeek write FFirstDayOfWeek;
    property Date: TDateTime read FDate write SetDate;
    property EndDate: TDateTime read FEndDate write FEndDate;
    property MaxDate: TDateTime read FMaxDate write FMaxDate;
    property MinDate: TDateTime read FMinDate write FMinDate;
    property MultiSelect: boolean read FMultiSelect write FMultiSelect default false;
    property ShowToday: boolean read FShowToday write FShowToday default true;
    property WeekNumbers: boolean read FWeekNumbers write FWeekNumbers default false;
    property Theme;
    property OnDateClick: TDateEvent read FOnDateClick write FOnDateClick;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXMenu = class(TWebJQXCustomControl)
  private
    FMode: TJQXMenuMode;
    FMenu: TWebMainMenu;
    FOnItemClick: TItemEvent;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetItemEnabled(AMenuItem: TMenuItem);
    procedure SetItemDisabled(AMenuItem: TMenuItem);
  published
    property Enabled;
    property Mode: TJQXMenuMode read FMode write FMode default mmHorizontal;
    property Menu: TWebMainMenu read FMenu write FMenu;
    property Theme;
    property OnItemClick: TItemEvent read FOnItemClick write FOnItemClick;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXMaskedInput = class(TWebJQXCustomControl)
  private
    FMask: string;
    FText: string;
    FOnChange: TNotifyEvent;
    procedure SetMask(const Value: string);
  public
    constructor Create(AOwner: TComponent); override;
  protected
    procedure Paint; override;
  published
    property Enabled;
    property Mask: string read FMask write SetMask;
    property Text: string read FText write FText;
    property Theme;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXColorPicker = class(TWebJQXCustomControl)
  private
    FColor: TColor;
    FColorMode: TJQXColorMode;
    FOnChange: TNotifyEvent;
    procedure SetColor(const Value: TColor);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled;
    property Color: TColor read FColor write SetColor default clBlack;
    property ColorMode: TJQXColorMode read FColorMode write FColorMode default cmSaturation;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXComboBox = class(TWebJQXCustomControl)
  private
    FAutoComplete: Boolean;
    FItemIndex: integer;
    FItems: TStringList;
    FMultiSelect: Boolean;
    FTextHint: string;
    FOnChange: TNotifyEvent;
    procedure SetItems(const Value: TStringList);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function GetDisabled(AIndex: integer): boolean;
    procedure SetDisabled(AIndex: Integer; AValue: boolean);
    function GetSelected(AIndex: integer): boolean;
    procedure SetSelected(AIndex: Integer; AValue: boolean);

    property Disabled[i: Integer]: Boolean read GetDisabled write SetDisabled;
    property Selected[i: Integer]: Boolean read GetSelected write SetSelected;
  published
    property AutoComplete: Boolean read FAutoComplete write FAutoComplete default false;
    property Enabled;
    property ItemIndex: integer read FItemIndex write FItemIndex default -1;
    property Items: TStringList read FItems write SetItems;
    property MultiSelect: Boolean read FMultiSelect write FMultiSelect;
    property TextHint: string read FTextHint write FTextHint;
    property Theme;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXDropDownList = class(TWebJQXCustomControl)
  private
    FItemIndex: integer;
    FItems: TStringList;
    FMultiSelect: Boolean;
    FTextHint: string;
    FOnChange: TNotifyEvent;
    procedure SetItems(const Value: TStringList);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function GetDisabled(AIndex: integer): boolean;
    procedure SetDisabled(AIndex: Integer; AValue: boolean);
    function GetSelected(AIndex: integer): boolean;
    procedure SetSelected(AIndex: Integer; AValue: boolean);

    property Disabled[i: Integer]: Boolean read GetDisabled write SetDisabled;
    property Selected[i: Integer]: Boolean read GetSelected write SetSelected;
  published
    property Enabled;
    property ItemIndex: integer read FItemIndex write FItemIndex;
    property Items: TStringList read FItems write SetItems;
    property MultiSelect: Boolean read FMultiSelect write FMultiSelect;
    property TextHint: string read FTextHint write FTextHint;
    property Theme;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXNumberInput = class(TWebJQXCustomControl)
  private
    FDigits: Integer;
    FDecimalDigits: Integer;
    FMaxValue: Double;
    FMinValue: Double;
    FInputMode: TJQXInputMode;
    FShowSpinButtons: Boolean;
    FSpinButtonsStep: Integer;
    FSymbol: string;
    FSymbolPosition: TJQXSymbolPosition;
    FValue: Double;
    FOnChange: TNotifyEvent;
    procedure SetValue(const Value: Double);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled;
    property Digits: Integer read FDigits write FDigits;
    property DecimalDigits: Integer read FDecimalDigits write FDecimalDigits;
    property InputMode: TJQXInputMode read FInputMode write FInputMode;
    property MaxValue: Double read FMaxValue write FMaxValue;
    property MinValue: Double read FMinValue write FMinValue;
    property ShowSpinButtons: Boolean read FShowSpinButtons write FShowSpinButtons;
    property SpinButtonsStep: Integer read FSpinButtonsStep write FSpinButtonsStep;
    property Symbol: string read FSymbol write FSymbol;
    property SymbolPosition: TJQXSymbolPosition read FSymbolPosition write FSymbolPosition;
    property Value: Double read FValue write SetValue;
    property Theme;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXRating = class(TWebJQXCustomControl)
  private
    FItemCount: Integer;
//    FPrecision: TJQXRatingPrecision;
    FValue: Double;
    FOnChange: TNotifyEvent;
    procedure SetValue(const Value: Double);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled;
    property ItemCount: Integer read FItemCount write FItemCount;
//    property Precision: TJQXRatingPrecision read FPrecision write FPrecision;
    property Value: Double read FValue write SetValue;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXButton = class(TWebJQXCustomControl)
  private
    FOnClick: TNotifyEvent;
    FCaption: string;
    FRoundedBorders: Boolean;
    FImageURL: string;
    FImageHeight: Integer;
    FImagePosition: TJQXPosition;
    FCaptionPosition: TJQXPosition;
    FImageWidth: Integer;
    FCaptionImageRelation: TJQXImagePosition;
    FTemplate: TJQXTemplate;
    procedure SetCaption(const Value: string);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Caption: string read FCaption write SetCaption;
    property CaptionPosition: TJQXPosition read FCaptionPosition write FCaptionPosition default xpLeft;
    property CaptionImageRelation: TJQXImagePosition read FCaptionImageRelation write FCaptionImageRelation default ipAfterImage;
    property ImageURL: string read FImageURL write FImageURL;
    property ImagePosition: TJQXPosition read FImagePosition write FImagePosition default xpLeft;
    property ImageHeight: Integer read FImageHeight write FImageHeight default 16;
    property ImageWidth: Integer read FImageWidth write FImageWidth default 16;
    property Enabled;
    property RoundedBorders: Boolean read FRoundedBorders write FRoundedBorders default true;
    property Template: TJQXTemplate read FTemplate write FTemplate;
    property Theme;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXButtonGroup = class(TWebJQXCustomControl)
  private
    FTemplate: TJQXTemplate;
    FMode: TJQXGroupMode;
    FItems: TStringList;
    FEnableHover: Boolean;
    FOnClick: TJQXButtonEvent;
    procedure SetItems(const Value: TStringList);
    function GetButtonSelect(Button: integer): boolean;
    procedure SetButtonSelect(Button: integer; const Value: boolean);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ButtonSelect[Button: integer]: boolean read GetButtonSelect write SetButtonSelect;
  published
    property Enabled;
    property EnableHover: Boolean read FEnableHover write FEnableHover default True;
    property Items: TStringList read FItems write SetItems;
    property Mode: TJQXGroupMode read FMode write FMode default gmDefault;
    property Template: TJQXTemplate read FTemplate write FTemplate;
    property Theme;
    property OnClick: TJQXButtonEvent read FOnClick write FOnClick;
  end;

  TWebJQXKnob = class;
  TJQXKnobAppearance = class;

  TJQXKnobLabels = class(TPersistent)
  private
    FOwner: TJQXKnobAppearance;
    FVisible: Boolean;
    FStep: Integer;
    FOffset: Integer;
  public
    constructor Create(AOwner: TJQXKnobAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Offset: Integer read FOffset write FOffset default 88;
    property Step: Integer read FStep write FStep default 10;
    property Visible: Boolean read FVisible write FVisible default true;
  end;

  TJQXKnobProgressBar = class(TPersistent)
  private
    FOwner: TJQXKnobAppearance;
    FBorderColor: TColor;
    FSize: Integer;
    FBackgroundColor: TColor;
    FOffset: Integer;
    FColor: TColor;
  public
    constructor Create(AOwner: TJQXKnobAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    {$IFNDEF LCLTMSWEB}
    property Color: TColor read FColor write FColor default clWebSilver;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    property Color: TColor read FColor write FColor default clSilver;
    {$ENDIF}
    property Size: Integer read FSize write FSize default 9;
    property Offset: Integer read FOffset write FOffset default 60;
    {$IFNDEF LCLTMSWEB}
    property BackgroundColor: TColor read FBackgroundColor write FBackgroundColor default clWebGray;
    property BorderColor: TColor read FBorderColor write FBorderColor default clWebBlack;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    property BackgroundColor: TColor read FBackgroundColor write FBackgroundColor default clGray;
    property BorderColor: TColor read FBorderColor write FBorderColor default clBlack;
    {$ENDIF}
  end;

  TJQXKnobMarks = class(TPersistent)
  private
    FOwner: TJQXKnobAppearance;
    FOffset: Integer;
    FMajorSize: Integer;
    FColorProgress: TColor;
    FWidth: Integer;
    FBorderColorRemaining: TColor;
    FBorderColorProgress: TColor;
    FMinorInterval: Integer;
    FSize: Integer;
    FColorRemaining: TColor;
    FMajorInterval: Integer;
    FMarkType: TJQXKnobMarkType;
  public
    constructor Create(AOwner: TJQXKnobAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property ColorRemaining: TColor read FColorRemaining write FColorRemaining default clGray;
    property BorderColorRemaining: TColor read FBorderColorRemaining write FBorderColorRemaining default clGray;
    property ColorProgress: TColor read FColorProgress write FColorProgress default clSilver;
    property BorderColorProgress: TColor read FBorderColorProgress write FBorderColorProgress default clSilver;
    property MarkType: TJQXKnobMarkType read FMarkType write FMarkType default mtLine;
    property Offset: Integer read FOffset write FOffset default 71;
    property Width: Integer read FWidth write FWidth default 3;
    property Size: Integer read FSize write FSize default 6;
    property MajorSize: Integer read FMajorSize write FMajorSize default 9;
    property MajorInterval: Integer read FMajorInterval write FMajorInterval default 10;
    property MinorInterval: Integer read FMinorInterval write FMinorInterval default 2;
  end;

  TJQXKnobPointer = class(TPersistent)
  private
    FOwner: TJQXKnobAppearance;
    FOffset: Integer;
    FColor: TColor;
    FWidth: Integer;
    FVisible: Boolean;
    FSize: Integer;
    FPointerType: TJQXKnobPointerType;
    FBorderColor: TColor;
  public
    constructor Create(AOwner: TJQXKnobAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property PointerType: TJQXKnobPointerType read FPointerType write FPointerType default ptArrow;
    property Color: TColor read FColor write FColor default clGray;
    property BorderColor: TColor read FBorderColor write FBorderColor default clSilver;
    property Size: Integer read FSize write FSize default 59;
    property Offset: Integer read FOffset write FOffset default 49;
    property Visible: Boolean read FVisible write FVisible default true;
    property Width: Integer read FWidth write FWidth default 20;
  end;

  TJQXKnobAppearance = class(TPersistent)
  private
    FOwner: TWebJQXKnob;
    FLabels: TJQXKnobLabels;
    FProgressBar: TJQXKnobProgressBar;
    FMarks: TJQXKnobMarks;
    FPointer: TJQXKnobPointer;
    FBorderColor: TColor;
    FColor: TColor;
    FBorderWidth: Integer;
  public
    constructor Create(AOwner: TWebJQXKnob); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property Color: TColor read FColor write FColor default clNone;
    property BorderColor: TColor read FBorderColor write FBorderColor default clNone;
    property BorderWidth: Integer read FBorderWidth write FBorderWidth default 1;
    property Labels: TJQXKnobLabels read FLabels write FLabels;
    property ProgressBar: TJQXKnobProgressBar read FProgressBar write FProgressBar;
    property Marks: TJQXKnobMarks read FMarks write FMarks;
    property Pointer: TJQXKnobPointer read FPointer write FPointer;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXKnob = class(TWebJQXCustomControl)
  private
    FValue: Double;
    FOnChange: TNotifyEvent;
    FEndAngle: Integer;
    FStartAngle: Integer;
    FAppearance: TJQXKnobAppearance;
    FMaximum: Integer;
    FMinimum: Integer;
    FStep: Integer;
    procedure SetValue(const Value: Double);
  protected
    procedure Paint; override;
    property Theme;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Appearance: TJQXKnobAppearance read FAppearance write FAppearance;
    property Maximum: Integer read FMaximum write FMaximum default 100;
    property Minimum: Integer read FMinimum write FMinimum default 0;
    property Step: Integer read FStep write FStep default 1;
    property Enabled;
    property StartAngle: Integer read FStartAngle write FStartAngle default 120;
    property EndAngle: Integer read FEndAngle write FEndAngle default 420;
    property Value: Double read FValue write SetValue;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TWebJQXProgressBar = class;
  TJQXProgressBarColors = class;

  TJQXProgressBarColor = class(TCollectionItem)
  private
    FStop: Integer;
    FOwner: TJQXProgressBarColors;
    FColor: TColor;
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property Color: TColor read FColor write FColor;
    property Stop: Integer read FStop write FStop;
  end;

  TJQXProgressBarColors = class(TCollection)
  private
    FOwner: TWebJQXProgressBar;
    function GetItems(Index: Integer): TJQXProgressBarColor;
    procedure SetItems(Index: Integer; const Value: TJQXProgressBarColor);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TWebJQXProgressBar); overload;
    function GetOwner: TPersistent; override;
    function Add: TJQXProgressBarColor;
    function Insert(Index: Integer):TJQXProgressBarColor;
    property Items[Index: Integer]: TJQXProgressBarColor read GetItems write SetItems; default;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXProgressBar = class(TWebJQXCustomControl)
  private
    FValue: Integer;
    FLayout: TJQXProgressLayout;
    FMaximum: Integer;
    FShowValue: Boolean;
    FMinimum: Integer;
    FOrientation: TJQXProgressOrientation;
    FAnimationDuration: Integer;
    FColorRanges: TJQXProgressBarColors;
    FTemplate: TJQXTemplate;
    procedure SetValue(const Value: Integer);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled;
    property AnimationDuration: Integer read FAnimationDuration write FAnimationDuration;
    property ColorRanges: TJQXProgressBarColors read FColorRanges write FColorRanges;
    property Maximum: Integer read FMaximum write FMaximum;
    property Minimum: Integer read FMinimum write FMinimum;
    property Value: Integer read FValue write SetValue;
    property ShowValue: Boolean read FShowValue write FShowValue;
    property Orientation: TJQXProgressOrientation read FOrientation write FOrientation;
    property Layout: TJQXProgressLayout read FLayout write FLayout;
    property Template: TJQXTemplate read FTemplate write FTemplate;
    property Theme;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXSlider = class(TWebJQXCustomControl)
  private
    FValue: Double;
    FOnChange: TJQXSliderEvent;
    FLayout: TJQXProgressLayout;
    FMaximumValue: Double;
    FTemplate: TJQXTemplate;
    FMinorTickSize: Integer;
    FShowRangeSlider: Boolean;
    FMaximum: Double;
    FMinimumValue: Double;
    FTicksFrequency: Integer;
    FMinimum: Double;
    FShowRange: Boolean;
    FShowButtons: Boolean;
    FShowTicks: Boolean;
    FMinorTicksFrequency: Integer;
    FButtonsPosition: TJQXHorizontalPosition;
    FTicksPosition: TJQXVerticalPosition;
    FShowMinorTicks: Boolean;
    FTickSize: Integer;
    FStep: Integer;
    FMode: TJQXSliderMode;
    procedure SetValue(const Value: Double);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled;
    property ButtonsPosition: TJQXHorizontalPosition read FButtonsPosition write FButtonsPosition default hpBoth;
    property TicksPosition: TJQXVerticalPosition read FTicksPosition write FTicksPosition default vpBoth;
    property TicksFrequency: Integer read FTicksFrequency write FTicksFrequency default 2;
    property TickSize: Integer read FTickSize write FTickSize default 5;
    property MinorTicksFrequency: Integer read FMinorTicksFrequency write FMinorTicksFrequency default 2;
    property MinorTickSize: Integer read FMinorTickSize write FMinorTickSize default 5;
    property ShowButtons: Boolean read FShowButtons write FShowButtons default true;
    property ShowTicks: Boolean read FShowTicks write FShowTicks default true;
    property ShowMinorTicks: Boolean read FShowMinorTicks write FShowMinorTicks default true;
    property ShowRange: Boolean read FShowRange write FShowRange default true;
    property ShowRangeSlider: Boolean read FShowRangeSlider write FShowRangeSlider default false;
    property MaximumValue: Double read FMaximumValue write FMaximumValue;
    property MinimumValue: Double read FMinimumValue write FMinimumValue;
    property Maximum: Double read FMaximum write FMaximum;
    property Minimum: Double read FMinimum write FMinimum;
    property Mode: TJQXSliderMode read FMode write FMode default smDefault;
    property Step: Integer read FStep write FStep default 2;
    property Template: TJQXTemplate read FTemplate write FTemplate default ttDefault;
    property Layout: TJQXProgressLayout read FLayout write FLayout default plNormal;
    property Value: Double read FValue write SetValue;
    property Theme;
    property OnChange: TJQXSliderEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXRangeSelector = class(TWebJQXCustomControl)
  private
    FOnChange: TNotifyEvent;
    FMaximumValue: Double;
    FMaximum: Double;
    FMinimumValue: Double;
    FMinimum: Double;
    FShowMajorTicks: Boolean;
    FMoveOnClick: Boolean;
    FResizable: Boolean;
    FMinorTicksInterval: Integer;
    FMajorTicksInterval: Integer;
    FShowMarkers: Boolean;
    FShowMinorTicks: Boolean;
    procedure SetMaximumValue(const Value: Double);
    procedure SetMinimumValue(const Value: Double);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled;
    property MaximumValue: Double read FMaximumValue write SetMaximumValue;
    property MinimumValue: Double read FMinimumValue write SetMinimumValue;
    property Maximum: Double read FMaximum write FMaximum;
    property Minimum: Double read FMinimum write FMinimum;
    property MajorTicksInterval: Integer read FMajorTicksInterval write FMajorTicksInterval default 5;
    property MinorTicksInterval: Integer read FMinorTicksInterval write FMinorTicksInterval default 1;
    property MoveOnClick: Boolean read FMoveOnClick write FMoveOnClick default True;
    property Resizable: Boolean read FResizable write FResizable default True;
    property ShowMajorTicks: Boolean read FShowMajorTicks write FShowMajorTicks default True;
    property ShowMinorTicks: Boolean read FShowMinorTicks write FShowMinorTicks default True;
    property ShowMarkers: Boolean read FShowMarkers write FShowMarkers default True;
    property Theme;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TWebJQXTagCloud = class;
  TJQXCloudTags = class;

  TJQXCloudTag = class(TCollectionItem)
  private
    FOwner: TJQXCloudTags;
    FTagValue: Integer;
    FTag: Integer;
    FTagName: String;
    FTagLabel: String;
    procedure SetTagLabel(const Value: String);
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property Tag: Integer read FTag write FTag;
    property TagLabel: String read FTagLabel write SetTagLabel;
    property TagName: String read FTagName write FTagName;
    property TagValue: Integer read FTagValue write FTagValue;
  end;

  TJQXCloudTags = class(TCollection)
  private
    FOwner: TWebJQXTagCloud;
    function GetItems(Index: Integer): TJQXCloudTag;
    procedure SetItems(Index: Integer; const Value: TJQXCloudTag);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TWebJQXTagCloud); overload;
    function GetOwner: TPersistent; override;
    function Add: TJQXCloudTag; reintroduce;
    function Insert(Index: Integer):TJQXCloudTag; reintroduce;
    property Items[Index: Integer]: TJQXCloudTag read GetItems write SetItems; default;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXTagCloud = class(TWebJQXCustomControl)
  private
    FDesignTime: Boolean;
    FOnClick: TJQXTagEvent;
    FItems: TJQXCloudTags;
    FSortBy: TJQXTagCloudSort;
    FDisplayMinValue: Integer;
    FMaxFontSize: Integer;
    FDisplayTopWeighted: Boolean;
    FMaxColor: TColor;
    FDisplayLimit: Integer;
    FTextCase: TJQXTextCase;
    FMinFontSize: Integer;
    FMinColor: TColor;
    FDisplayMaxValue: Integer;
    FSortOrder: TJQXSortOrder;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled;
    property DisplayLimit: Integer read FDisplayLimit write FDisplayLimit default -1;
    property DisplayTopWeighted: Boolean read FDisplayTopWeighted write FDisplayTopWeighted default False;
    property DisplayMaxValue: Integer read FDisplayMaxValue write FDisplayMaxValue default -1;
    property DisplayMinValue: Integer read FDisplayMinValue write FDisplayMinValue default -1;
    property Items: TJQXCloudTags read FItems write FItems;
    property MaxColor: TColor read FMaxColor write FMaxColor default clNone;
    property MinColor: TColor read FMinColor write FMinColor default clNone;
    property MaxFontSize: Integer read FMaxFontSize write FMaxFontSize default 36;
    property MinFontSize: Integer read FMinFontSize write FMinFontSize default 8;
    property SortBy: TJQXTagCloudSort read FSortBy write FSortBy default tcsNone;
    property SortOrder: TJQXSortOrder read FSortOrder write FSortOrder default soAscending;
    property TextCase: TJQXTextCase read FTextCase write FTextCase default tcOriginal;
    property OnClick: TJQXTagEvent read FOnClick write FOnClick;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXResponsivePanel = class(TWebJQXCustomControl)
  private
    FOnOpen: TNotifyEvent;
    FOnClose: TNotifyEvent;
    FOnCollapse: TNotifyEvent;
    FOnExpand: TNotifyEvent;
    FAutoClose: Boolean;
    FToggleButtonSize: Integer;
    FCollapseBreakPoint: Integer;
    FAnimationType: TJQXAnimationType;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Open;
    procedure Close;
  published
    property Enabled;
    property Theme;
    property AnimationType: TJQXAnimationType read FAnimationType write FAnimationType default atNone;
    property AutoClose: Boolean read FAutoClose write FAutoClose default True;
    property CollapseBreakPoint: Integer read FCollapseBreakPoint write FCollapseBreakPoint default 1000;
    property ToggleButtonSize: Integer read FToggleButtonSize write FToggleButtonSize default 30;

    property OnOpen: TNotifyEvent read FOnOpen write FOnOpen;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnCollapse: TNotifyEvent read FOnCollapse write FOnCollapse;
    property OnExpand: TNotifyEvent read FOnExpand write FOnExpand;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXTabSheet = class(TWebCustomControl)
  private
    FCaption: string;
    FColor: TColor;
    procedure SetCaption(const Value: string);
  protected
    procedure Paint; override;
    procedure SetParent(AParent: TWinControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Color: TColor read FColor write FColor;
  published
    property Caption: string read FCaption write SetCaption;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebJQXTabs = class(TWebJQXCustomControl)
  private
    FItems: TStringList;
    FList: TList;
    FDesignTime: boolean;
    FTabIndex: integer;
    FOnSelected: TJQXTabEvent;
    FScrollStep: Integer;
    FScrollPosition: TJQXHorizontalPosition;
    FOnTabClick: TJQXTabEvent;
    FCollapsible: Boolean;
    FToggleMode: TJQXToggleMode;
    FSelectionTracker: Boolean;
    FReorder: Boolean;
    FEnableHover: Boolean;
    FPosition: TJQXTabPosition;
    FEnableScrollAnimation: Boolean;
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure SetItems(const Value: TStringList);
    function GetPage(index: integer): TWebJQXTabSheet;
  protected
    procedure ItemsChanged(Sender: TObject);
    procedure SetTabIndex(const Value: integer);
    procedure AlignControls(AControl: TControl; var ARect: TRect); override;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    procedure Paint; override;
    procedure AddSheet(ASheet: TWebJQXTabSheet);
    procedure RemoveSheet(ASheet: TWebJQXTabSheet);
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    property Items: TStringList read FItems write SetItems;
    procedure SelectNextPage(GoForward: boolean);
    function PageCount: integer;
    property Pages[index: integer]: TWebJQXTabSheet read GetPage;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Enabled;
    property Height;
    property Left;
    property TabIndex: integer read FTabIndex write SetTabIndex;
    property Visible;
    property Width;

    property Collapsible: Boolean read FCollapsible write FCollapsible default False;
    property EnableHover: Boolean read FEnableHover write FEnableHover default True;
    property EnableScrollAnimation: Boolean read FEnableScrollAnimation write FEnableScrollAnimation default True;
    property Position: TJQXTabPosition read FPosition write FPosition default tpTop;
    property Reorder: Boolean read FReorder write FReorder default False;
    property SelectionTracker: Boolean read FSelectionTracker write FSelectionTracker default False;
    property ScrollPosition: TJQXHorizontalPosition read FScrollPosition write FScrollPosition default hpRight;
    property ScrollStep: Integer read FScrollStep write FScrollStep default 70;
    property ToggleMode: TJQXToggleMode read FToggleMode write FToggleMode default tmClick;

    property OnTabClick: TJQXTabEvent read FOnTabClick write FOnTabClick;
    property OnSelected: TJQXTabEvent read FOnSelected write FOnSelected;
  end;

implementation

{ TWebJQXCalendar }

constructor TWebJQXCalendar.Create(AOwner: TComponent);
begin
  inherited;
  FDate := Now;
  FFirstDayOfWeek := dowMonday;
  FMaxDate := EncodeDate(2100, 12, 31);
  FMinDate := EncodeDate(1970, 1, 1);
  FMultiSelect := False;
  FOtherMonthDays := True;
  FShowToday := False;
  FWeekNumbers := False;
  Enabled := True;
  Width := 220;
  Height := 220;
end;

procedure TWebJQXCalendar.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, DateToStr(Date));
end;

procedure TWebJQXCalendar.SetDate(const Value: TDateTime);
begin
  if (FDate <> Value) then
  begin
    FDate := Value;
    Invalidate;
  end;
end;

{ TWebJQXDateTimeInput }

constructor TWebJQXDateTimeInput.Create(AOwner: TComponent);
begin
  inherited;
  FDate := Now;
  FFirstDayOfWeek := dowMonday;
  FMaxDate := EncodeDate(2100, 12, 31);
  FMinDate := EncodeDate(1970, 1, 1);
  FMultiSelect := False;
  FShowToday := False;
  FWeekNumbers := False;
  Enabled := True;
  Width := 220;
  Height := 21;
end;

procedure TWebJQXDateTimeInput.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, DateToStr(Date));
end;

procedure TWebJQXDateTimeInput.SetDate(const Value: TDateTime);
begin
  if (FDate <> Value) then
  begin
    FDate := Value;
    Invalidate;
  end;
end;

{ TWebJQXMenu }

constructor TWebJQXMenu.Create(AOwner: TComponent);
begin
  inherited;
  FMode := mmHorizontal;
  Enabled := True;
  Width := 220;
  Height := 30;
end;

procedure TWebJQXMenu.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, Name);
end;

procedure TWebJQXMenu.SetItemDisabled(AMenuItem: TMenuItem);
begin
//
end;

procedure TWebJQXMenu.SetItemEnabled(AMenuItem: TMenuItem);
begin
//
end;

{ TWebJQXMaskedInput }

constructor TWebJQXMaskedInput.Create(AOwner: TComponent);
begin
  inherited;
  FMask := '(###)###-####';
  Enabled := True;
  Width := 120;
  Height := 21;
end;

procedure TWebJQXMaskedInput.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);

  if (Text = '') and (Mask <> '') then
    Canvas.TextOut(4,4, Mask)
  else
    Canvas.TextOut(4,4, Text);
end;

procedure TWebJQXMaskedInput.SetMask(const Value: string);
begin
  if (FMask <> Value) then
  begin
    FMask := Value;
    Invalidate;
  end;
end;

{ TWebJQXColorPicker }

constructor TWebJQXColorPicker.Create(AOwner: TComponent);
begin
  inherited;
  FColor := clBlack;
  FColorMode := cmSaturation;
  Enabled := True;
  Width := 200;
  Height := 200;
end;

procedure TWebJQXColorPicker.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := clWhite;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;

  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  Canvas.Brush.Color := Color;


  lRect := Rect((Width div 2) - 16, (Height div 2) - 16, (Width div 2) + 16, (Height div 2) + 16);
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
end;

procedure TWebJQXColorPicker.SetColor(const Value: TColor);
begin
  if (FColor <> Value) then
  begin
    FColor := Value;
    Invalidate;
  end;
end;

{ TWebJQXComboBox }

constructor TWebJQXComboBox.Create(AOwner: TComponent);
begin
  inherited;
  FItemIndex := -1;
  FItems := TStringList.Create;
  FAutoComplete := false;
  FMultiSelect := false;
  FTextHint := '';
  Width := 200;
  Height := 21;
end;

destructor TWebJQXComboBox.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TWebJQXComboBox.GetDisabled(AIndex: Integer): Boolean;
begin
  Result := false;
end;

procedure TWebJQXComboBox.SetDisabled(AIndex: Integer; AValue: Boolean);
begin
end;

procedure TWebJQXComboBox.SetItems(const Value: TStringList);
begin
  if Assigned(FItems) then
    FItems.Assign(Value)
  else
    FItems := Value;
end;

function TWebJQXComboBox.GetSelected(AIndex: Integer): Boolean;
begin
  Result := false;
end;

procedure TWebJQXComboBox.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, Name);
end;

procedure TWebJQXComboBox.SetSelected(AIndex: Integer; AValue: Boolean);
begin
end;

{ TWebJQXDropDownList }

constructor TWebJQXDropDownList.Create(AOwner: TComponent);
begin
  inherited;
  FItemIndex := -1;
  FItems := TStringList.Create;
  FMultiSelect := false;
  FTextHint := '';
  Width := 200;
  Height := 21;
end;

destructor TWebJQXDropDownList.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TWebJQXDropDownList.GetDisabled(AIndex: Integer): Boolean;
begin
  Result := false;
end;

procedure TWebJQXDropDownList.SetDisabled(AIndex: Integer; AValue: Boolean);
begin
end;

procedure TWebJQXDropDownList.SetItems(const Value: TStringList);
begin
  if Assigned(FItems) then
    FItems.Assign(Value)
  else
    FItems := Value;
end;

function TWebJQXDropDownList.GetSelected(AIndex: Integer): Boolean;
begin
  Result := false;
end;

procedure TWebJQXDropDownList.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, Name);
end;

procedure TWebJQXDropDownList.SetSelected(AIndex: Integer; AValue: Boolean);
begin
end;

{ TWebJQXNumberInput }

constructor TWebJQXNumberInput.Create(AOwner: TComponent);
begin
  inherited;
  FDigits := 3;
  FDecimalDigits := 0;
  FMaxValue := 0;
  FMinValue := 0;
  FShowSpinButtons := True;
  FSpinButtonsStep := 1;
  FValue := 0;
  Width := 120;
  Height := 21;
end;

procedure TWebJQXNumberInput.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, FloatToStr(Value));
end;

procedure TWebJQXNumberInput.SetValue(const Value: Double);
begin
  if (FValue <> Value) then
  begin
    FValue := Value;
    Invalidate;
  end;
end;

{ TWebJQXRating }

constructor TWebJQXRating.Create(AOwner: TComponent);
begin
  inherited;
  FItemCount := 5;
//  Precision := rpFull;
  FValue := 0;
  Width := 100;
  Height := 20;
end;

procedure TWebJQXRating.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, FloatToStr(Value));
end;


procedure TWebJQXRating.SetValue(const Value: Double);
begin
  if FValue <> Value then
  begin
    FValue := Value;
    Invalidate;
  end;
end;

{ TWebJQXButton }

constructor TWebJQXButton.Create(AOwner: TComponent);
begin
  inherited;
  FCaption := Name;
  FCaptionPosition := xpLeft;
  FCaptionImageRelation := ipAfterImage;
  FImageURL := '';
  FImagePosition := xpLeft;
  FImageHeight := 16;
  FImageWidth := 16;
  FRoundedBorders := True;
  FTemplate := ttDefault;
  Width := 96;
  Height := 25;
end;

procedure TWebJQXButton.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, Caption);
end;

procedure TWebJQXButton.SetCaption(const Value: string);
begin
  if (FCaption <> Value) then
  begin
    FCaption := Value;
    Invalidate;
  end;
end;

{ TWebJQXGroupButton }

constructor TWebJQXButtonGroup.Create(AOwner: TComponent);
begin
  inherited;
  FEnableHover := True;
  FItems := TStringList.Create;
  FMode := gmDefault;
  FTemplate := ttDefault;
  Width := 250;
  Height := 25;

  FItems.Add('Button 1');
end;

destructor TWebJQXButtonGroup.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TWebJQXButtonGroup.GetButtonSelect(Button: integer): boolean;
begin
  Result := False;
end;

procedure TWebJQXButtonGroup.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, Name);
end;

procedure TWebJQXButtonGroup.SetButtonSelect(Button: integer;
  const Value: boolean);
begin

end;

procedure TWebJQXButtonGroup.SetItems(const Value: TStringList);
begin
  if Assigned(FItems) then
    FItems.Assign(Value)
  else
    FItems := Value;
end;

{ TWebJQXKnob }

constructor TWebJQXKnob.Create(AOwner: TComponent);
begin
  inherited;
  FAppearance := TJQXKnobAppearance.Create(Self);
  FMaximum := 100;
  FMinimum := 0;
  FStep := 1;
  FStartAngle := 120;
  FEndAngle := 420;
  FValue := 0;
  Width := 200;
  Height := 200;
end;

destructor TWebJQXKnob.Destroy;
begin
  FAppearance.Free;
  inherited;
end;

procedure TWebJQXKnob.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, FloatToStr(Value));
end;

procedure TWebJQXKnob.SetValue(const Value: Double);
begin
  if FValue <> Value then
  begin
    FValue := Value;
    Invalidate;
  end;
end;

{ TWebJQXProgressBar }

constructor TWebJQXProgressBar.Create(AOwner: TComponent);
begin
  inherited;
  FAnimationDuration := 300;
  FColorRanges := TJQXProgressBarColors.Create(Self);
  FOrientation := poHorizontal;
  FLayout := plNormal;
  FShowValue := True;
  FMaximum := 100;
  FMinimum := 0;
  FValue := 0;
  Width := 200;
  Height := 21;
  FTemplate := ttDefault;
end;

destructor TWebJQXProgressBar.Destroy;
begin
  FColorRanges.Free;
  inherited;
end;

procedure TWebJQXProgressBar.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, FloatToStr(Value));
end;

procedure TWebJQXProgressBar.SetValue(const Value: Integer);
begin
  if FValue <> Value then
  begin
    FValue := Value;
    Invalidate;
  end;
end;

{ TJQXProgressBarColor }

constructor TJQXProgressBarColor.Create(ACollection: TCollection);
begin
  inherited;
  FColor := clNone;
  FStop := 0;

  if ACollection is TJQXProgressBarColors then
  begin
    FOwner := ACollection as TJQXProgressBarColors;
  end;
end;

destructor TJQXProgressBarColor.Destroy;
begin
  inherited;
end;

function TJQXProgressBarColor.GetDisplayName: string;
begin
  Result := 'Color '+ IntToStr(Index);
end;

function TJQXProgressBarColor.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TJQXProgressBarColors }

function TJQXProgressBarColors.Add: TJQXProgressBarColor;
begin
  Result := TJQXProgressBarColor(inherited Add);
end;

constructor TJQXProgressBarColors.Create(AOwner: TWebJQXProgressBar);
begin
  inherited Create(TJQXProgressBarColor);
  FOwner := AOwner;
end;

function TJQXProgressBarColors.GetItems(Index: Integer): TJQXProgressBarColor;
begin
  Result := TJQXProgressBarColor(inherited Items[Index]);
end;

function TJQXProgressBarColors.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TJQXProgressBarColors.Insert(Index: Integer): TJQXProgressBarColor;
begin
  Result := TJQXProgressBarColor(inherited Insert(Index));
end;

procedure TJQXProgressBarColors.SetItems(Index: Integer; const Value: TJQXProgressBarColor);
begin
  inherited Items[Index] := Value;
end;

procedure TJQXProgressBarColors.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TWebJQXSlider }

constructor TWebJQXSlider.Create(AOwner: TComponent);
begin
  inherited;
  FButtonsPosition := hpBoth;
  FTicksPosition := vpBoth;
  FTicksFrequency := 2;
  FTickSize := 5;
  FMinorTicksFrequency := 2;
  FMinorTickSize := 5;
  FShowButtons := true;
  FShowTicks := true;
  FShowMinorTicks := true;
  FShowRange := true;
  FShowRangeSlider := false;
  FMaximumValue := 10;
  FMinimumValue := 0;
  FMaximum := 10;
  FMinimum := 0;
  FMode := smDefault;
  FStep := 2;
  FTemplate := ttDefault;
  FLayout := plNormal;
  Value := 0;
  Width := 200;
  Height := 22;
end;

procedure TWebJQXSlider.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, FloatToStr(Value));
end;

procedure TWebJQXSlider.SetValue(const Value: Double);
begin
  if FValue <> Value then
  begin
    FValue := Value;
    Invalidate;
  end;
end;

{ TJQXKnobAppearance }

procedure TJQXKnobAppearance.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TJQXKnobAppearance) then
  begin
    FColor := (Source as TJQXKnobAppearance).Color;
    FBorderColor := (Source as TJQXKnobAppearance).BorderColor;
    FBorderWidth := (Source as TJQXKnobAppearance).BorderWidth;
    FLabels := (Source as TJQXKnobAppearance).Labels;
    FProgressBar := (Source as TJQXKnobAppearance).ProgressBar;
  end;
end;

constructor TJQXKnobAppearance.Create(AOwner: TWebJQXKnob);
begin
  inherited Create;
  FColor := clNone;
  FBorderColor := clNone;
  FBorderWidth := 1;
  FLabels := TJQXKnobLabels.Create(Self);
  FProgressBar := TJQXKnobProgressBar.Create(Self);
  FMarks := TJQXKnobMarks.Create(Self);
  FPointer := TJQXKnobPointer.Create(Self);
  FOwner := AOwner;
end;

destructor TJQXKnobAppearance.Destroy;
begin
  FLabels.Free;
  FProgressBar.Free;
  FMarks.Free;
  FPointer.Free;
  inherited;
end;

function TJQXKnobAppearance.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXKnobLabels }

procedure TJQXKnobLabels.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TJQXKnobLabels) then
  begin
    FOffset := (Source as TJQXKnobLabels).Offset;
    FStep := (Source as TJQXKnobLabels).Step;
    FVisible := (Source as TJQXKnobLabels).Visible;
  end;
end;

constructor TJQXKnobLabels.Create(AOwner: TJQXKnobAppearance);
begin
  FOffset := 88;
  FStep := 10;
  FVisible := True;
  FOwner := AOwner;
end;

destructor TJQXKnobLabels.Destroy;
begin
  inherited;
end;

function TJQXKnobLabels.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXKnobProgressBar }

procedure TJQXKnobProgressBar.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TJQXKnobProgressBar) then
  begin
    FColor := (Source as TJQXKnobProgressBar).Color;
    FSize := (Source as TJQXKnobProgressBar).Size;
    FOffset := (Source as TJQXKnobProgressBar).Offset;
    FBackgroundColor := (Source as TJQXKnobProgressBar).BackgroundColor;
    FBorderColor := (Source as TJQXKnobProgressBar).BorderColor;
  end;
end;

constructor TJQXKnobProgressBar.Create(AOwner: TJQXKnobAppearance);
begin
  FColor := clSilver;
  FSize := 9;
  FOffset := 60;
  FBackgroundColor := clGray;
  FBorderColor := clBlack;
  FOwner := AOwner;
end;

destructor TJQXKnobProgressBar.Destroy;
begin
  inherited;
end;

function TJQXKnobProgressBar.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXKnobMarks }

procedure TJQXKnobMarks.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TJQXKnobMarks) then
  begin
    FOffset := (Source as TJQXKnobMarks).Offset;
    FMajorSize := (Source as TJQXKnobMarks).MajorSize;
    FColorProgress := (Source as TJQXKnobMarks).ColorProgress;
    FWidth := (Source as TJQXKnobMarks).Width;
    FBorderColorRemaining := (Source as TJQXKnobMarks).BorderColorRemaining;
    FBorderColorProgress := (Source as TJQXKnobMarks).BorderColorProgress;
    FMinorInterval := (Source as TJQXKnobMarks).MinorInterval;
    FSize := (Source as TJQXKnobMarks).Size;
    FColorRemaining := (Source as TJQXKnobMarks).ColorRemaining;
    FMajorInterval := (Source as TJQXKnobMarks).MajorInterval;
    FMarkType := (Source as TJQXKnobMarks).MarkType;
  end;
end;

constructor TJQXKnobMarks.Create(AOwner: TJQXKnobAppearance);
begin
  inherited Create;
  FColorRemaining := clGray;
  FBorderColorRemaining := clGray;
  FColorProgress := clSilver;
  FBorderColorProgress := clSilver;
  FMarkType := mtLine;
  FOffset := 71;
  FWidth := 3;
  FSize := 6;
  FMajorSize := 9;
  FMajorInterval := 10;
  FMinorInterval := 2;
  FOwner := AOwner;
end;

destructor TJQXKnobMarks.Destroy;
begin
  inherited;
end;

function TJQXKnobMarks.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

{ TJQXKnobPointer }

procedure TJQXKnobPointer.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TJQXKnobPointer) then
  begin
    FPointerType := (Source as TJQXKnobPointer).PointerType;
    FColor := (Source as TJQXKnobPointer).Color;
    FBorderColor := (Source as TJQXKnobPointer).BorderColor;
    FSize := (Source as TJQXKnobPointer).Size;
    FOffset := (Source as TJQXKnobPointer).Offset;
    FVisible := (Source as TJQXKnobPointer).Visible;
    FWidth := (Source as TJQXKnobPointer).Width;
  end;
end;

constructor TJQXKnobPointer.Create(AOwner: TJQXKnobAppearance);
begin
  FPointerType := ptArrow;
  FColor := clGray;
  FBorderColor := clSilver;
  FSize := 59;
  FOffset := 49;
  FVisible := True;
  FWidth := 20;
  FOwner := AOwner;
end;

destructor TJQXKnobPointer.Destroy;
begin
  inherited;
end;

function TJQXKnobPointer.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TWebJQXRangeSelector }

constructor TWebJQXRangeSelector.Create(AOwner: TComponent);
begin
  inherited;
  FMaximumValue := 10;
  FMinimumValue := 0;
  FMaximum := 10;
  FMinimum := 0;
  FShowMajorTicks := True;
  FMoveOnClick := True;
  FResizable := True;
  FMinorTicksInterval := 1;
  FMajorTicksInterval := 5;
  FShowMarkers := True;
  FShowMinorTicks := True;
  Width := 200;
  Height := 60;
end;

procedure TWebJQXRangeSelector.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, FloatToStr(MinimumValue) + ' - ' + FloatToStr(MaximumValue));
end;

procedure TWebJQXRangeSelector.SetMaximumValue(const Value: Double);
begin
  if FMaximumValue <> Value then
  begin
    FMaximumValue := Value;
    Invalidate;
  end;
end;

procedure TWebJQXRangeSelector.SetMinimumValue(const Value: Double);
begin
  if FMinimumValue <> Value then
  begin
    FMinimumValue := Value;
    Invalidate;
  end;
end;

{ TWebJQXTagCloud }

constructor TWebJQXTagCloud.Create(AOwner: TComponent);
var
  ltag: TJQXCloudTag;
begin
  inherited;
  FItems := TJQXCloudTags.Create(Self);
  FSortBy := tcsNone;
  FDisplayMinValue := -1;
  FMaxFontSize := 36;
  FDisplayTopWeighted := False;
  FMaxColor := clNone;
  FDisplayLimit := -1;
  FTextCase := tcOriginal;
  FMinFontSize := 8;
  FMinColor := clNone;
  FDisplayMaxValue := -1;
  FSortOrder := soAscending;
  Width := 200;
  Height := 50;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  if FDesignTime then
  begin
    ltag := Items.Add;
    ltag.TagLabel := 'Tag Item 1';
    ltag.TagValue := 50;

    ltag := Items.Add;
    ltag.TagLabel := 'Tag Item 2';
    ltag.TagValue := 100;

    ltag := Items.Add;
    ltag.TagLabel := 'Tag Item 3';
    ltag.TagValue := 75;
  end;
end;

destructor TWebJQXTagCloud.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TWebJQXTagCloud.Paint;
var
  lRect: TRect;
  lBuffer: string;
  stext: string;
  I: Integer;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  for I := 0 to Items.Count - 1 do
  begin
    if I > 0 then
      stext := stext + ' ';
    stext := stext + Items[I].TagLabel;
  end;

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4,4, stext);
end;

{ TJQXCloudTag }

procedure TJQXCloudTag.Assign(Source: TPersistent);
begin
  if (Source is TJQXCloudTag) then
  begin
    FTag := (Source as TJQXCloudTag).Tag;
    FTagName := (Source as TJQXCloudTag).TagName;
    FTagValue := (Source as TJQXCloudTag).TagValue;
  end;
end;

constructor TJQXCloudTag.Create(ACollection: TCollection);
begin
  inherited;
  FTag := -1;
  FTagName := '';
  FTagValue := 0;

  if ACollection is TJQXCloudTags then
  begin
    FOwner := ACollection as TJQXCloudTags;
  end;
end;

destructor TJQXCloudTag.Destroy;
begin
  inherited;
end;

function TJQXCloudTag.GetDisplayName: string;
begin
  Result := 'Tag '+ IntToStr(Index);
end;

function TJQXCloudTag.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

procedure TJQXCloudTag.SetTagLabel(const Value: String);
begin
  if FTagLabel <> Value then
  begin
    FTagLabel := Value;
    FOwner.FOwner.Invalidate;
  end;
end;

{ TJQXCloudTags }

function TJQXCloudTags.Add: TJQXCloudTag;
begin
  Result := TJQXCloudTag(inherited Add);
end;

constructor TJQXCloudTags.Create(AOwner: TWebJQXTagCloud);
begin
  inherited Create(TJQXCloudTag);
  FOwner := AOwner;
end;

function TJQXCloudTags.GetItems(Index: Integer): TJQXCloudTag;
begin
  Result := TJQXCloudTag(inherited Items[Index]);
end;

function TJQXCloudTags.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TJQXCloudTags.Insert(Index: Integer): TJQXCloudTag;
begin
  Result := TJQXCloudTag(inherited Insert(Index));
end;

procedure TJQXCloudTags.SetItems(Index: Integer; const Value: TJQXCloudTag);
begin
  inherited Items[Index] := Value;
end;

procedure TJQXCloudTags.Update(Item: TCollectionItem);
begin
  inherited;
end;

{ TWebJQXResponsivePanel }

procedure TWebJQXResponsivePanel.Close;
begin
//
end;

constructor TWebJQXResponsivePanel.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  FAnimationType := atNone;
  FAutoClose := True;
  FCollapseBreakpoint := 1000;
  FToggleButtonSize := 30;
  Width := 200;
  Height := 200;
end;

procedure TWebJQXResponsivePanel.Open;
begin
//
end;

procedure TWebJQXResponsivePanel.Paint;
begin
  inherited;
end;

{ TWebJQXTabSheet }

constructor TWebJQXTabSheet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csAcceptsControls];
end;

destructor TWebJQXTabSheet.Destroy;
begin
  if Assigned(Owner) and (Owner is TWebJQXTabs) then
    (Owner as TWebJQXTabs).RemoveSheet(Self);

  inherited;
end;

procedure TWebJQXTabSheet.Paint;
begin
end;

procedure TWebJQXTabSheet.SetCaption(const Value: string);
begin
  FCaption := Value;
  if Assigned(Parent) then
    Parent.Invalidate;
end;

procedure TWebJQXTabSheet.SetParent(AParent: TWinControl);
begin
  inherited;

  if Assigned(AParent) and (AParent is TWebJQXTabs) then
    (AParent as TWebJQXTabs).AddSheet(Self);
end;

{ TWebJQXTabs }

procedure TWebJQXTabs.AddSheet(ASheet: TWebJQXTabSheet);
var
  i: integer;
  found: boolean;
begin
  found := false;
  for i := 0 to FList.Count - 1 do
  begin
    if TWebJQXTabSheet(FList.Items[i]) = ASheet then
      found := true;
  end;

  if not found then
    FList.Add(ASheet);

  Invalidate;
end;

procedure TWebJQXTabs.AlignControls(AControl: TControl; var ARect: TRect);
var
  i: integer;
begin
  inherited;

  for I := 0 to ControlCount - 1 do
  begin
    if Controls[i] is TWebJQXTabSheet then
    begin
      Controls[i].SetBounds(0, 20, Width, Height - 20);
    end;
  end;
end;

procedure TWebJQXTabs.CMDesignHitTest(var Message: TCMDesignHitTest);
var
  p: TPoint;
  Tab, i, x, bw: integer;
begin
  Tab := -1;

  if (csDesigning in ComponentState) then
  begin
    GetCursorPos(P);
    P := ScreenToClient(P);
    x := 0;

    if (p.Y < 20) and
      ({$IFDEF LCLTMSWEB}GetKeyState{$ELSE}GetAsyncKeyState{$ENDIF}(VK_LBUTTON) <> 0) then
    begin
      for i := 0 to FList.Count - 1 do
      begin
        bw := Canvas.TextWidth(TWebTabSheet(FList.Items[i]).Caption) + 10;

        if (p.X > x) and (p.X < x + BW) then
        begin
          TabIndex := i;
          break;
        end;
        x := x + BW + 10;
      end;
    end;
  end;

  if Tab = -1 then
    inherited;
end;

constructor TWebJQXTabs.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  FItems := TStringList.Create;
  FItems.OnChange := ItemsChanged;
  FList := TList.Create;
  FCollapsible := False;
  FEnableHover := True;
  FEnableScrollAnimation := True;
  FPosition := tpTop;
  FReorder := False;
  FSelectionTracker := False;
  FScrollPosition := hpRight;
  FScrollStep := 70;
  FToggleMode := tmClick;
  Width := 400;
  Height := 300;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));
end;

procedure TWebJQXTabs.CreateWnd;
var
  TS: TWebJQXTabSheet;
begin
  inherited;

  if FDesignTime then
  begin
    FDesignTime := false;
    TS := TWebJQXTabSheet.Create(Owner);
    TS.Parent := Self;
    TS.Caption := Name + 'Sheet1';
    TS.Name := TS.Caption;

    TS := TWebJQXTabSheet.Create(Owner);
    TS.Parent := Self;
    TS.Caption := Name + 'Sheet2';
    TS.Name := TS.Caption;

    TS := TWebJQXTabSheet.Create(Owner);
    TS.Parent := Self;
    TS.Caption := Name + 'Sheet3';
    TS.Name := TS.Caption;
  end;
end;

destructor TWebJQXTabs.Destroy;
begin
  FList.Free;
  FItems.Free;
  inherited;
end;

procedure TWebJQXTabs.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
  Control: TControl;
begin
  for I := 0 to FList.Count - 1 do Proc(TComponent(FList[I]));

  for I := 0 to ControlCount - 1 do
  begin
    Control := Controls[I];
    if (Control.Owner = Root) and (FList.IndexOf(Control) < 0) then Proc(Control);
  end;
end;

function TWebJQXTabs.GetPage(index: integer): TWebJQXTabSheet;
begin
  Result := nil;

  if (index >= 0) and (index < FList.Count) then
    Result :=  TWebJQXTabSheet(FList.Items[index]);
end;

procedure TWebJQXTabs.ItemsChanged(Sender: TObject);
begin

end;

function TWebJQXTabs.PageCount: integer;
begin
  Result := FList.Count;
end;

procedure TWebJQXTabs.Paint;
var
  i: integer;
  x: integer;
  s: string;
  bw: integer;
  TS: TWebTabSheet;
begin
  Canvas.Brush.Color := clBtnFace;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBtnFace;
  Canvas.Pen.Style := psSolid;
  Canvas.Rectangle(ClientRect);

  x := 2;
  Canvas.Brush.Style := bsClear;

  for i := 0 to FList.Count - 1 do
  begin
    TS := TWebTabSheet(FList.Items[i]);
    s := TS.Caption;
    bw := Canvas.TextWidth(s);
    if i = TabIndex then
    begin
      Canvas.Brush.Color := clWhite;
      Canvas.Brush.Style := bsSolid;
      Canvas.Pen.Color := clWhite;
      Canvas.Pen.Style := psSolid;
      Canvas.Rectangle(x, 0, x + bw + 4, 20);
    end;

    Canvas.Brush.Style := bsClear;
    Canvas.TextOut(x,2, s);
    x := x + bw + 10;
  end;
end;

procedure TWebJQXTabs.RemoveSheet(ASheet: TWebJQXTabSheet);
var
  i: integer;
begin
  for i := 0 to FList.Count - 1 do
  begin
    if TWebJQXTabSheet(FList.Items[i]) = ASheet then
    begin
      FList.Delete(i);
      Break;
    end;
  end;
  Invalidate;
end;

procedure TWebJQXTabs.SelectNextPage(GoForward: boolean);
begin
  if GoForward then
  begin
    if TabIndex < PageCount - 1 then
      TabIndex := TabIndex + 1
    else
      TabIndex := 0;
  end
  else
  begin
    if TabIndex > 0 then
      TabIndex := TabIndex - 1
    else
      TabIndex := PageCount - 1;
  end;
end;

procedure TWebJQXTabs.SetChildOrder(Child: TComponent; Order: Integer);
begin
  inherited;

end;

procedure TWebJQXTabs.SetItems(const Value: TStringList);
begin
  FItems.Assign(Value);
end;

procedure TWebJQXTabs.SetTabIndex(const Value: integer);
var
  TS: TWebTabSheet;
begin
  if (Value >= 0) and (Value < FList.Count) then
  begin
    TS := TWebTabSheet(FList.Items[Value]);
    TS.BringToFront;
    FTabIndex := Value;
    Invalidate;
  end;
end;

end.
