unit WEBLib.PushNotifications;

interface

uses
  Classes, SysUtils, WEBLib.Controls, js, web, libserviceworkers, libpushmanager, libpushsubscription, jsdelphisystem;

const
  DefaultServiceWorker = 'service-worker.js';
  DefaultVapidPublicKeyURL = 'vapidPublicKey';
  DefaultRegisterSubscriptionURL = 'registerSubscription';
  DefaultUnregisterSubscriptionURL = 'unregisterSubscription';
  DefaultLoginURL = 'loginUser';
  DefaultLogoutURL = 'logoutUser';

type
  EPushNotifications = class(Exception);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebPushNotifications = class(TComponent)
  private
    FOnUnsubscribed: TNotifyEvent;
    FOnSubscriptionRegistered: TNotifyEvent;
    FServiceWorkerURL: string;
    FRegistrationUserData: string;
    FAutoRegisterSubscription: Boolean;
    FOnGetRegistration: TNotifyEvent;
    FLogoutURL: string;
    FMyRegistration: TJSServiceWorkerRegistration;
    FOnGetSubscription: TNotifyEvent;
    FVapidPublicKeyURL: string;
    FLoginURL: string;
    FAutoGetSubscription: Boolean;
    FSubScription: TJSPushSubscription;
    FOnGetVapidPublicKey: TNotifyEvent;
    FRegistrationUserID: string;
    FVapidPublicKey: string;
    FRegisterSubscriptionURL: string;
    FUnregisterSubscriptionURL: string;
    function IsLoginURLStored: Boolean;
    function IsLogoutURLStored: Boolean;
    function IsRegisterSubscriptionURLStored: Boolean;
    function IsServiceWorkerStored: Boolean;
    function IsUnregisterSubscriptionURLStored: Boolean;
    function IsVapidPublicKeyURLStored: Boolean;
  public
    constructor Create(aOwner: TComponent); override;
    // Create a new subscription.
    procedure CreateNewSubscription;
    // Get the vapidPublicKey from the server
    procedure GetVapidPublicKey;
    procedure Login;
    procedure Logout;
    // Registerserviceworker will create a subscription if needed.
    procedure RegisterServiceWorker;
    // Registersubscription registers the subscription on the server.
    procedure RegisterSubscription;
    procedure Unsubscribe(aAll: Boolean = True);
    property Subscription: TJSPushSubscription read FSubScription;
    property Registration: TJSServiceWorkerRegistration read FMyRegistration;
  published
    // Should we automatically get a subscription if a vapid key is received
    property AutoGetSubscription: Boolean read FAutoGetSubscription write FAutoGetSubscription default True;
    // Should the subscription be automatically registered ?
    property AutoRegisterSubscription: Boolean read FAutoRegisterSubscription write FAutoRegisterSubscription default True;
    // Url for service worker
    property LoginURL: string read FLoginURL write FLoginURL stored IsLoginURLStored;
    property LogoutURL: string read FLogoutURL write FLogoutURL stored IsLogoutURLStored;
    // If set, passed on when registering subscription
    property RegistrationUserID: string read FRegistrationUserID write FRegistrationUserID;
    property RegistrationUserData: string read FRegistrationUserData write FRegistrationUserData;
    // Url for subscription registration.
    property RegisterSubscriptionURL: string read FRegisterSubscriptionURL write FRegisterSubscriptionURL stored IsRegisterSubscriptionURLStored;
    property ServiceWorkerURL: string read FServiceWorkerURL write FServiceWorkerURL stored IsServiceWorkerStored;
    property UnregisterSubscriptionURL: string read FUnregisterSubscriptionURL write FUnregisterSubscriptionURL stored IsUnregisterSubscriptionURLStored;
    // Vapid Public key. Can be fetched from server using VapidPubickeyURL
    property VapidPublicKey: string read FVapidPublicKey write FVapidPublicKey;
    // Url to fetch VapidPublicKey if it is not set.
    property VapidPublickeyURL: string read FVapidPublicKeyURL write FVapidPublicKeyUrl stored IsVapidPublicKeyURLStored;
    // called when service worker registration is available.
    property OnGetRegistration: TNotifyEvent read FOnGetRegistration write FOnGetRegistration;
    // called when a subscription is available.
    property OnGetSubscription: TNotifyEvent read FOnGetSubscription write FOnGetSubscription;
    // called when vapidPublicKey is fetched from the server.
    property OnGetVapidPublicKey: TNotifyEvent read FOnGetVapidPublicKey write FOnGetVapidPublicKey;
    // called when a subscription is sucessfully registered.
    property OnSubscriptionRegistered: TNotifyEvent read FOnSubscriptionRegistered write FOnSubscriptionRegistered;
    property OnUnsubscribed: TNotifyEvent read FOnUnsubscribed write FOnUnsubscribed;
  end;

function NavigatorServiceWorker: TJSServiceWorkerContainer;
function GetPushManager(aReg: TJSServiceWorkerRegistration): TJSPushManager;

implementation

function NavigatorServiceWorker: TJSServiceWorkerContainer;
begin
  Result := nil;
end;

function GetPushManager(aReg: TJSServiceWorkerRegistration): TJSPushManager;
begin
  Result := nil;
end;

{ TWebPushNotifications }

constructor TWebPushNotifications.Create(aOwner: TComponent);
begin
  inherited;
  FAutoRegisterSubscription := True;
  FAutoGetSubscription := True;
end;


procedure TWebPushNotifications.CreateNewSubscription;
begin

end;


procedure TWebPushNotifications.GetVapidPublicKey;
begin

end;


function TWebPushNotifications.IsLoginURLStored: Boolean;
begin
  Result := FLoginURL <> DefaultLoginURL;
end;


function TWebPushNotifications.IsLogoutURLStored: Boolean;
begin
  Result := FLogoutURL <> DefaultLogoutURL;
end;


function TWebPushNotifications.IsRegisterSubscriptionURLStored: Boolean;
begin
  Result := FRegisterSubscriptionURL <> DefaultRegisterSubscriptionURL;
end;


function TWebPushNotifications.IsServiceWorkerStored: Boolean;
begin
  Result := FServiceWorkerURL <> DefaultServiceWorker;
end;


function TWebPushNotifications.IsUnregisterSubscriptionURLStored: Boolean;
begin
  Result := FUnregisterSubscriptionURL <> DefaultUnregisterSubscriptionURL;
end;


function TWebPushNotifications.IsVapidPublicKeyURLStored: Boolean;
begin
  Result := FVapidPublicKeyURL <> DefaultVapidPublicKeyURL;
end;


procedure TWebPushNotifications.Login;
begin

end;


procedure TWebPushNotifications.Logout;
begin

end;


procedure TWebPushNotifications.RegisterServiceWorker;
begin

end;


procedure TWebPushNotifications.RegisterSubscription;
begin

end;


procedure TWebPushNotifications.Unsubscribe(aAll: Boolean);
begin

end;



end.


