{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.RadServerCDS;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, DB, DBClient, WEBLib.Controls, WEBLib.REST, Web, jsdelphisystem, js, WEBLib.CDS;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, DB, WEBLib.Controls, WEBLib.REST, Web, jsdelphisystem, WEBLib.CDS;
  {$ENDIF}

type
  TRadServerErrorEvent = procedure(Sender: TObject; errMsg: String) of object;
  TRadServerGetQueryParameterEvent = function(Sender: TObject; aCommand: THTTPCommand; var Handled: Boolean): String of object;
  TRadServerSignupUserExistsEvent = procedure(Sender: TObject; UserName: String) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebRadServerClientDataset = Class(TWebClientDataSet)
  private
    FTableName, FRadServerURL: String;
    FKeyFieldName, FUserName, FPassword,
    FMasterSecret, FAppSecret: String;

    FOnError: TRadServerErrorEvent;
    FOnGetQueryParameter: TRadServerGetQueryParameterEvent;
    FOnUserExists: TRadServerSignupUserExistsEvent;
  protected
    procedure InternalClose; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalOpen; override;
    function IsCursorOpen: Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh(Force: Boolean=False); 
    procedure AddSortFieldDef(aField: String; isAscending: Boolean);
    procedure ClearSortFieldDefs;
    procedure Login(aUserName, aPassword: String; IsSignUp: Boolean=False);
  published
    property RadServerURL: String read FRadServerURL write FRadServerURL;
    property TableName: String read FTableName write FTableName;
    property KeyFieldName: String read FKeyFieldName write FKeyFieldName;

    // If an AppSecret has been set up in Rad Server, it must
    // be specified in addition to the login credentials
    property AppSecret: String read FAppSecret write FAppSecret;

    // If a MasterSecret has been set up in Rad Server, specify
    // this as an Altnernative to all of above, including Login.
    // It will always get access for all operations.
    property MasterSecret: String read FMasterSecret write FMasterSecret;

    property OnError: TRadServerErrorEvent read FOnError write FOnError;
    property OnUserExists: TRadServerSignupUserExistsEvent read FOnUserExists write FOnUserExists;

    // App can send custom query parameters to attach to a REST command by
    // this event. Note that if it is used for httpGET then the App should
    // take care of sorting parameters completely. In its absence, the component
    // inserts sorting parameters with the assumption of using FireDac
    property OnGetQueryParameter: TRadServerGetQueryParameterEvent read FOnGetQueryParameter write FOnGetQueryParameter;

    // Other
    property Active;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnFilterRecord;
    property OnNewRecord;
    property OnPostError;
  end;


implementation

procedure TWebRadServerClientDataset.InternalClose;
begin
  //
end;

procedure TWebRadServerClientDataset.InternalHandleException;
begin
  //
end;

procedure TWebRadServerClientDataset.InternalInitFieldDefs;
begin
  //
end;

procedure TWebRadServerClientDataset.InternalOpen;
begin
  //
end;

function TWebRadServerClientDataset.IsCursorOpen: Boolean;
begin
  Result := false;
end;

constructor TWebRadServerClientDataset.Create(AOwner: TComponent);
begin
  inherited;
  FRadServerURL := '';
  FTableName := '';
  FKeyFieldName := '';
  FUserName := '';
  FPassword := '';
  FMasterSecret := '';
  FAppSecret := '';
end;

destructor TWebRadServerClientDataset.Destroy;
begin
  inherited;
end;

procedure TWebRadServerClientDataset.refresh(Force: Boolean=False);
begin
end;

procedure TWebRadServerClientDataset.AddSortFieldDef(aField: String; isAscending: Boolean);
begin
end;

procedure TWebRadServerClientDataset.ClearSortFieldDefs;
begin
end;

procedure TWebRadServerClientDataset.Login(aUserName, aPassword: String; IsSignUp: Boolean=False);
begin
end;

end.
