{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.RegularExpressions;

interface

uses
  System.Sysutils, System.Classes, Web, js;

type
  TRegExOption = (roNone, roIgnoreCase, roMultiLine,
    roSingleLine, roJSUnicode);

  TRegExOptions = set of TRegExOption;

  { Difference from Delphi TGroup:
    JS does not return Index and Length for a Group. So
    these propertiess are absent.
  }
  TGroup = record
  private
    FValue: String;
    FSuccess: Boolean;
  public
    property Value: String read FValue;
    property Success: Boolean read FSuccess;
  end;

  TGroupCollectionEnumerator = class;

  TGroupArray = array of TGroup;

  TGroupCollection = record
  private
    FCount: Integer;
    function GetItem(Index: JSValue): TGroup;
  public
    function GetEnumerator: TGroupCollectionEnumerator;
    property Count: Integer read FCount;
    property Item[Index: JSValue]: TGroup read GetItem; default;
  end;

  TGroupCollectionEnumerator = class
  private
    FCurrent: TGroup;
  public
    function GetCurrent: TGroup;
    function MoveNext: Boolean;
    property Current: TGroup read FCurrent;
  end;

  TMatch = record
  private
    FIndex: Integer;
    FLength: Integer;
    FSuccess: Boolean;
    FGroups: TGroupCollection;
    FValue: String;
  public
    function NextMatch: TMatch;
    property Groups: TGroupCollection read FGroups;
    property Index: Integer read FIndex;
    property Length: Integer read FLength;
    property Success: Boolean read FSuccess write FSuccess;
    property Value: String read FValue;
  end;

  TMatchCollectionEnumerator = class;

  TMatchArray = array of TMatch;

  TMatchCollection = record
  private
    FCount: Integer;
    function GetItem(Index: Integer): TMatch;
  public
    function GetEnumerator: TMatchCollectionEnumerator;
    property Count: Integer read FCount;
    property Item[Index: Integer]: TMatch read GetItem; default;
  end;

  TMatchCollectionEnumerator = class
  private
    FCurrent: TMatch;
  public
    function GetCurrent: TMatch;
    function MoveNext: Boolean;
    property Current: TMatch read FCurrent;
  end;

  TMatchEvaluator = function(const Match: TMatch): String of object;

  TRegEx = record
  public
    constructor Create(const Pattern: String); overload;
    constructor Create(const Pattern: String; Options: TRegExOptions); overload;

    function IsMatch(const Input: String): Boolean; overload;
    function IsMatch(const Input: String; StartPos: Integer): Boolean; overload;
    class function IsMatch(const Input, Pattern: String): Boolean; overload; static;
    class function IsMatch(const Input, Pattern: String; Options: TRegExOptions): Boolean; overload; static;

    class function Escape(const Str: String; UseWildCards: Boolean = False): String; static;

    function Match(const Input: String): TMatch; overload;
    function Match(const Input: String; StartPos: Integer): TMatch; overload;
    function Match(const Input: String; StartPos, Length: Integer): TMatch; overload;
    class function Match(const Input, Pattern: String): TMatch; overload; static;
    class function Match(const Input, Pattern: String; Options: TRegExOptions): TMatch; overload; static;

    function Matches(const Input: String): TMatchCollection; overload;
    function Matches(const Input: String; StartPos: Integer): TMatchCollection; overload;
    class function Matches(const Input, Pattern: String): TMatchCollection; overload; static;
    class function Matches(const Input, Pattern: String; Options: TRegExOptions): TMatchCollection; overload; static;

    function Replace(const Input, Replacement: String): String; overload;
    function Replace(const Input: String; Evaluator: TMatchEvaluator): String; overload;
    // Note: overloaded Replace functions with Count are not supported.
    class function Replace(const Input, Pattern, Replacement: String): String; overload; static;
    class function Replace(const Input, Pattern: String; Evaluator: TMatchEvaluator): String; overload; static;
    class function Replace(const Input, Pattern, Replacement: String; Options: TRegExOptions): String; overload; static;
    class function Replace(const Input, Pattern: String; Evaluator: TMatchEvaluator; Options: TRegExOptions): String; overload; static;

    function Split(const Input: String): TStringDynArray; overload;
    // Note: overloaded Split function with Count are not supported.
    class function Split(const Input, Pattern: String): TStringDynArray; overload; static;
    class function Split(const Input, Pattern: String; Options: TRegExOptions): TStringDynArray; overload; static;
  end;

implementation

{$HINTS OFF}
{$WARNINGS OFF}

{ TGroupCollection }

function TGroupCollection.GetEnumerator: TGroupCollectionEnumerator;
begin
end;

function TGroupCollection.GetItem(Index: JSValue): TGroup;
begin
end;

{ TGroupCollectionEnumerator }

function TGroupCollectionEnumerator.GetCurrent: TGroup;
begin
end;

function TGroupCollectionEnumerator.MoveNext: Boolean;
begin
end;

{ TMatch }

function TMatch.NextMatch: TMatch;
begin
end;

{ TMatchCollection }

function TMatchCollection.GetEnumerator: TMatchCollectionEnumerator;
begin
end;

function TMatchCollection.GetItem(Index: Integer): TMatch;
begin
end;

{ TMatchCollectionEnumerator }

function TMatchCollectionEnumerator.GetCurrent: TMatch;
begin
end;

function TMatchCollectionEnumerator.MoveNext: Boolean;
begin
end;

{ TRegEx }

constructor TRegEx.Create(const Pattern: String);
begin
end;

constructor TRegEx.Create(const Pattern: String; Options: TRegExOptions); overload;
begin
end;

function TRegEx.IsMatch(const Input: String): Boolean;
begin
end;

function TRegEx.IsMatch(const Input: String; StartPos: Integer): Boolean;
begin
end;

class function TRegEx.IsMatch(const Input, Pattern: String): Boolean;
begin
end;

class function TRegEx.IsMatch(const Input, Pattern: String; Options: TRegExOptions): Boolean;
begin
end;

class function TRegEx.Escape(const Str: String; UseWildCards: Boolean): String;
begin
end;

function TRegEx.Match(const Input: String): TMatch;
begin
end;

function TRegEx.Match(const Input: String; StartPos: Integer): TMatch; overload;
begin
end;

function TRegEx.Match(const Input: String; StartPos, Length: Integer): TMatch; overload;
begin
end;

class function TRegEx.Match(const Input, Pattern: String): TMatch;
begin
end;

class function TRegEx.Match(const Input, Pattern: String; Options: TRegExOptions): TMatch;
begin
end;

function TRegEx.Matches(const Input: String): TMatchCollection;
begin
end;

function TRegEx.Matches(const Input: String; StartPos: Integer): TMatchCollection;
begin
end;

class function TRegEx.Matches(const Input, Pattern: String): TMatchCollection;
begin
end;

class function TRegEx.Matches(const Input, Pattern: String; Options: TRegExOptions): TMatchCollection;
begin
end;

function TRegEx.Replace(const Input, Replacement: String): String;
begin
end;

function TRegEx.Replace(const Input: String; Evaluator: TMatchEvaluator): String; overload;
begin
end;

class function TRegEx.Replace(const Input, Pattern, Replacement: String): String;
begin
end;

class function TRegEx.Replace(const Input, Pattern: String; Evaluator: TMatchEvaluator): String;
begin
end;

class function TRegEx.Replace(const Input, Pattern, Replacement: String; Options: TRegExOptions): String;
begin
end;

class function TRegEx.Replace(const Input, Pattern: String; Evaluator: TMatchEvaluator; Options: TRegExOptions): String;
begin
end;

function TRegEx.Split(const Input: String): TStringDynArray;
begin
end;

class function TRegEx.Split(const Input, Pattern: String): TStringDynArray;
begin
end;

class function TRegEx.Split(const Input, Pattern: String; Options: TRegExOptions): TStringDynArray;
begin
end;

end.
