{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ThreeJsControls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Types, System.SysUtils, Rtti, Vcl.Controls, Vcl.Forms,
  Winapi.Messages, Winapi.Windows, Vcl.Graphics, WEBLib.Controls,
  Web, libthreejs, js, JSDelphisystem;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Types, SysUtils, Controls, Forms, Messages,
  Graphics, Web, libthreejs, js, JSDelphisystem, WEBLib.Controls;
  {$ENDIF}

const
  // Use this prefix on object names that you don't want to
  // clear when the scene is cleared
  BuiltinPrefix: string = 'builtin-';

type

  TThreeJsTextStyle = class
  private
    FFontFace: string;
    FFontHeight: double;
    FIsBold: boolean;
    FIsTransparentFill: boolean;
    FTextColor: TColor;
    FFillColor: TColor;
    FBorderWidth: integer;
    FBorderColor: TColor;
  public
    property FontFace: string read FFontFace write FFontFace;
    property FontHeight: double read FFontHeight write FFontHeight;
    property IsBold: boolean read FIsBold write FIsBold;
    property IsTransparentFill: boolean read FIsTransparentFill write FIsTransparentFill;
    property TextColor: TColor read FTextColor write FTextColor;
    property FillColor: TColor read FFillColor write FFillColor;
    property BorderWidth: integer read FBorderWidth write FBorderWidth;
    property BorderColor: TColor read FBorderColor write FBorderColor;
  end;

  TThreeJsCustomControl = class(TWebCustomControl)
    constructor Create(AOwner: TComponent); override;
  private
    FRenderer: TThreeJsRenderer;
    FScene: TThreeJsScene;
    FCamera: TThreeJsCamera;

    FAmbientLight: TThreeJsAmbientLight;
    FSpotLight: TThreeJsSpotLight;
    FDefaultAmbientLightColor: TColor;
    FDefaultAmbientLightIntensity: double;
    FDefaultSpotLightColor: TColor;
    FDefaultSpotLightIntensity: double;

    FOrbitControls: TThreeJsOrbitControls;
    FAxes: TThreeJsAxes;
    FCenterPointTarget: TThreeJsVector3;
    FInitialCameraPosition: TThreeJsVector3;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
  public
    procedure SaveState;
    procedure ResetState;
    procedure ResetCameraPosition;
    procedure ResetSceneAndCamera;

    function GetObjectByName(aname: string): TThreeJsObject3D;
    procedure RemoveObject(anObject: TThreeJsObject3D); overload;
    procedure RemoveObject(aname: string); overload;
    procedure RemoveObjectFromCamera(aname: string);

    function AddAxes(aLength: double): TThreeJsObject3D;

    function AddChartGridXY(aWidth, aHeight: double;
      numDivisionsHorizontal, numDivisionsVertical: integer;
      aGridColor: TColor=$ffffff; aLineColor: TColor=$000000;
        isThickLines:boolean=false; isTransparent:boolean=true): TThreeJsObject3D;

    function AddChartGridYZ(aWidth, aHeight: double;
      numDivisionsHorizontal, numDivisionsVertical: integer;
      aGridColor: TColor=$ffffff; aLineColor: TColor=$000000;
        isThickLines:boolean=false; isTransparent:boolean=true): TThreeJsObject3D;


    function AddPlane(
                aWidth: double; aHeight: double;
                aColor: TColor=$ffffff
              ): TThreeJsObject3D;
    function AddFloorPlane(
                xWidth: double; zWidth: double;
                aColor: TColor=$E6D8AD
              ): TThreeJsObject3D;

    function AddLabelZ(aText: string; aZValue: double; floorPlaneWidth: double; isFixed: boolean=false;
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        textColor: TColor=$000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $eeeeee;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD;
        marginPixels: double = 24;
        radiusPixels: double = 6
        ): TThreeJsObject3D;
    function AddLabelX(aText: string; anXValue: double; floorPlaneDepth: double;
        isFixed: boolean=false;
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        textColor: TColor = $000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $eeeeee;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD;
        marginPixels: double = 24;
        radiusPixels: double = 6
        ): TThreeJsObject3D;
    function AddLabelY(aText: string; aYValue: double; floorPlaneDepth: double; isFixed: boolean=false;
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        textColor: TColor = $000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $eeeeee;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD;
        marginPixels: double = 24;
        radiusPixels: double = 6
        ): TThreeJsObject3D;
    function addText(aText: string;
        X, Y, Z: double;
        isFixed: boolean=false;
        isDrawTopmost: boolean=false;
        // alignment of whole sprite with respect to x , y, z point
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        // alignment of internal text in case of Multi-line text
        halignMultiLineText: TTXHorizontalAlign = txhLeft;
        textColor: TColor = $000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $ffffff;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD): TThreeJsObject3D;

    function AddBar(width, height, depth: double;
                aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6
              ): TThreeJsObject3D;
    function AddCube(
                length: double; aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6
              ): TThreeJsObject3D;
    function AddSphere(radiusPixels: Double;
                aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6;
                widthSegments: integer=32;
                heightSegments: integer=32
              ): TThreeJsObject3D;
    function AddCylinder(radiusTop, radisBottom, height: Double;
                aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6;
                radialSegments: integer=32
              ): TThreeJsObject3D;

    function AddLine(points: TJSArray;
         aColor: TColor; aLineWidth: double;
         castShadow: boolean = true;
         isTransparent: boolean = false;
         opacity: double=0.6
         ): TThreeJsObject3D;

    function Add2dShape(
                geo: TThreeJsGeometry; aColor: TColor;
                aWidth: double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6
              ): TThreeJsObject3D;
    // If you want lines without shadow and click features
    // such as on our chart walls, use this.
    function Add2dLine(aGeometry: TThreeJsGeometry;
         aColor: TColor; aLineWidth: double;
         isTransparent: boolean = false;
         opacity: double=0.6
         ): TThreeJsObject3D;

    procedure SetTargetViewVector(aVector: TThreeJsVector3);
    procedure RotateLeft(deltaAngleDeg: double);
    procedure RotateUp(deltaAngleDeg: double);
    procedure ZoomIn(deltaScale: double);
    procedure PanLeft;
    procedure PanRight;
    procedure PanUp;
    procedure PanDown;
    procedure DumpOrbitControlsPosition;
    procedure ShowDebugArrow(origin, terminus: TThreeJsVector3;
                        aColor: TColor=$0000ff; isShow: boolean=true);

    property Renderer: TThreeJsRenderer read FRenderer;
    property Scene: TThreeJsScene read FScene;
    property Camera: TThreeJsCamera read FCamera;

    property SpotLight: TThreeJsSpotLight read FSpotLight;
    property AmbientLight: TThreeJsAmbientLight read FAmbientLight;
    property DefaultAmbientLightColor: TColor read FDefaultAmbientLightColor write FDefaultAmbientLightColor;
    property DefaultAmbientLightIntensity: double read FDefaultAmbientLightIntensity write FDefaultAmbientLightIntensity;
    property DefaultSpotLightColor: TColor read FDefaultSpotLightColor write FDefaultSpotLightColor;
    property DefaultSpotLightIntensity: double read FDefaultSpotLightIntensity write FDefaultSpotLightIntensity;

    property OrbitControls: TThreeJsOrbitControls read FOrbitControls;
    property Axes: TThreeJsAxes read FAxes;
    property CenterPointTarget: TThreeJsVector3 read FCenterPointTarget;
    property InitialCameraPosition: TThreeJsVector3 read FInitialCameraPosition write FInitialCameraPosition;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;

  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Height;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property Top;
    property Visible;
    property Width;
    property TabStop;
  end;

  TThreeJsLoadObjectEvent = procedure(Sender: TObject; success: boolean; anObject: TThreeJsObject3D; errorMsg: string) of object;
  TThreeJsObjectEvent = procedure(Sender: TObject;
        frontObject: TThreeJsObject3D; intersection: TThreeJsIntersection) of object;
  TThreeJsObjectNotifyEvent = procedure(Sender: TObject; frontObject: TThreeJsObject3D) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebThreeJsPaintBox = class(TThreeJsCustomControl)
  private
    FAnnotationTextStyle: TThreeJsTextStyle;

    FOnItemClick: TThreeJsObjectEvent;
    FOnItemExit: TThreeJsObjectNotifyEvent;
    FOnItemDblClick: TThreeJsObjectNotifyEvent;
    FOnItemMouseMove: TThreeJsObjectEvent;
    FOnItemMouseEnter: TThreeJsObjectNotifyEvent;
    FOnItemMouseLeave: TThreeJsObjectNotifyEvent;
  public
    function ShowAnnotation(aname: string; x, y, z: double; aText: string;
        targetObject: TThreeJsObject3D = nil;
        forcedProportionalHeight: double = 0;
        // alignment of whole sprite with respect to x , y, z point
        valign: TTXVerticalAlign = txvBottom;
        halign: TTXHorizontalAlign = txhCenter;
        // alignment of internal text in case of Multi-line text
        halignMultiLineText: TTXHorizontalAlign = txhCenter
       ): TThreeJsObject3D;

    property AnnotationTextStyle: TThreeJsTextStyle read FAnnotationTextStyle write FAnnotationTextStyle;
  published	
    property OnClick;
    property OnDblClick;
    property OnItemClick: TThreeJsObjectEvent read FOnItemClick write FOnItemClick;
    property OnItemExit: TThreeJsObjectNotifyEvent read FOnItemExit write FOnItemExit;
    property OnItemDblClick: TThreeJsObjectNotifyEvent read FOnItemDblClick write FOnItemDblClick;
    property OnItemMouseMove: TThreeJsObjectEvent read FOnItemMouseMove write FOnItemMouseMove;
    property OnItemMouseEnter: TThreeJsObjectNotifyEvent read FOnItemMouseEnter write FOnItemMouseEnter;
    property OnItemMouseLeave: TThreeJsObjectNotifyEvent read FOnItemMouseLeave write FOnItemMouseLeave;
  end;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebThreeJsModelBox = class(TWebThreeJsPaintBox)
  private
    FDefaultLightColor: TColor;
    FDefaultLightIntensity: double;
    FUseGammaCorrection: boolean;
    FAdditionalLights: TJSArray;

    FOnObjectLoad: TThreeJsLoadObjectEvent;
  public
    procedure AddObjectMtl(aName: string;
         objFile, objPath, mtlFile, mtlPath, texturePath: string;
         positionX: double=0; positionY: double=0; positionZ: Double=0);
    procedure AddObjectGltf(aName: string;
         gltfFile, gltfPath: string;
         positionX: double=0; positionY: double=0; positionZ: Double=0);
    // useful for above addObject functions
    procedure BringObjectInFullView(anObject: TThreeJsObject3d);
    procedure AddLights;
    procedure RemoveLights;
    procedure ExportSceneGltf(gltfFileName: string);

    property DefaultLightColor: TColor read FDefaultLightColor write FDefaultLightColor;
    property DefaultLightIntensity: double read FDefaultLightIntensity write FDefaultLightIntensity;
    property UseGammaCorrection: boolean read FUseGammaCorrection write FUseGammaCorrection;
    property AdditionalLights: TJSArray read FAdditionalLights;
  published
    property OnObjectLoad: TThreeJsLoadObjectEvent read FOnObjectLoad write FOnObjectLoad;
  end;

implementation


constructor TThreeJsCustomControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 400;
  Height := 400;
end;

procedure TThreeJsCustomControl.saveState;
begin
end;

procedure TThreeJsCustomControl.resetState;
begin
end;

procedure TThreeJsCustomControl.resetCameraPosition;
begin
end;

function TThreeJsCustomControl.getObjectByName(aname: string): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TThreeJsCustomControl.removeObject(anObject: TThreeJsObject3D);
begin
end;

procedure TThreeJsCustomControl.removeObject(aname: string);
begin
end;

procedure TThreeJsCustomControl.removeObjectFromCamera(aname: string);
begin
end;

function TThreeJsCustomControl.addAxes(aLength: double): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TThreeJsCustomControl.showDebugArrow(origin, terminus: TThreeJsVector3;
                  aColor: TColor=$0000ff; isShow: boolean=true);
begin
end;

function TThreeJsCustomControl.addChartGridXY(aWidth, aHeight: double;
  numDivisionsHorizontal, numDivisionsVertical: integer;
  aGridColor: TColor=$ffffff; aLineColor: TColor=$000000;
    isThickLines:boolean=false; isTransparent:boolean=true): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addChartGridYZ(aWidth, aHeight: double;
  numDivisionsHorizontal, numDivisionsVertical: integer;
  aGridColor: TColor=$ffffff; aLineColor: TColor=$000000;
    isThickLines:boolean=false; isTransparent:boolean=true): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addPlane(
            aWidth: double; aHeight: double;
            aColor: TColor=$ffffff
          ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addFloorPlane(
            xWidth: double; zWidth: double;
            aColor: TColor=$E6D8AD
          ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addLabelZ(aText: string; aZValue: double; floorPlaneWidth: double; isFixed: boolean=false;
    valign: TTXVerticalAlign = txvCenter;
    halign: TTXHorizontalAlign = txhLeft;
    textColor: TColor=$000000;
    fontFace: string = 'Arial';
    fontHeight: double = 1;
    isBold: boolean = true;
    isTransparentFill: boolean = true;
    fillColor: TColor = $eeeeee;
    borderWidth: double = 0;
    borderColor: TColor = $E6D8AD;
    marginPixels: double = 24;
    radiusPixels: double = 6
    ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addLabelX(aText: string; anXValue: double; floorPlaneDepth: double;
    isFixed: boolean=false;
    valign: TTXVerticalAlign = txvCenter;
    halign: TTXHorizontalAlign = txhLeft;
    textColor: TColor = $000000;
    fontFace: string = 'Arial';
    fontHeight: double = 1;
    isBold: boolean = true;
    isTransparentFill: boolean = true;
    fillColor: TColor = $eeeeee;
    borderWidth: double = 0;
    borderColor: TColor = $E6D8AD;
    marginPixels: double = 24;
    radiusPixels: double = 6
    ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addLabelY(aText: string; aYValue: double; floorPlaneDepth: double; isFixed: boolean=false;
    valign: TTXVerticalAlign = txvCenter;
    halign: TTXHorizontalAlign = txhLeft;
    textColor: TColor = $000000;
    fontFace: string = 'Arial';
    fontHeight: double = 1;
    isBold: boolean = true;
    isTransparentFill: boolean = true;
    fillColor: TColor = $eeeeee;
    borderWidth: double = 0;
    borderColor: TColor = $E6D8AD;
    marginPixels: double = 24;
    radiusPixels: double = 6
    ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addText(aText: string;
    X, Y, Z: double;
    isFixed: boolean=false;
    isDrawTopmost: boolean=false;
    // alignment of whole sprite with respect to x , y, z point
    valign: TTXVerticalAlign = txvCenter;
    halign: TTXHorizontalAlign = txhLeft;
    // alignment of internal text in case of Multi-line text
    halignMultiLineText: TTXHorizontalAlign = txhLeft;
    textColor: TColor = $000000;
    fontFace: string = 'Arial';
    fontHeight: double = 1;
    isBold: boolean = true;
    isTransparentFill: boolean = true;
    fillColor: TColor = $ffffff;
    borderWidth: double = 0;
    borderColor: TColor = $E6D8AD): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addBar(width, height, depth: double;
            aColor: TColor;
            positionX, positionY, positionZ: Double;
            castShadow: boolean = true;
            isTransparent: boolean = false;
            opacity: double=0.6
          ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addCube(
                length: double; aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6
              ): TThreeJsObject3D;

begin
  result := nil;
end;

function TThreeJsCustomControl.addSphere(radiusPixels: Double;
            aColor: TColor;
            positionX, positionY, positionZ: Double;
            castShadow: boolean = true;
            isTransparent: boolean = false;
            opacity: double=0.6;
            widthSegments: integer=32;
            heightSegments: integer=32
          ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addCylinder(radiusTop, radisBottom, height: Double;
            aColor: TColor;
            positionX, positionY, positionZ: Double;
            castShadow: boolean = true;
            isTransparent: boolean = false;
            opacity: double=0.6;
            radialSegments: integer=32
          ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.addLine(points: TJSArray;
     aColor: TColor; aLineWidth: double;
     castShadow: boolean = true;
     isTransparent: boolean = false;
     opacity: double=0.6
     ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.add2dShape(
            geo: TThreeJsGeometry; aColor: TColor;
            aWidth: double;
            castShadow: boolean = true;
            isTransparent: boolean = false;
            opacity: double=0.6
          ): TThreeJsObject3D;
begin
  result := nil;
end;

function TThreeJsCustomControl.add2dLine(aGeometry: TThreeJsGeometry;
     aColor: TColor; aLineWidth: double;
     isTransparent: boolean = false;
     opacity: double=0.6
     ): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TThreeJsCustomControl.setTargetViewVector(aVector: TThreeJsVector3);
begin
end;

procedure TThreeJsCustomControl.rotateLeft(deltaAngleDeg: double);
begin
end;

procedure TThreeJsCustomControl.rotateUp(deltaAngleDeg: double);
begin
end;

procedure TThreeJsCustomControl.zoomIn(deltaScale: double);
begin
end;

procedure TThreeJsCustomControl.panLeft;
begin
end;

procedure TThreeJsCustomControl.panRight;
begin
end;

procedure TThreeJsCustomControl.panUp;
begin
end;

procedure TThreeJsCustomControl.panDown;
begin
end;

procedure TThreeJsCustomControl.dumpOrbitControlsPosition;
begin
end;

procedure TThreeJsCustomControl.ResetSceneAndCamera;
begin
end;

function TWebThreeJsPaintBox.showAnnotation(aname: string; x, y, z: double; aText: string;
    targetObject: TThreeJsObject3D = nil;
    forcedProportionalHeight: double = 0;
    // alignment of whole sprite with respect to x , y, z point
    valign: TTXVerticalAlign = txvBottom;
    halign: TTXHorizontalAlign = txhCenter;
    // alignment of internal text in case of Multi-line text
    halignMultiLineText: TTXHorizontalAlign = txhCenter
   ): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TWebThreeJsModelBox.addObjectMtl(aName: string;
     objFile, objPath, mtlFile, mtlPath, texturePath: string;
     positionX: double=0; positionY: double=0; positionZ: Double=0);
begin
end;

procedure TWebThreeJsModelBox.addObjectGltf(aName: string;
     gltfFile, gltfPath: string;
     positionX: double=0; positionY: double=0; positionZ: Double=0);
// useful for above addObject functions
begin
end;

procedure TWebThreeJsModelBox.BringObjectInFullView(anObject: TThreeJsObject3d);
begin
end;

procedure TWebThreeJsModelBox.AddLights;
begin
end;

procedure TWebThreeJsModelBox.RemoveLights;
begin
end;

procedure TWebThreeJsModelBox.exportSceneGltf(gltfFileName: string);
begin
end;


end.
