{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ThreeJsMathChart;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Types, System.SysUtils, Rtti, Vcl.Controls, Vcl.Forms,
  Winapi.Messages, Winapi.Windows, Vcl.Graphics, WEBLib.Controls,
  Web, libthreejs, js, WEBLib.ThreeJsControls, JSDelphisystem,
  WEBLib.ThreeJsChart;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Types, SysUtils, Controls, Forms, Messages,
  Graphics, Web, js, WEBLib.ThreeJsControls, WEBLib.ThreeJsChart,
  JSDelphisystem, libthreejs;
  {$ENDIF}

type
  TThreeJsMathChartType = (ctScatterChart, ctSurfaceChart);
  TThreeJsMathPointShape = (psSphere, psCube);

  TThreeJsSurfaceCallBack = function(x, y: double): double of object;

  TThreeJsMathChartSeries = class
  private
    FChartType: TThreeJsMathChartType;

    FValueFormatFloat: string; //global
    // overrides
    FXAxisValueFormat, FYAxisValueFormat, FZAxisValueFormat: string;

    // Chart Point properties
    FPointTextStyle: TThreeJsTextStyle;
    FDefaultPointSize: double;
    FDefaultPointColor: TColor;

    // Titles
    FTitle: string;
    FTitleStyle: TThreeJsTextStyle;
    FTitleSpace: double;
    FAxisTitleStyle: TThreeJsTextStyle;
    FXAxisTitle: string;
    FXAxisTitleSpace: double;
    FYAxisTitle: string;
    FYAxisTitleSpace: double;
    FZAxisTitle: string;
    FZAxisTitleSpace: double;

    // Axis properties
    FXAxisLength, FYAxisLength, FZAxisLength: double;
    FAutoScale: boolean;
    FXAxisMarkMaximum, FYAxisMarkMaximum, FZAxisMarkMaximum: double;
    FXAxisMarkMinimum, FYAxisMarkMinimum, FZAxisMarkMinimum: double;
    FXAxisMarkStep, FYAxisMarkStep, FZAxisMarkStep: double;
    FXAxisLabelStyle: TThreeJsTextStyle;
    FYAxisLabelStyle: TThreeJsTextStyle;
    FZAxisLabelStyle: TThreeJsTextStyle;
    FAxisLabelSpace: double;
    FSuppressZStartingLabel: boolean; //see meaning in property def
    FNumDivisionsSurface: integer;
  public
    constructor create;

    procedure AddData(x, y, z: double; aPointShape: TThreeJsMathPointShape; userdata: JSValue; aPointSize: double; aPointColor: TColor);
    procedure GetDataFromObject(obj: TJSObject; out x: double; out y: double; out z: double;
                     out aPointShape: TThreeJsMathPointShape; out aPointSize: double; out aPointColor: TColor; out aUserData: JSValue);
    procedure AddParametricSurface(xMin, xMax, yMin, yMax: double; numDivisions: integer; surfaceCallbackFunction: TThreeJsSurfaceCallBack);
    procedure SetSurfaceColors(colors: array of JSValue);
    procedure ResetSurfaceColors;
    function GetValueStr(aValue: double): string;

    property ChartType: TThreeJsMathChartType read FChartType write FChartType;

    // Resolution is nothing but the number of divisions
    property Resolution: integer read FNumDivisionsSurface write FNumDivisionsSurface;

    // Format string for FormatFloat function
    // To display value in all places
    property ValueFormatFloat: string read FValueFormatFloat write FValueFormatFloat;

    // Optional overrides, To display different value format on axis
    property XAxisValueFormat: string read FXAxisValueFormat write FXAxisValueFormat;
    property YAxisValueFormat: string read FYAxisValueFormat write FYAxisValueFormat;
    property ZAxisValueFormat: string read FZAxisValueFormat write FZAxisValueFormat;

    // properties of a Point Item
    property PointTextStyle: TThreeJsTextStyle read FPointTextStyle write FPointTextStyle;
    property DefaultPointSize: double read FDefaultPointSize write FDefaultPointSize;
    property DefaultPointColor: TColor read FDefaultPointColor write FDefaultPointColor;

    // Chart Title
    property Title: string read FTitle write FTitle;
    property TitleStyle: TThreeJsTextStyle read FTitleStyle write FTitleStyle;
    property TitleSpace: double read FTitleSpace write FTitleSpace;

    // Axis properties
    // --  Axis Titles
    property AxisTitleStyle: TThreeJsTextStyle read FAxisTitleStyle write FAxisTitleStyle;
    property XAxisTitle: string read FXAxisTitle write FXAxisTitle;
    property XAxisTitleSpace: double read FXAxisTitleSpace write FXAxisTitleSpace;
    property YAxisTitle: string read FYAxisTitle write FYAxisTitle;
    property YAxisTitleSpace: double read FYAxisTitleSpace write FYAxisTitleSpace;
    property ZAxisTitle: string read FZAxisTitle write FZAxisTitle;
    property ZAxisTitleSpace: double read FZAxisTitleSpace write FZAxisTitleSpace;

    //  -- Axis Lengths
    property XAxisLength: double read FXAxisLength write FXAxisLength;
    property YAxisLength: double read FYAxisLength write FYAxisLength;
    property ZAxisLength: double read FZAxisLength write FZAxisLength;

    // -- Axis Marks (range of values used for labeling axes)
    // If Auto flag is set, the following properties are determined
    // automatically from data
    property AutoScale: boolean read FAutoScale write FAutoScale;
    // Use the following properties only in non-Auto mode
    property XAxisMarkMaximum: double read FXAxisMarkMaximum write FXAxisMarkMaximum;
    property XAxisMarkMinimum: double read FXAxisMarkMinimum write FXAxisMarkMinimum;
    property XAxisMarkStep: double read FXAxisMarkStep write FXAxisMarkStep;
    property YAxisMarkMaximum: double read FYAxisMarkMaximum write FYAxisMarkMaximum;
    property YAxisMarkMinimum: double read FYAxisMarkMinimum write FYAxisMarkMinimum;
    property YAxisMarkStep: double read FYAxisMarkStep write FYAxisMarkStep;
    property ZAxisMarkMaximum: double read FZAxisMarkMaximum write FZAxisMarkMaximum;
    property ZAxisMarkMinimum: double read FZAxisMarkMinimum write FZAxisMarkMinimum;
    property ZAxisMarkStep: double read FZAxisMarkStep write FZAxisMarkStep;

    // -- Axis label styles
    property XAxisLabelStyle: TThreeJsTextStyle read FXAxisLabelStyle write FXAxisLabelStyle;
    property YAxisLabelStyle: TThreeJsTextStyle read FYAxisLabelStyle write FYAxisLabelStyle;
    property ZAxisLabelStyle: TThreeJsTextStyle read FZAxisLabelStyle write FZAxisLabelStyle;
    property AxisLabelSpace: double read FAxisLabelSpace write FAxisLabelSpace;

    // In math charts, displaying both Z and X starting labels make them clash
    // as they are almost at the same position. This flag, by default ON,
    // suppresses the starting Z label to avoid the clash.
    property SuppressZStartingLabel: boolean read FSuppressZStartingLabel write FSuppressZStartingLabel;
  end;

  TThreeJsPointItemEventArgs = class
  private
    FItem: TThreeJsObject3D;
    FX: double;
    FY: double;
    FZ: double;
    FValueStr: string;
  public
    property Item: TThreeJsObject3D read FItem write FItem;
    property X: double read FX write FX;
    property Y: double read FY write FY;
    property Z: double read FZ write FZ;
    property ValueStr: string read FValueStr write FValueStr;
  end;

  TThreeJsPointEvent = procedure(Sender: TObject; Args: TThreeJsPointItemEventArgs) of object;
  TThreeJsPointNotifyEvent = procedure(Sender: TObject; aPointItem: TThreeJsObject3D) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebThreeJsMathChart = class(TThreeJsCustomChart)
 private
    FSeries: TThreeJsMathChartSeries;
    FShowWireframe: boolean;
    FIsWireframeTexture: boolean;
    FCustomTextureUrl: string;

    FOnItemClick: TThreeJsPointEvent;
    FOnItemExit: TThreeJsPointNotifyEvent;
    FOnItemDblClick: TThreeJsPointNotifyEvent;
    FOnItemMouseMove: TThreeJsPointEvent;
    FOnItemMouseEnter: TThreeJsPointNotifyEvent;
    FOnItemMouseLeave: TThreeJsPointNotifyEvent;
  protected
    function GetGlowChartItem(chartItem: TThreeJsObject3D): boolean;
    procedure SetGlowChartItem(chartItem: TThreeJsObject3D; avalue: boolean);

  public
    procedure CreateChartFromData;
    procedure ClearPointLabels;

    function GetItemObject(x, y, z: double): TThreeJsObject3D;
    function ShowPointValue(x, y, z: double; aValueStr: string): TThreeJsObject3D;
    procedure RemoveChartValue(aChartItem: TThreeJsObject3D);
    function ShowSurfacePointValue(x, y, z: double; aValueStr: string): TThreeJsObject3D;
    procedure RemoveSurfacePointValue;

    property Series: TThreeJsMathChartSeries read FSeries write FSeries;

    property Glow[chartItem: TThreeJsObject3D]: boolean read getGlowChartItem write setGlowChartItem;
    property ShowWireframe: boolean read FShowWireframe write FShowWireframe;
    property IsWireframeTexture: boolean read FIsWireframeTexture write FIsWireframeTexture;
    property CustomTextureUrl: string read FCustomTextureUrl write FCustomTextureUrl;

  published
    property OnClick;
    property OnDblClick;
    property OnItemClick: TThreeJsPointEvent read FOnItemClick write FOnItemClick;
    property OnItemExit: TThreeJsPointNotifyEvent read FOnItemExit write FOnItemExit;
    property OnItemDblClick: TThreeJsPointNotifyEvent read FOnItemDblClick write FOnItemDblClick;
    property OnItemMouseMove: TThreeJsPointEvent read FOnItemMouseMove write FOnItemMouseMove;

    property OnItemMouseEnter: TThreeJsPointNotifyEvent read FOnItemMouseEnter write FOnItemMouseEnter;
    property OnItemMouseLeave: TThreeJsPointNotifyEvent read FOnItemMouseLeave write FOnItemMouseLeave;
  end;

implementation

constructor TThreeJsMathChartSeries.create;
begin
end;

procedure TThreeJsMathChartSeries.addData(x, y, z: double; aPointShape: TThreeJsMathPointShape; userdata: JSValue; aPointSize: double; aPointColor: TColor);
begin
end;

procedure TThreeJsMathChartSeries.getDataFromObject(obj: TJSObject; out x: double; out y: double; out z: double;
                   out aPointShape: TThreeJsMathPointShape; out aPointSize: double; out aPointColor: TColor; out aUserData: JSValue);
begin
end;

procedure TThreeJsMathChartSeries.addParametricSurface(xMin, xMax, yMin, yMax: double; numDivisions: integer; surfaceCallbackFunction: TThreeJsSurfaceCallBack);
begin
end;

procedure TThreeJsMathChartSeries.setSurfaceColors(colors: array of JSValue);
begin
end;

procedure TThreeJsMathChartSeries.resetSurfaceColors;
begin
end;

function TThreeJsMathChartSeries.getValueStr(aValue: double): string;
begin
  result := '';
end;

function TWebThreeJsMathChart.getGlowChartItem(chartItem: TThreeJsObject3D): boolean;
begin
  result := false;
end;

procedure TWebThreeJsMathChart.setGlowChartItem(chartItem: TThreeJsObject3D; avalue: boolean);
begin
end;

procedure TWebThreeJsMathChart.createChartFromData;
begin
end;

procedure TWebThreeJsMathChart.clearPointLabels;
begin
end;

function TWebThreeJsMathChart.getItemObject(x, y, z: double): TThreeJsObject3D;
begin
  result := nil;
end;

function TWebThreeJsMathChart.showPointValue(x, y, z: double; aValueStr: string): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TWebThreeJsMathChart.removeChartValue(aChartItem: TThreeJsObject3D);
begin
end;

function TWebThreeJsMathChart.showSurfacePointValue(x, y, z: double; aValueStr: string): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TWebThreeJsMathChart.removeSurfacePointValue;
begin
end;

end.
