{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.WebCtrls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, VCL.Forms, VCL.Graphics, WEBLib.Controls, WEBLib.StdCtrls, System.SysUtils,
  VCL.Controls, Web, JS, System.TypInfo, Dialogs, Types, Windows;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Web, JS,
  LCLIntf, LCLType, Forms, Controls, Graphics, WEBLib.Controls, WEBLib.StdCtrls;
  {$ENDIF}

type
  TBrowserSandboxType = (stAllowForms, stAllowModals, stAllowOrientationLock, stAllowPointerLock, stAllowPopups, stAllowPopupsToEscapeSandbox,
    stAllowPresentation, stAllowSameOrigin, stAllowScripts, stAllowTopNavigation, stAllowTopNavigationByUserActivation);

  TBrowserSandboxTypes = set of TBrowserSandboxType;

  TBrowserReferrerPolicy = (rfNone, rfNoReferrer, rfNoReferrerWhenDowngrade, rfOrigin, rfOriginWhenCrossOrigin, rfUnsafeUrl);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebBrowserControl = class(TWebCustomControl)
  private
    FURL: string;
    FSandbox: TBrowserSandboxTypes;
    FReferrerPolicy: TBrowserReferrerPolicy;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Navigate(const AURL: string);
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BorderStyle default bsSingle;
    property ChildOrder;
    property ReferrerPolicy: TBrowserReferrerPolicy read FReferrerPolicy write FReferrerPolicy default rfNone;
    property Sandbox: TBrowserSandboxTypes read FSandbox write FSandbox;
    property URL: string read FURL write FURL;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebYoutube = class(TWebCustomControl)
  private
    FVideoID: string;
    FAllowFullScreen: boolean;
    FAutoPlay: boolean;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property AllowFullScreen: boolean read FAllowFullScreen write FAllowFullScreen;
    property Anchors;
    property AutoPlay: boolean read FAutoPlay write FAutoPlay;
    property ChildOrder;
    property VideoID: string read FVideoID write FVideoID;
    property Visible;
  end;

  TMapClickEvent = procedure(Sender: TObject; Lon, Lat: double) of object;
  TMapZoomEvent = procedure(Sender: TObject; ZoomLevel: integer) of object;
  TMapMarkerClickEvent = procedure(Sender: TObject; AIndex: integer; AMarker: TJSObjectRecord) of object;
  TMapPolygonClickEvent = procedure(Sender: TObject; AIndex: integer; APolygon: TJSObjectRecord) of object;
  TMapPolylineClickEvent = procedure(Sender: TObject; AIndex: integer; APolyline: TJSObjectRecord) of object;
  TMapCircleClickEvent = procedure(Sender: TObject; AIndex: integer; ACircle: TJSObjectRecord) of object;
  TMapRectangleClickEvent = procedure(Sender: TObject; AIndex: integer; ARectangle: TJSObjectRecord) of object;
  TMapKMLClickEvent = procedure(Sender: TObject; AIndex: integer; AKML: TJSObjectRecord) of object;

  TGoogleMarkerColor = (mcDefault, mcRed, mcBlue, mcGreen, mcPurple, mcYellow);
  TGoogleMarkerShape = (msPin, msPinDot, msFlag, msBookmark, msFlagSmall, msHome, msFavorite, msStar);
  TGoogleTravelMode = (tmDriving, tmWalking, tmBicycling, tmTransit);
  TGoogleMapType = (mtDefault, mtSatellite, mtHybrid, mtTerrain);
  TGoogleMapStyle = (mstDefault, mstNightMode, mstCustom);

  TWebGoogleMaps = class;

  TGoogleMapsOptions = class(TPersistent)
  private
    FOwner: TWebGoogleMaps;
    FCustomStyle: TStringList;
    FMapStyle: TGoogleMapStyle;
    FDefaultLongitude: Double;
    FDefaultLatitude: Double;
    FDefaultZoomLevel: Integer;
    procedure SetCustomStyle(const Value: TStringList);
  public
    constructor Create(AGoogleMaps: TWebGoogleMaps);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property CustomStyle: TStringList read FCustomStyle write SetCustomStyle;
    property MapStyle: TGoogleMapStyle read FMapStyle write FMapStyle default mstDefault;
    property DefaultLatitude: Double read FDefaultLatitude write FDefaultLatitude;
    property DefaultLongitude: Double read FDefaultLongitude write FDefaultLongitude;
    property DefaultZoomLevel: Integer read FDefaultZoomLevel write FDefaultZoomLevel default 8;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebGoogleMaps = class(TWebCustomControl)
  private
    FAPIKey:string;
    FOnMapClick: TMapClickEvent;
    FOnMapDblClick: TMapClickEvent;
    FOnMapZoom: TMapZoomEvent;
    FOnMapPan: TMapClickEvent;
    FOnGeoCoded: TMapClickEvent;
    FOnMapLoaded: TNotifyEvent;
    FOnMapIdle: TNotifyEvent;
    FOnMarkerClick: TMapMarkerClickEvent;
    FOnPolygonClick: TMapPolygonClickEvent;
    FOnPolylineClick: TMapPolylineClickEvent;
    FOnRectangleClick: TMapRectangleClickEvent;
    FOnCircleClick: TMapCircleClickEvent;
    FOnKMLClick: TMapKMLClickEvent;
    FOptions: TGoogleMapsOptions;
    function GetCircle(AIndex: integer): TJSObject;
    function GetPolygon(AIndex: integer): TJSObject;
    function GetPolyline(AIndex: integer): TJSObject;
    function GetRectangle(AIndex: integer): TJSObject;
    function GetMarker(AIndex: integer): TJSObject;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetCenter(Lat, Lon: double);
    function GetCenter(var Lat, Lon: Double): Boolean;
    function GetBounds(var NorthEastLat, NorthEastLon, SouthWestLat, SouthWestLon: Double): Boolean;
    procedure SetDoubleClickZoom(AValue: Boolean);
    procedure SetDraggable(AValue: Boolean);
    procedure SetScrollWheel(AValue: Boolean);
    procedure SetMapType(AMapType: TGoogleMapType = mtDefault);
    procedure SetZoom(Zoom: integer);
    procedure PanTo(Lat, Lon: double);
    procedure AddGPX(AGPX: string; AColor: TColor = clRed; AWidth: integer = 2; AOpacity: Double = 1);
    procedure AddKML(Url: string; ZoomToBounds: Boolean = true);
    procedure AddPolyLine(Points: TJSArray; AColor: TColor = clRed; AWidth: integer = 2; AOpacity: double = 1);
    procedure AddPolygon(Points: TJSArray; AFillColor: TColor = clRed; AStrokeColor: TColor = clBlack; AWidth: integer = 2; AOpacity: double = 1);
    procedure AddCircle(Lat, Lon: double; Radius: Integer; AFillColor: TColor = clRed; AStrokeColor: TColor = clBlack; AWidth: integer = 2; AOpacity: double = 1);
    procedure AddRectangle(NorthEastLat, NorthEastLon, SouthWestLat, SouthWestLon: double; AFillColor: TColor = clRed; AStrokeColor: TColor = clBlack; AWidth: integer = 2; AOpacity: double = 1);
    procedure AddMarker(Lat,Lon: double; Title: string = ''); overload;
    procedure AddMarker(Lat,Lon: double; AColor: TGoogleMarkerColor; Title: string = ''); overload;
    procedure AddMarker(Lat,Lon: double; AColor: TColor; PinLetter: string; Title: string = ''); overload;
    procedure AddMarker(Lat,Lon: double; PinIcon: string; Title: string; XOffset: integer = 0; YOffset: integer = 0); overload;
    procedure AddMarker(Lat,Lon: double; Shape: TGoogleMarkerShape; AColor: TColor; Title: string = ''); overload;
    procedure SetMarkerTitle(AIndex: Integer; ATitle: string);
    procedure SetMarkerLocation(AIndex: Integer; Lat, Lon: Double);
    procedure SetMarkerIcon(AIndex: Integer; Url: string);
    procedure SetCircleCenter(AIndex: Integer; Lat, Lon: Double);
    procedure SetCircleRadius(AIndex, Radius: Integer);
    procedure SetCircleColors(AIndex: Integer; AFillColor, AStrokeColor: TColor);
    procedure SetRectangleLocation(AIndex: Integer; NorthEastLat, NorthEastLon, SouthWestLat, SouthWestLon: double);
    procedure SetRectangleColors(AIndex: Integer; AFillColor, AStrokeColor: TColor);
    procedure SetPolylineColor(AIndex: Integer; AColor: TColor);
    procedure SetPolylinePoints(AIndex: Integer; Points: TJSArray);
    procedure SetPolygonColors(AIndex: Integer; AFillColor, AStrokeColor: TColor);
    procedure SetPolygonPoints(AIndex: Integer; Points: TJSArray);
    procedure FitBounds(LatMin,LonMin,LatMax,LonMax: double);
    procedure ClearMarkers;
    procedure ClearKMLs;
    procedure ClearPolylines;
    procedure ClearPolygons;
    procedure ClearCircles;
    procedure ClearRectangles;
    procedure ShowDirections(Source, Destination: string; ATravelMode: TGoogleTravelMode = tmDriving; WayPoints: TStringList = nil; OptimizeWayPoints: Boolean = False; AvoidHighways: Boolean = False; AvoidTolls: Boolean = False); overload;
    procedure ShowDirections(SourceLon, SourceLat, DestLon, DestLat: double; ATravelMode: TGoogleTravelMode = tmDriving; WayPoints: TStringList = nil; OptimizeWayPoints: Boolean = False; AvoidHighways: Boolean = False; AvoidTolls: Boolean = False); overload;
    procedure ShowStreetView(Lat, Lon: Double; Heading: Integer = 0; Zoom: Integer = 0; Pitch: Integer = 0);
    procedure HideStreetView;
    procedure RemoveDirections;
    procedure RemoveMarker(AIndex: Integer);
    procedure RemovePolygon(AIndex: Integer);
    procedure RemovePolyline(AIndex: Integer);
    procedure RemoveCircle(AIndex: Integer);
    procedure RemoveRectangle(AIndex: Integer);
    procedure GeoCode(const Address: string);
    property Markers[AIndex: integer]: TJSObject read GetMarker;
    property Polygons[AIndex: integer]: TJSObject read GetPolygon;
    property Polylines[AIndex: integer]: TJSObject read GetPolyline;
    property Circles[AIndex: integer]: TJSObject read GetCircle;
    property Rectangles[AIndex: integer]: TJSObject read GetRectangle;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property APIKey: string read FAPIKey write FAPIKey;
    property ChildOrder;
    property ElementPosition;
    property Options: TGoogleMapsOptions read FOptions write FOptions;
    property Visible;

    property OnGeoCoded: TMapClickEvent read FOnGeoCoded write FOnGeoCoded;
    property OnMapClick: TMapClickEvent read FOnMapClick write FOnMapClick;
    property OnMapDblClick: TMapClickEvent read FOnMapDblClick write FOnMapDblClick;
    property OnMapIdle: TNotifyEvent read FOnMapIdle write FOnMapIdle;
    property OnMapPan: TMapClickEvent read FOnMapPan write FOnMapPan;
    property OnMapLoaded: TNotifyEvent read FOnMapLoaded write FOnMapLoaded;
    property OnMapZoom: TMapZoomEvent read FOnMapZoom write FOnMapZoom;
    property OnMarkerClick: TMapMarkerClickEvent read FOnMarkerClick write FOnMarkerClick;
    property OnPolygonClick: TMapPolygonClickEvent read FOnPolygonClick write FOnPolygonClick;
    property OnPolylineClick: TMapPolylineClickEvent read FOnPolylineClick write FOnPolylineClick;
    property OnCircleClick: TMapCircleClickEvent read FOnCircleClick write FOnCircleClick;
    property OnRectangleClick: TMapRectangleClickEvent read FOnRectangleClick write FOnRectangleClick;
    property OnKMLClick: TMapKMLClickEvent read FOnKMLClick write FOnKMLClick;
  end;

  TDriveView = (dvList, dvGrid);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebGoogleDrive = class(TWebCustomControl)
  private
    FView: TDriveView;
    FFolderID: string;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property ChildOrder;
    property FolderID: string read FFolderID write FFolderID;
    property View: TDriveView read FView write FView default dvList;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebTwitterFeed = class(TWebCustomControl)
  private
    FFeed: string;
    FFeedLinkText: string;
    procedure SetFeed(const Value: string);
  protected
    function GetDesignText: string; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property Feed: string read FFeed write SetFeed;
    property FeedLinkText: string read FFeedLinkText write FFeedLinkText;
    property Visible;
  end;

  TFilesDroppedEvent = procedure(Sender: TObject; AFileList: TStringList) of object;

  TFileAsTextEvent = procedure(Sender: TObject; AFileIndex: integer; AText: string) of object;

  TFileAsBase64Event = procedure(Sender: TObject; AFileIndex: integer; ABase64: string) of object;

  TFileAsDataURLEvent = procedure(Sender: TObject; AFileIndex: integer; AURL: string) of object;

  TFileAsArrayBufferEvent = procedure(Sender: TObject; AFileIndex: integer; ABuffer: TJSArrayBufferRecord) of object;

  TFileUploadEvent = procedure(Sender: TObject; AFileIndex: integer) of object;

  TFileUploadErrorEvent = procedure(Sender: TObject; AFileIndex: integer; AError: string) of object;

  TFileUploadProgressEvent = procedure(Sender: TObject; AFileIndex: integer; APosition, ATotalSize: longint) of object;

  TFileGetAsBase64 = procedure(Sender: TObject; ABase64: string) of object;

  TFileGetAsText = procedure(Sender: TObject; AText: string) of object;

  TFileGetAsURL = procedure(Sender: TObject; AURL: string) of object;

  TFileGetAsArrayBuffer = procedure(Sender: TObject; ABuffer: TJSArrayBufferRecord) of object;

  {$IFNDEF LCLTMSWEB}
  TGetAsStringProc = reference to procedure(AValue: string);
  {$ENDIF}

  {$IFDEF LCLTMSWEB}
  TGetAsStringProc = procedure(AValue: string);
  {$ENDIF}

  {$IFNDEF LCLTMSWEB}
  TGetAsArrayBufferProc = reference to procedure(AValue: TJSArrayBuffer);
  {$ENDIF}

  {$IFDEF LCLTMSWEB}
  TGetAsArrayBufferProc = procedure(AValue: TJSArrayBuffer);
  {$ENDIF}

  TFile = class(TCollectionItem)
  private
    FModified: TDateTime;
    FName: string;
    FMimeType: string;
    FSize: integer;
    FTag: integer;
    FObject: TObject;
    FFileObject: TJSHTMLFile;
    FOnGetFileAsBase64: TFileGetAsBase64;
    FOnGetFileAsArrayBuffer: TFileGetAsArrayBuffer;
    FOnGetFileAsText: TFileGetAsText;
    FOnGetFileAsURL: TFileGetAsURL;
  protected
  public
    property FileObject: TJSHTMLFile read FFileObject write FFileObject;
    property Name: string read FName write FName;
    property MimeType: string read FMimeType write FMimeType;
    property Modified: TDateTime read FModified write FModified;
    property &Object: TObject read FObject write FObject;
    property Size: integer read FSize write FSize;
    property Tag: integer read FTag write FTag;
    procedure GetFileAsText; overload;
    procedure GetFileAsText(GetAsString: TGetAsStringProc); overload;
    procedure GetFileAsText(AEncoding: string); overload;
    procedure GetFileAsBase64; overload;
    procedure GetFileAsBase64(GetAsString: TGetAsStringProc); overload;
    procedure GetFileAsArrayBuffer; overload;
    procedure GetFileAsArrayBuffer(GetAsArrayBuffer: TGetAsArrayBufferProc); overload;
    procedure GetFileAsDataURL; overload;
    procedure GetFileAsDataURL(GetAsString: TGetAsStringProc); overload;
    procedure Upload(AAction: string);
    property OnGetFileAsText: TFileGetAsText read FOnGetFileAsText write FOnGetFileAsText;
    property OnGetFileAsBase64: TFileGetAsBase64 read FOnGetFileAsBase64 write FOnGetFileAsBase64;
    property OnGetFileAsURL: TFileGetAsURL read FOnGetFileAsURL write FOnGetFileAsURL;
    property OnGetFileAsArrayBuffer: TFileGetAsArrayBuffer read FOnGetFileAsArrayBuffer write FOnGetFileAsArrayBuffer;
  end;

  TFiles = class(TCollection)
  private
    function GetItem(Index: integer): TFile;
    procedure SetItem(Index: integer; const Value: TFile);
  public
    property Items[Index: integer]: TFile read GetItem write SetItem; default;
    function Add: TFile; reintroduce;
    function Insert(Index: integer): TFile; reintroduce;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebFileUpload = class(TWebCustomControl)
  private
    FAccept: string;
    FFiles: TFiles;
    FOnDroppedFiles: TFilesDroppedEvent;
    FMultiFile: boolean;
    FShowFiles: boolean;
    FDragCaption: string;
    FCaption: string;
    FOnGetFileAsArrayBuffer: TFileAsArrayBufferEvent;
    FGetFileAsText: TFileAsTextEvent;
    FGetFileAsBase64: TFileAsBase64Event;
    FOnGetFileAsDataURL: TFileAsDataURLEvent;
    FOnUploadFileAbort: TFileUploadEvent;
    FOnUploadFileComplete: TFileUploadEvent;
    FOnUploadFileProgress: TFileUploadProgressEvent;
    FOnUploadFileError: TFileUploadErrorEvent;
  public
    constructor Create(AOwner: TComponent); override;
    property Files: TFiles read FFiles;
    procedure Clear;
  published
    property Accept: string read FAccept write FAccept;
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Caption: string read FCaption write FCaption;
    property ChildOrder;
    property DragCaption: string read FDragCaption write FDragCaption;
    property Multifile: boolean read FMultiFile write FMultiFile default false;
    property ShowFiles: boolean read FShowFiles write FShowFiles default false;
    property Visible;
    property OnDroppedFiles: TFilesDroppedEvent read FOnDroppedFiles write FOnDroppedFiles;
    property OnGetFileAsText: TFileAsTextEvent read FGetFileAsText write FGetFileAsText;
    property OnGetFileAsBase64: TFileAsBase64Event read FGetFileAsBase64 write FGetFileAsBase64;
    property OnGetFileAsArrayBuffer: TFileAsArrayBufferEvent read FOnGetFileAsArrayBuffer write FOnGetFileAsArrayBuffer;
    property OnGetFileAsDataURL: TFileAsDataURLEvent read FOnGetFileAsDataURL write FOnGetFileAsDataURL;
    property OnUploadFileComplete: TFileUploadEvent read FOnUploadFileComplete write FOnUploadFileComplete;
    property OnUploadFileAbort: TFileUploadEvent read FOnUploadFileAbort write FOnUploadFileAbort;
    property OnUploadFileError: TFileUploadErrorEvent read FOnUploadFileError write FOnUploadFileError;
    property OnUploadFileProgress: TFileUploadProgressEvent read FOnUploadFileProgress write FOnUploadFileProgress;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebFilePicker = class(TWebCustomControl)
  private
    FOnChange: TNotifyEvent;
    FMultiFile: boolean;
    FFiles: TFiles;
    FGetFileAsText: TFileAsTextEvent;
    FOnGetFileAsArrayBuffer: TFileAsArrayBufferEvent;
    FAccept: string;
    FGetFileAsBase64: TFileAsBase64Event;
    FOnGetFileAsDataURL: TFileAsDataURLEvent;
    FOnUploadFileAbort: TFileUploadEvent;
    FOnUploadFileComplete: TFileUploadEvent;
    FOnUploadFileProgress: TFileUploadProgressEvent;
    FOnUploadFileError: TFileUploadErrorEvent;
  public
    property Files: TFiles read FFiles;
    constructor Create(AOwner: TComponent); override;
    procedure Clear;
  published
    property Accept: string read FAccept write FAccept;
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property ChildOrder;
    property ElementFont;
    property ElementPosition;
    property Multifile: boolean read FMultiFile write FMultiFile default false;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnGetFileAsText: TFileAsTextEvent read FGetFileAsText write FGetFileAsText;
    property OnGetFileAsBase64: TFileAsBase64Event read FGetFileAsBase64 write FGetFileAsBase64;
    property OnGetFileAsArrayBuffer: TFileAsArrayBufferEvent read FOnGetFileAsArrayBuffer write FOnGetFileAsArrayBuffer;
    property OnGetFileAsDataURL: TFileAsDataURLEvent read FOnGetFileAsDataURL write FOnGetFileAsDataURL;
    property OnUploadFileComplete: TFileUploadEvent read FOnUploadFileComplete write FOnUploadFileComplete;
    property OnUploadFileAbort: TFileUploadEvent read FOnUploadFileAbort write FOnUploadFileAbort;
    property OnUploadFileError: TFileUploadErrorEvent read FOnUploadFileError write FOnUploadFileError;
    property OnUploadFileProgress: TFileUploadProgressEvent read FOnUploadFileProgress write FOnUploadFileProgress;
 end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebHTMLDiv = class(TWebCustomControl)
  private
    FHTML: TStrings;
    FLinkTouchEvents: boolean;
    FElementHandle: TJSHTMLElement;
    FRole: string;
    procedure SetHTML(const Value: TStrings);
  protected
    function GetDesignText: string; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property LinkTouchEvents: boolean read FLinkTouchEvents write FLinkTouchEvents;
    property ElementHandle: TJSHTMLElement read FElementHandle;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property DragMode;
    property ElementPosition;
    property ElementFont;
    property HTML: TStrings read FHTML write SetHTML;
    property Role: string read FRole write FRole;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnTouchStart;
    property OnTouchEnd;
    property OnTouchMove;
    property OnTouchCancel;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebHTMLSpan = class(TWebCustomControl)
  private
    FHTML: TStrings;
    FLinkTouchEvents: boolean;
    FElementHandle: TJSHTMLElement;
    FRole: string;
    procedure SetHTML(const Value: TStrings);
  protected
    function GetDesignText: string; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property LinkTouchEvents: boolean read FLinkTouchEvents write FLinkTouchEvents;
    property ElementHandle: TJSHTMLElement read FElementHandle;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property DragMode;
    property ElementPosition;
    property ElementFont;
    property HTML: TStrings read FHTML write SetHTML;
    property Role: string read FRole write FRole;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnTouchStart;
    property OnTouchEnd;
    property OnTouchMove;
    property OnTouchCancel;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebHTMLAnchor = class(TWebCustomControl)
  private
    FElementHandle: TJSHTMLElement;
    FHREF: string;
    FTarget: string;
  protected
    function GetDesignText: string; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    property ElementHandle: TJSHTMLElement read FElementHandle;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Caption;
    property ChildOrder;
    property ElementPosition;
    property ElementFont;
    property HREF: string read FHREF write FHREF;
    property Target: string read FTarget write FTarget;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseEnter;
    property OnMouseLeave;
  end;


  TURLValidatorEvent = procedure(Sender: TObject; IsValid: boolean) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebURLValidator = class(TComponent)
  private
    FURL: string;
    FOnValidated: TURLValidatorEvent;
  public
    procedure Validate;
  published
    property URL: string read FURL write FURL;
    property OnValidated: TURLValidatorEvent read FOnValidated write FOnValidated;
  end;

  TJSHTMLFileArray = array of TJSHTMLFile;

  TShare = class(TComponent)
  public
    function Share(ATitle,AText,AURL: string): boolean; overload;
    function Share(ATitle,AText,AURL: string; AFiles: TJSHTMLFileArray): boolean; overload;
    function CanShareFiles: boolean;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebShare = class(TShare);


implementation

{ TWebGoogleMaps }

procedure TWebGoogleMaps.AddMarker(Lat, Lon: double; Title: string);
begin
//
end;

procedure TWebGoogleMaps.AddMarker(Lat, Lon: double; AColor: TColor; PinLetter,
  Title: string);
begin
//
end;

procedure TWebGoogleMaps.AddMarker(Lat, Lon: double; AColor: TGoogleMarkerColor;
  Title: string);
begin
//
end;

procedure TWebGoogleMaps.AddCircle(Lat, Lon: double; Radius: Integer;
  AFillColor, AStrokeColor: TColor; AWidth: integer; AOpacity: double);
begin
//
end;

procedure TWebGoogleMaps.AddGPX(AGPX: string; AColor: TColor; AWidth: integer;
  AOpacity: Double);
begin
//
end;

procedure TWebGoogleMaps.AddKML(Url: string; ZoomToBounds: Boolean);
begin
//
end;

procedure TWebGoogleMaps.AddMarker(Lat, Lon: double; Shape: TGoogleMarkerShape;
  AColor: TColor; Title: string);
begin
//
end;

procedure TWebGoogleMaps.AddPolygon(Points: TJSArray; AFillColor,
  AStrokeColor: TColor; AWidth: integer; AOpacity: double);
begin
//
end;

procedure TWebGoogleMaps.AddPolyLine(Points: TJSArray; AColor: TColor;
  AWidth: integer; AOpacity: double);
begin
//
end;

procedure TWebGoogleMaps.AddRectangle(NorthEastLat, NorthEastLon, SouthWestLat,
  SouthWestLon: double; AFillColor, AStrokeColor: TColor; AWidth: integer;
  AOpacity: double);
begin
//
end;

procedure TWebGoogleMaps.AddMarker(Lat, Lon: double; PinIcon, Title: string; XOffset: integer = 0; YOffset: integer = 0);
begin
//
end;

procedure TWebGoogleMaps.ClearCircles;
begin
//
end;

procedure TWebGoogleMaps.ClearKMLs;
begin
//
end;

procedure TWebGoogleMaps.ClearMarkers;
begin
//
end;

procedure TWebGoogleMaps.ClearPolygons;
begin
//
end;

procedure TWebGoogleMaps.ClearPolylines;
begin
//
end;

procedure TWebGoogleMaps.ClearRectangles;
begin
//
end;

constructor TWebGoogleMaps.Create(AOwner: TComponent);
begin
  inherited;
  FOptions := TGoogleMapsOptions.Create(Self);
  Width := 640;
  Height := 480;
end;

destructor TWebGoogleMaps.Destroy;
begin
  FOptions.Free;
  inherited;
end;

procedure TWebGoogleMaps.FitBounds(LatMin, LonMin, LatMax, LonMax: double);
begin
//
end;

procedure TWebGoogleMaps.GeoCode(const Address: string);
begin
//
end;

function TWebGoogleMaps.GetBounds(var NorthEastLat, NorthEastLon, SouthWestLat,
  SouthWestLon: Double): Boolean;
begin
  Result := False;
end;

function TWebGoogleMaps.GetCenter(var Lat, Lon: Double): Boolean;
begin
  Result := False;
end;

function TWebGoogleMaps.GetCircle(AIndex: integer): TJSObject;
begin
  Result := nil;
end;

function TWebGoogleMaps.GetMarker(AIndex: integer): TJSObject;
begin
  Result := nil;
end;

function TWebGoogleMaps.GetPolygon(AIndex: integer): TJSObject;
begin
  Result := nil;
end;

function TWebGoogleMaps.GetPolyline(AIndex: integer): TJSObject;
begin
  Result := nil;
end;

function TWebGoogleMaps.GetRectangle(AIndex: integer): TJSObject;
begin
  Result := nil;
end;

procedure TWebGoogleMaps.HideStreetView;
begin
//
end;

procedure TWebGoogleMaps.PanTo(Lat, Lon: double);
begin
//
end;

procedure TWebGoogleMaps.RemoveCircle(AIndex: Integer);
begin
//
end;

procedure TWebGoogleMaps.RemoveDirections;
begin
//
end;

procedure TWebGoogleMaps.RemoveMarker(AIndex: Integer);
begin
//
end;

procedure TWebGoogleMaps.RemovePolygon(AIndex: Integer);
begin
//
end;

procedure TWebGoogleMaps.RemovePolyline(AIndex: Integer);
begin
//
end;

procedure TWebGoogleMaps.RemoveRectangle(AIndex: Integer);
begin
//
end;

procedure TWebGoogleMaps.SetCenter(Lat, Lon: double);
begin
//
end;

procedure TWebGoogleMaps.SetCircleCenter(AIndex: Integer; Lat, Lon: Double);
begin
//
end;

procedure TWebGoogleMaps.SetCircleColors(AIndex: Integer; AFillColor,
  AStrokeColor: TColor);
begin
//
end;

procedure TWebGoogleMaps.SetCircleRadius(AIndex, Radius: Integer);
begin
//
end;

procedure TWebGoogleMaps.SetDoubleClickZoom(AValue: Boolean);
begin
//
end;

procedure TWebGoogleMaps.SetDraggable(AValue: Boolean);
begin
//
end;

procedure TWebGoogleMaps.SetMapType(AMapType: TGoogleMapType);
begin
//
end;

procedure TWebGoogleMaps.SetMarkerIcon(AIndex: Integer; Url: string);
begin
//
end;

procedure TWebGoogleMaps.SetMarkerLocation(AIndex: Integer; Lat, Lon: Double);
begin
//
end;

procedure TWebGoogleMaps.SetMarkerTitle(AIndex: Integer; ATitle: string);
begin
//
end;

procedure TWebGoogleMaps.SetPolygonColors(AIndex: Integer; AFillColor,
  AStrokeColor: TColor);
begin
//
end;

procedure TWebGoogleMaps.SetPolygonPoints(AIndex: Integer; Points: TJSArray);
begin
//
end;

procedure TWebGoogleMaps.SetPolylineColor(AIndex: Integer; AColor: TColor);
begin
//
end;

procedure TWebGoogleMaps.SetPolylinePoints(AIndex: Integer; Points: TJSArray);
begin
//
end;

procedure TWebGoogleMaps.SetRectangleColors(AIndex: Integer; AFillColor,
  AStrokeColor: TColor);
begin
//
end;

procedure TWebGoogleMaps.SetRectangleLocation(AIndex: Integer; NorthEastLat,
  NorthEastLon, SouthWestLat, SouthWestLon: double);
begin
//
end;

procedure TWebGoogleMaps.SetScrollWheel(AValue: Boolean);
begin
//
end;

procedure TWebGoogleMaps.SetZoom(zoom: integer);
begin
//
end;


procedure TWebGoogleMaps.ShowDirections(SourceLon, SourceLat, DestLon,
  DestLat: double; ATravelMode: TGoogleTravelMode; WayPoints: TStringList;
  OptimizeWayPoints, AvoidHighways, AvoidTolls: Boolean);
begin
//
end;

procedure TWebGoogleMaps.ShowStreetView(Lat, Lon: Double; Heading, Zoom,
  Pitch: Integer);
begin
//
end;

procedure TWebGoogleMaps.ShowDirections(Source, Destination: string;
  ATravelMode: TGoogleTravelMode; WayPoints: TStringList; OptimizeWayPoints,
  AvoidHighways, AvoidTolls: Boolean);
begin
//
end;

{ TWebYoutube }

constructor TWebYoutube.Create(AOwner: TComponent);
begin
  inherited;
  Width := 480;
  Height := 360;
end;

{ TWebBrowserControl }

constructor TWebBrowserControl.Create(AOwner: TComponent);
begin
  inherited;
  Width := 480;
  Height := 360;
end;

procedure TWebBrowserControl.Navigate(const AURL: string);
begin
  URL := AURL;
end;


{ TWebGoogleDrive }

constructor TWebGoogleDrive.Create(AOwner: TComponent);
begin
  inherited;
  Width := 500;
  Height := 350;
end;

{ TFiles }

function TFiles.Add: TFile;
begin
  Result := TFile(inherited Add);
end;

function TFiles.GetItem(Index: integer): TFile;
begin
  Result := TFile(inherited Items[Index]);
end;

function TFiles.Insert(Index: integer): TFile;
begin
  Result := TFile(inherited Insert(Index));
end;

procedure TFiles.SetItem(Index: integer; const Value: TFile);
begin
  inherited Items[Index] := Value;
end;

{ TFile }

procedure TFile.GetFileAsArrayBuffer;
begin
  //
end;

procedure TFile.GetFileAsBase64;
begin
  //
end;

procedure TFile.GetFileAsDataURL;
begin
  //
end;

procedure TFile.GetFileAsText(GetAsString: TGetAsStringProc);
begin
  //
end;

procedure TFile.GetFileAsText;
begin
  //
end;

procedure TFile.Upload(AAction: string);
begin
  //
end;

procedure TFile.GetFileAsArrayBuffer(GetAsArrayBuffer: TGetAsArrayBufferProc);
begin
  //
end;

procedure TFile.GetFileAsBase64(GetAsString: TGetAsStringProc);
begin
  //
end;

procedure TFile.GetFileAsDataURL(GetAsString: TGetAsStringProc);
begin
  //
end;

procedure TFile.GetFileAsText(AEncoding: string);
begin
  //
end;

{ TWebFilePicker }

procedure TWebFilePicker.Clear;
begin
//
end;

constructor TWebFilePicker.Create(AOwner: TComponent);
begin
  inherited;
  Width := 121;
  Height := 19;
end;

{ TWebFileUpload }

procedure TWebFileUpload.Clear;
begin
//
end;

constructor TWebFileUpload.Create(AOwner: TComponent);
begin
  inherited;

  FCaption := 'Choose a file';
  FDragCaption := 'or drag it here';

  Width := 170;
  Height := 190;
end;

{ TWebHTMLDiv }

constructor TWebHTMLDiv.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  FHTML := TStringList.Create;
end;

destructor TWebHTMLDiv.Destroy;
begin
  FHTML.Free;
  inherited;
end;

function TWebHTMLDiv.GetDesignText: string;
begin
  if ControlCount = 0 then
    Result := inherited GetDesignText
  else
    Result := '';
end;

procedure TWebHTMLDiv.SetHTML(const Value: TStrings);
begin
  FHTML.Assign(Value);
end;

{ TWebHTMLSpan }

constructor TWebHTMLSpan.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  FHTML := TStringList.Create;
end;

destructor TWebHTMLSpan.Destroy;
begin
  FHTML.Free;
  inherited;
end;

function TWebHTMLSpan.GetDesignText: string;
begin
  if ControlCount = 0 then
    Result := inherited GetDesignText
  else
    Result := '';
end;

procedure TWebHTMLSpan.SetHTML(const Value: TStrings);
begin
  FHTML.Assign(Value);
end;

{ TWebHTMLAnchor }

constructor TWebHTMLAnchor.Create(AOwner: TComponent);
begin
  inherited;
  Width := 96;
  Height := 24;
end;

function TWebHTMLAnchor.GetDesignText: string;
begin
  Result := Caption;
  if Result = '' then
    Result := HREF;

  if Result = '' then
    Result := Name;
end;

procedure TWebHTMLAnchor.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := clWindow;
  Canvas.Pen.Color := clWindow;

  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := GetDesignText;

  InflateRect(lRect,-1,-1);

  Canvas.Font.Color := clBlue;
  Canvas.Font.Style := [fsUnderline];

  if (Width > 20) and (Height > 10) then
    DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER { or DT_HIDEPREFIX } or DT_END_ELLIPSIS);
end;

{ TWebURLValidator }

procedure TWebURLValidator.Validate;
begin
//
end;

{ TGoogleMapsOptions }

procedure TGoogleMapsOptions.Assign(Source: TPersistent);
begin
  inherited;
  FCustomStyle := (Source as TGoogleMapsOptions).CustomStyle;
  FMapStyle := (Source as TGoogleMapsOptions).MapStyle;
  FDefaultLatitude := (Source as TGoogleMapsOptions).DefaultLatitude;
  FDefaultLongitude := (Source as TGoogleMapsOptions).DefaultLongitude;
  FDefaultZoomLevel := (Source as TGoogleMapsOptions).DefaultZoomLevel;
end;

constructor TGoogleMapsOptions.Create(AGoogleMaps: TWebGoogleMaps);
begin
  inherited Create;
  FCustomStyle := TStringList.Create;
  FMapStyle := mstDefault;
  FDefaultLatitude := -34.397;
  FDefaultLongitude := 150.644;
  FDefaultZoomLevel := 8;
  FOwner := AGoogleMaps;
end;

destructor TGoogleMapsOptions.Destroy;
begin
  FCustomStyle.Free;
  inherited;
end;

procedure TGoogleMapsOptions.SetCustomStyle(const Value: TStringList);
begin
  FCustomStyle.Assign(Value);
end;

{ TWebTwitterFeed }

constructor TWebTwitterFeed.Create(AOwner: TComponent);
begin
  inherited;
  Width := 300;
  Height := 400;
end;

function TWebTwitterFeed.GetDesignText: string;
begin
  if Feed <> '' then
    Result := Feed
  else
    Result := inherited;
end;

procedure TWebTwitterFeed.SetFeed(const Value: string);
begin
  FFeed := Value;
  Invalidate;
end;

{ TShare }

function TShare.CanShareFiles: boolean;
begin
  Result := true;
end;

function TShare.Share(ATitle, AText, AURL: string): boolean;
begin
  Result := true;
end;

function TShare.Share(ATitle, AText, AURL: string;
  AFiles: TJSHTMLFileArray): boolean;
begin
  Result := true;
end;

end.
