unit Web;

interface

uses
  Types, JS, jsdelphisystem;

type
  TJSAttr = class;
  TEventListenerEvent = class;
  TJSEventTarget = class;
  TJSNode = class;
  TJSNodeList = class;
  TJSNamedNodeMap = class;
  TJSHTMLCollection = class;
  TJSDOMTokenList = class;
  TJSDOMRect = class;
  TJSElement = class;
  TJSDocumentType = class;
  TJSDOMImplementation = class;
  TJSLocation = class;
  TJSStyleSheet = class;
  TJSCSSRule = class;
  TJSCSSRuleList = class;
  TJSCSSStyleSheet = class;
  TJSStyleSheetList = class;
  TJSDocumentFragment = class;
  TJSEvent = class;
  TJSXPathExpression = class;
  TJSXPathNSResolver = class;
  TJSCharacterData = class;
  TJSProcessingInstruction = class;
  TJSRange = class;
  TJSTreeWalker = class;
  TJSNodeFilter = class;
  TJSXPathResult = class;
  TJSSelection = class;
  TDataTransfer = class;
  TJSDragEvent = class;
  TJSAnimationEvent = class;
  TJSLoadEvent = class;
  TJSErrorEvent = class;
  TJsPageTransitionEvent = class;
  TJSHashChangeEvent = class;
  TJSPopStateEvent = class;
  TJSStorageEvent = class;
  TJSProgressEvent = class;
  TJSDocument = class;
  TJSConsole = class;
  TJSPromise = class;
  TJSCryptoKey = class;
  TJSBufferSource = class;
  TJSTypedArray = class;
  TJSSubtleCrypto = class;
  TJSCrypto = class;
  TJSHistory = class;
  TJSIDBTransactionMode = class;
  TJSIDBTransaction = class;
  TJSIDBKeyRange = class;
  TJSIDBIndex = class;
  TJSIDBCursorDirection = class;
  TJSIDBCursor = class;
  TJSIDBObjectStore = class;
  TJSIDBRequest = class;
  TJSIDBOpenDBRequest = class;
  TIDBDatabase = class;
  TJSIDBFactory = class;
  TJSStorage = class;
  TJSVisibleItem = class;
  TJSLocationBar = class;
  TJSMenuBar = class;
  TJSToolBar = class;
  TJSPersonalBar = class;
  TJSScrollBars = class;
  TJSGeoLocation = class;
  TJSMediaDevices = class;
  TJSWorker = class;
  TJSMessagePort = class;
  TJSSharedWorker = class;
  TJSServiceWorker = class;
  TJSServiceWorkerRegistration = class;
  TJSServiceWorkerContainer = class;
  TJSNavigator = class;
  TJSPerformance = class;
  TJSScreen = class;
  TJSURL = class;
  TJSMediaQueryList = class;
  TJSWindow = class;
  TJSCSSStyleDeclaration = class;
  TJSHTMLElement = class;
  TJSHTMLFormControlsCollection = class;
  TJSHTMLFormElement = class;
  TJSValidityState = class;
  TJSBlob = class;
  TJSHTMLFile = class;
  TJSHTMLFileList = class;
  TJSHTMLInputElement = class;
  TJSHTMLOptionElement = class;
  TJSHTMLOptGroupElement = class;
  TJSHTMLOptionsCollection = class;
  TJSHTMLSelectElement = class;
  TJSHTMLCanvasElement = class;
  TJSCanvasGradient = class;
  TJSCanvasPattern = class;
  TJSPath2D = class;
  TJSImageData = class;
  TJSCanvasRenderingContext2D = class;
  TJSXMLHttpRequestEventTarget = class;
  TJSXMLHttpRequestUpload = class;
  TJSXMLHttpRequest = class;
  TJSUIEvent = class;
  TJSMouseEvent = class;
  TJSWheelEvent = class;
  TJSPointerEvent = class;
  TJSKeyNames = class;
  TJSKeyboardEvent = class;
  TJSMessageEvent = class;
  TJSInputEvent = class;

  TJSAttr = class(TObject)
  strict private
    fLocalName: string;
    fName: string;
    fNameSpaceURI: string;
    fPrefix: string;
  public
    value: JSValue;
    property localName: string read fLocalName;
    property name: string read fName;
    property namespaceURI: string read fNameSpaceURI;
    property prefix: string read fPrefix;
  end;

  TEventListenerEvent = class(TObject)
  strict private
    FTarget: TJSEventTarget;
  public
    property target: TJSEventTarget read FTarget;
  end;

  TJSEventHandler = function(Event: TEventListenerEvent) : Boolean;
  TJSEventHandlerMethod = function(Event: TEventListenerEvent) : Boolean of object;

  TJSEventTarget = class(TObject)
  public
    procedure addEventListener(aname: string; aListener: JSValue); overload;
    procedure addEventListener(aname: string; aListener: TJSEventHandler); overload;
    procedure addEventListener(aname: string; aListener: TJSEventHandlerMethod); overload;
    function dispatchEvent(event: JSValue): Boolean;
    procedure removeEventListener(aname: string; aListener: JSValue); overload;
    procedure removeEventListener(aname: string; aListener: TJSEventHandler); overload;
  end;

  TJSNode = class(TJSEventTarget)
  strict private
    FBaseURI: string;
    FChildNodes: TJSNodeList;
    FFirstChild: TJSNode;
    FNextSibling: TJSNode;
    FNodeName: string;
    FNodeType: NativeInt;
    FOwnerDocument: TJSDocument;
    FParentElement: TJSElement;
    FParentNode: TJSNode;
    FPreviousSibling: TJSNode;
  public const
    COMMENT_NODE = 8;
    DOCUMENT_FRAGMENT_NODE = 11;
    DOCUMENT_NODE = 9;
    DOCUMENT_POSITION_CONTAINED_BY = 16;
    DOCUMENT_POSITION_CONTAINS = 8;
    DOCUMENT_POSITION_DISCONNECTED = 1;
    DOCUMENT_POSITION_FOLLOWING = 4;
    DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC = 32;
    DOCUMENT_POSITION_PRECEDING = 2;
    DOCUMENT_TYPE_NODE = 10;
    ELEMENT_NODE = 1;
    PROCESSING_INSTRUCTION_NODE = 7;
    TEXT_NODE = 3;
  public
    innerText: string;
    nodeValue: string;
    textContent: string;
    function appendChild(aChild: TJSNode): TJSNode;
    function cloneNode(deep: Boolean): TJSNode;
    function compareDocumentPosition(aNode: TJSNode): NativeInt;
    function contains(aNode: TJSNode): Boolean;
    function hasChildNodes: Boolean;
    function insertBefore(newNode: TJSNode; referenceNode: TJSNode): TJSNode;
    function isDefaultNameSpace(aNameSpaceURI: string): Boolean;
    function isEqualNode(aNode: TJSNode): Boolean;
    function isSameNode(aNode: TJSNode): Boolean;
    function lookupNamespaceURI(aPrefix: string): string;
    function lookupPrefix(aPrefix: string): string;
    procedure normalize;
    function removeChild(aChild: TJSNode): TJSNode;
    function replaceChild(aNewChild: TJSNode; aOldChild: TJSNode): TJSNode;
    property baseURI: string read FBaseURI;
    property childNodes: TJSNodeList read FChildNodes;
    property firstChild: TJSNode read FFirstChild;
    property nextSibling: TJSNode read FNextSibling;
    property nodeName: string read FNodeName;
    property nodeType: NativeInt read FNodeType;
    property ownerDocument: TJSDocument read FOwnerDocument;
    property parentElement: TJSElement read FParentElement;
    property parentNode: TJSNode read FParentNode;
    property previousSibling: TJSNode read FPreviousSibling;
  end;

  TJSNodeListCallBack = procedure(currentValue: TJSNode; currentIndex: NativeInt; list: TJSNodeList);
  TJSNodeListEvent = procedure(currentValue: TJSNode; currentIndex: NativeInt; list: TJSNodeList) of object;

  TJSNodeList = class(TObject)
  strict private
    FLength: NativeInt;
  public
    procedure forEach(const aCallBack: TJSNodeListCallBack); overload;
    procedure forEach(const aCallBack: TJSNodeListEvent); overload;
    function item(aIndex: NativeInt): TJSNode;
    property length: NativeInt read FLength;
    property Nodes[aIndex : NativeInt]: TJSNode read item; default;
  end;

  TJSNamedNodeMap = class(TObject)
  public
    function getNamedItem(aName: string): TJSAttr;
    property Attrs[aIndex : string]: TJSattr read getNamedItem; default;
  end;

  TJSHTMLCollection = class(TObject)
  strict private
    FLength: NativeInt;
  public
    function item(aIndex: Integer): TJSNode;
    function namedItem(aName: string): TJSNode;
    property Items[aIndex : Integer]: TJSNode read item; default;
    property length: NativeInt read FLength;
    property NamedItems[aName : string]: TJSNode read namedItem;
  end;

  TJSDOMString = string;
  TDOMTokenlistCallBack = procedure(Current: JSValue; currentIndex: NativeInt; list: TJSDOMTokenList);

  TJSDOMTokenList = class(TObject)
  strict private
    FLength: NativeInt;
  public
    procedure add(aToken: TJSDOMString);
    function contains(aToken: TJSDOMString): Boolean;
    procedure forEach(const callback: TDOMTokenlistCallBack);
    function item(aIndex: NativeInt): string;
    procedure remove(aToken: TJSDOMString);
    procedure replace(aOldToken: TJSDOMString; ANewToken: TJSDOMString);
    function supports(aToken: TJSDOMString): Boolean;
    function toggle(aToken: TJSDOMString): Boolean;
    property length: NativeInt read FLength;
  end;

  TJSDOMRect = class(TObject)
  public
    bottom: Double;
    height: Double;
    left: Double;
    right: Double;
    top: Double;
    width: Double;
    x: Double;
    y: Double;
  end;

  TJSClientRect = record
  public
    bottom : Double;
    left : Double;
    right : Double;
    top : Double;
  end;

  TJSClientRectArray = array of TJSClientRect;

  TJSElement = class(TJSNode)
  strict private
    FAttributes: TJSNamedNodeMap;
    FChildElementCount: NativeInt;
    FChildren: TJSHTMLCollection;
    FClassList: TJSDOMTokenList;
    FCLientHeight: NativeInt;
    FCLientLeft: NativeInt;
    FCLientTop: NativeInt;
    FCLientWidth: NativeInt;
    FFirstElementChild: TJSElement;
    FLastElementChild: TJSElement;
    FLocalName: string;
    FNameSpaceURI: string;
    FNextElementSibling: TJSElement;
    FPrefix: string;
    FPreviousElementSibling: TJSElement;
    FScrollHeight: NativeInt;
    FScrollLeft: NativeInt;
    FScrollTop: NativeInt;
    FScrollWidth: NativeInt;
    FTagName: string;
  public
    className: string;
    id: string;
    innerHTML: string;
    name: string;
    outerHTML: string;
    procedure append(aText: string); overload;
    procedure append(aText: string; aNode: TJSElement); overload;
    procedure append(aNode: TJSElement); overload;
    procedure append(aNode1: TJSElement; aNode2: TJSElement); overload;
    function getAttribute(aName: string): string;
    function getAttributeNode(aName: string): TJSAttr;
    function getAttributeNodeNS(aNameSpace: string; aName: string): TJSAttr;
    function getAttributeNS(aNameSpace: string; aName: string): string;
    function getBoundingClientRect: TJSDOMRect;
    function getClientRects: TJSClientRectArray;
    function getElementsByClassName(aClassName: string): TJSHTMLCollection;
    function getElementsByTagName(aTagName: string): TJSHTMLCollection;
    function getElementsByTagNameNS(aNameSpace: string; aTagName: string): TJSHTMLCollection;
    function hasAttribute(aName: string): Boolean;
    function hasAttributeNS(aNameSpace: string; aName: string): Boolean;
    function hasAttributes: Boolean;
    function insertAdjacentElement(aPosition: string; aElement: TJSElement): TJSElement;
    procedure insertAdjacentHTML(aPosition: string; aHTML: string);
    procedure insertAdjacentText(aPosition: string; aText: string);
    function matches(aSelectorString: string): Boolean;
    function querySelector(aSelectors: string): TJSElement;
    function querySelectorAll(aSelectors: string): TJSNodeList;
    procedure releasePointerCapture(evID: JSValue);
    procedure removeAttribute(aName: string);
    function removeAttributeNode(aAttr: TJSAttr): TJSAttr;
    procedure removeAttributeNS(aNameSpace: string; aName: string);
    procedure requestFullScreen;
    procedure setAttribute(aName: string; AValue: string);
    function setAttributeNode(aNode: TJSAttr): TJSAttr;
    function setAttributeNodeNS(aNode: TJSAttr): TJSAttr;
    procedure setAttributeNS(aNameSpace: string; aName: string; AValue: string);
    procedure setCapture(retargetToElement: Boolean);
    procedure setPointerCapture(pointerID: JSValue);
    property attributes: TJSNamedNodeMap read FAttributes;
    property Attrs[aName : string]: string read getAttribute write setAttribute; default;
    property childElementCount: NativeInt read FChildElementCount;
    property children: TJSHTMLCollection read FChildren;
    property classList: TJSDOMTokenList read FClassList;
    property clientHeight: NativeInt read FClientHeight;
    property clientLeft: NativeInt read FClientLeft;
    property clientTop: NativeInt read FClientTop;
    property clientWidth: NativeInt read FClientWidth;
    property firstElementChild: TJSElement read FFirstElementChild;
    property lastElementChild: TJSElement read FLastElementChild;
    property localName: string read FLocalName;
    property namespaceURI: string read FNameSpaceURI;
    property nextElementSibling: TJSElement read FNextElementSibling;
    property prefix: string read FPrefix;
    property previousElementSibling: TJSElement read FPreviousElementSibling;
    property scrollHeight: NativeInt read FScrollHeight;
    property scrollLeft: NativeInt read FScrollLeft;
    property scrollTop: NativeInt read FScrollTop;
    property scrollWidth: NativeInt read FScrollWidth;
    property tagName: string read FTagName;
  end;

  TJSElementCreationOptions = record
  public
    named : string;
  end;

  TJSDocumentType = class(TJSNode)
  strict private
    FName: string;
    FPublicID: string;
    FSystemID: string;
  public
    property name: string read FName;
    property publicId: string read FPublicID;
    property systemId: string read FSystemID;
  end;

  TJSDOMImplementation = class(TObject)
  public
    function createDocument(aNamespaceURI: string; aQualifiedNameStr: string; aDocumentType: TJSDocumentType): TJSDocument;
    function createDocumentType(aQualifiedNameStr: string; aPublicId: string; aSystemId: string): TJSDocumentType;
    function createHTMLDocument(aTitle: string): TJSDocument;
  end;

  TJSLocation = class(TObject)
  strict private
    FOrigin: string;
  public
    hash: string;
    host: string;
    hostname: string;
    href: string;
    password: string;
    pathname: string;
    port: string;
    protocol: string;
    search: string;
    username: string;
    procedure assign(aURL: string);
    procedure reload(aForce: Boolean);
    procedure replace(aURL: string);
    function toString: string; reintroduce;
    property origin: string read FOrigin;
  end;

  TJSStyleSheet = class(TJSEventTarget)
  strict private
    FHRef: string;
    FOwnerNode: TJSNode;
    FParentStyleSheet: TJSStyleSheet;
    FTitle: string;
    FType: string;
  public
    disabled: string;
    property href: string read FHRef;
    property ownerNode: TJSNode read FOwnerNode;
    property parentStyleSheet: TJSStyleSheet read FParentStyleSheet;
    property title: string read FTitle;
    property _type: string read FType;
  end;

  TJSCSSRule = class(TObject)
  strict private
    FCSSText: string;
    FparentRule: TJSCSSRule;
    FParentStyleSheet: TJSCSSStyleSheet;
  public
    property cssText: string read FCSSText;
    property parentRule: TJSCSSRule read FparentRule;
    property parentStyleSheet: TJSCSSStyleSheet read FParentStyleSheet;
  end;

  TJSCSSRuleList = class(TObject)
  strict private
    FLength: NativeInt;
  public
    function item(index: NativeInt): TJSCSSRule;
    property items[aIndex : NativeInt]: TJSCSSRule read item; default;
    property length: NativeInt read FLength;
  end;

  TJSCSSStyleSheet = class(TJSStyleSheet)
  strict private
    FCSSRules: TJSCSSRuleList;
  public
    procedure deleteRule(aIndex: NativeInt);
    function insertRule(aRule: string; aIndex: NativeInt): NativeInt;
    property cssRules: TJSCSSRuleList read FCSSRules;
  end;

  TJSStyleSheetList = class(TObject)
  strict private
    FLength: NativeInt;
  public
    function item(index: NativeInt): TJSStyleSheet;
    property items[aIndex : NativeInt]: TJSStyleSheet read item; default;
    property length: NativeInt read FLength;
  end;

  TJSDocumentFragment = class(TJSNode)
  strict private
    FchildElementCount: Integer;
    Fchildren: TJSHTMLCollection;
    FfirstElementChild: TJSElement;
    FlastElementChild: TJSElement;
  public
    constructor new;
    function querySelector(aSelector: string): TJSElement;
    function querySelectorAll(aSelector: string): TJSNodeList;
    property childElementCount: Integer read FchildElementCount;
    property children: TJSHTMLCollection read Fchildren;
    property firstElementChild: TJSElement read FfirstElementChild;
    property lastElementChild: TJSElement read FlastElementChild;
  end;

  TJSEventInit = record
  public
    bubbles : Boolean;
    cancelable : Boolean;
    composed : Boolean;
    scoped : Boolean;
  end;

  TJSEvent = class(TObject)
  strict private
    FBubbles: Boolean;
    FCancelable: Boolean;
    FComposed: Boolean;
    FCurrentTarget: TJSElement;
    FdefaultPrevented: Boolean;
    FEventPhase: NativeInt;
    FIsTrusted: Boolean;
    FTarget: TJSElement;
    FTimeStamp: NativeInt;
    FType: string;
  public const
    AT_TARGET = 2;
    BUBBLING_PHASE = 3;
    CAPTURING_PHASE = 1;
    NONE = 0;
  public
    cancelBubble: Boolean;
    constructor new(aType: string; const aInit: TJSEventInit);
    procedure preventDefault;
    procedure stopImmediatePropagation;
    procedure stopPropagation;
    property bubbles: Boolean read FBubbles;
    property cancelable: Boolean read FCancelable;
    property composed: Boolean read FComposed;
    property currentTarget: TJSElement read FCurrentTarget;
    property defaultPrevented: Boolean read FdefaultPrevented;
    property eventPhase: NativeInt read FEventPhase;
    property isTrusted: Boolean read FIsTrusted;
    property target: TJSElement read FTarget;
    property timestamp: NativeInt read FTimeStamp;
    property _type: string read FType;
  end;

  TJSXPathExpression = class(TObject)
  public
    function evaluate(contextNode: TJSNode; aType: NativeInt; aResult: TJSXPathResult): TJSXPathResult;
    function evaluateWithContext(contextNode: TJSNode; aPosition: NativeInt; aSize: NativeInt; aType: NativeInt; aResult: TJSXPathResult): TJSXPathResult;
  end;

  TJSXPathNSResolver = class(TObject)
  public
    function lookupNamespaceURI(prefix: string): string;
  end;

  TJSCharacterData = class(TJSNode)
  strict private
    FnextElementSibling: TJSElement;
    FpreviousElementSibling: TJSElement;
  public
    property nextElementSibling: TJSElement read FnextElementSibling;
    property previousElementSibling: TJSElement read FpreviousElementSibling;
  end;

  TJSProcessingInstruction = class(TJSCharacterData)
  end;

  TJSRange = class(TObject)
  strict private
    FCollapsed: Boolean;
    FcommonAncestorContainer: TJSNode;
    FendContainer: TJSNode;
    FEndOffset: NativeInt;
    FstartContainer: TJSNode;
    FstartOffset: NativeInt;
  public const
    END_TO_END = 0;
    END_TO_START = 1;
    START_TO_END = 2;
    START_TO_START = 3;
  public
    constructor new;
    function cloneContents: TJSDocumentFragment;
    function cloneRange: TJSRange;
    procedure collapse;
    function compareBoundaryPoints(aHow: NativeInt): NativeInt;
    function createContextualFragment(aTagstring: string): TJSDocumentFragment;
    procedure deleteContents;
    procedure detach;
    function extractContents: TJSDocumentFragment;
    procedure insertNode(aNode: TJSNode);
    procedure selectNode(aNode: TJSNode);
    procedure selectNodeContents(aNode: TJSNode);
    procedure setEnd(aEndNode: TJSNode; aEndOffset: NativeInt);
    procedure setEndAfter(aEndNode: TJSNode);
    procedure setEndBefore(aEndNode: TJSNode);
    procedure setStart(aStartNode: TJSNode; aStartOffset: NativeInt);
    procedure setStartAfter(aStartNode: TJSNode);
    procedure setStartBefore(aStartNode: TJSNode);
    procedure surroundContents(aNode: TJSNode);
    function toString: string; reintroduce;
    property collapsed: Boolean read FCollapsed;
    property commonAncestorContainer: TJSNode read FcommonAncestorContainer;
    property endContainer: TJSNode read FendContainer;
    property endOffset: NativeInt read FEndOffset;
    property startContainer: TJSNode read FstartContainer;
    property startOffset: NativeInt read FstartOffset;
  end;

  TJSTreeWalker = class(TObject)
  strict private
    FCurrentNode: TJSNode;
    FexpandEntityReference: Boolean;
    FFilter: TJSNodeFilter;
    FRoot: TJSNode;
    FWhatToShow: NativeInt;
  public
    function firstChild: TJSNode;
    function lastChild: TJSNode;
    function nextNode: TJSNode;
    function nextSibling: TJSNode;
    function parentNode: TJSNode;
    function previousNode: TJSNode;
    function previousSibling: TJSNode;
    property currentNode: TJSNode read FCurrentNode;
    property expandEntityReference: Boolean read FexpandEntityReference;
    property filter: TJSNodeFilter read FFilter;
    property root: TJSNode read FRoot;
    property whatToShow: NativeInt read FWhatToShow;
  end;

  TJSNodeFilter = class(TObject)
  public const
    SHOW_ALL = - 1;
    SHOW_ATTRIBUTE = 2;
    SHOW_CDATA_SECTION = 8;
    SHOW_COMMENT = 128;
    SHOW_DOCUMENT = 256;
    SHOW_DOCUMENT_FRAGMENT = 1024;
    SHOW_DOCUMENT_TYPE = 512;
    SHOW_ELEMENT = 1;
    SHOW_ENTITY = 32;
    SHOW_ENTITY_REFERENCE = 16;
    SHOW_NOTATION = 2048;
    SHOW_PROCESSING_INSTRUCTION = 64;
    SHOW_TEXT = 4;
  public
    function acceptNode(aNode: TJSNode): NativeInt;
  end;

  TJSXPathResult = class(TObject)
  strict private
    FBooleanValue: Boolean;
    FNumberValue: Double;
    FResultType: NativeInt;
    FSingleNodeValue: TJSNode;
    FSnaphotLength: NativeInt;
    FStringValue: string;
  public
    function iterateNext: TJSNode;
    function snapshotItem(Index: NativeInt): TJSNode;
    property booleanValue: Boolean read FBooleanValue;
    property numberValue: Double read FNumberValue;
    property resultType: NativeInt read FResultType;
    property singleNodeValue: TJSNode read FSingleNodeValue;
    property snapshotLength: NativeInt read FSnaphotLength;
    property stringValue: string read FStringValue;
  end;

  TJSSelection = class(TObject)
  strict private
    FanchorNode: TJSNode;
    FanchorOffset: NativeInt;
    FfocusNode: TJSNode;
    FfocusOffset: NativeInt;
    FisCollapsed: Boolean;
    FrangeCount: NativeInt;
    Ftype: string;
  public
    procedure addRange(aRange: TJSRange);
    procedure collapse(aParentNode: TJSNode; Offset: NativeInt);
    procedure collapseToEnd;
    procedure collapseToStart;
    function containsNode(aNode: TJSNode; aPartlyContained: Boolean): Boolean;
    procedure deleteFromDocument;
    procedure extend(aParentNode: TJSNode; Offset: NativeInt);
    function getRangeAt(aIndex: NativeInt): TJSRange;
    procedure removeAllRanges;
    procedure removeRange(aRange: TJSRange);
    procedure selectAllChildren(aParentNode: TJSNode);
    procedure setBaseAndExtent(aAnchorNode: TJSNode; aAnchorOffset: NativeInt; aFocusNode: TJSNode; aFocusOffset: NativeInt);
    function toString: string; reintroduce;
    property anchorNode: TJSNode read FAnchorNode;
    property anchorOffset: NativeInt read FAnchorOffset;
    property focusNode: TJSNode read FFocusNode;
    property focusOffset: NativeInt read FFocusOffset;
    property isCollapsed: Boolean read FIsCollapsed;
    property rangeCount: NativeInt read FRangeCount;
    property _type: string read FType;
  end;

  TJSNameSpaceMapperCallback = function(aNameSpace: string) : string;

  TDataTransfer = class(TObject)
  end;

  TJSDragEvent = class(TJSEvent)
  strict private
    FaltKey: Boolean;
    FctrlKey: Boolean;
    FmetaKey: Boolean;
    FrelatedTarget: TJSEventTarget;
    FscreenX: NativeInt;
    FshiftKey: Boolean;
  public
    property altKey: Boolean read FaltKey;
    property button: NativeInt read FScreenX;
    property buttons: NativeInt read FScreenX;
    property clientX: NativeInt read FScreenX;
    property clientY: NativeInt read FScreenX;
    property ctrlKey: Boolean read FctrlKey;
    property metaKey: Boolean read FmetaKey;
    property relatedTarget: TJSEventTarget read FRelatedTarget;
    property screenX: NativeInt read FScreenX;
    property screenY: NativeInt read FScreenX;
    property shiftKey: Boolean read FshiftKey;
  end;

  TJSDragDropEventHandler = function(aEvent: TJSDragEvent) : Boolean;
  THTMLClickEventHandler = function(aEvent: TJSMouseEvent) : Boolean;
  TJSFocusEvent = TJSEvent;

  TJSAnimationEvent = class(TJSEvent)
  end;

  TJSLoadEvent = class(TJSEvent)
  end;

  TJSErrorEvent = class(TJSEvent)
  strict private
    Fcolno: Integer;
    Ferror: TJSObject;
    Ffilename: string;
    Flineno: Integer;
    Fmessage: string;
  public
    property colno: Integer read FColNo;
    property error: TJSObject read FError;
    property filename: string read FFileName;
    property lineno: Integer read FLineNo;
    property message: string read FMessage;
  end;

  TJsPageTransitionEvent = class(TJSEvent)
  end;

  TJSHashChangeEvent = class(TJSEvent)
  strict private
    FnewURL: string;
    FoldURL: string;
  public
    property newURL: string read FNewURL;
    property oldURL: string read FOldURL;
  end;

  TJSPopStateEvent = class(TJSEvent)
  strict private
    FState: JSValue;
  public
    property state: JSValue read FState;
  end;

  TJSStorageEvent = class(TJSEvent)
  strict private
    Fkey: string;
    FnewValue: string;
    FoldValue: string;
    FstorageArea: string;
    Furl: string;
  public
    property key: string read FKey;
    property newValue: string read FNewValue;
    property oldValue: string read FOldValue;
    property storageArea: string read FstorageArea;
    property url: string read FURL;
  end;

  TJSProgressEvent = class(TJSEvent)
  strict private
    FlengthComputable: Boolean;
    Floaded: NativeUINT;
    FTotal: NativeUINT;
  public
    property lengthComputable: Boolean read FlengthComputable;
    property loaded: NativeUINT read FLoaded;
    property Total: NativeUINT read FTotal;
  end;

  TJSPageTransitionEventHandler = function(aEvent: TJsPageTransitionEvent): Boolean;
  TJSHashChangeEventhandler = function(aEvent: TJSHashChangeEvent): Boolean;
  TJSMouseEventHandler = function(aEvent: TJSMouseEvent) : Boolean;
  THTMLAnimationEventHandler = function(aEvent: TJSAnimationEvent): Boolean;
  TJSErrorEventHandler = function(aEvent: TJSErrorEvent) : Boolean;
  TJSFocusEventHandler = function(aEvent: TJSFocusEvent) : Boolean;
  TJSKeyEventhandler = function(aEvent: TJSKeyBoardEvent) : Boolean;
  TJSLoadEventhandler = function(aEvent: TJSLoadEvent) : Boolean;
  TJSPointerEventHandler = function(aEvent: TJSPointerEvent) : Boolean;
  TJSUIEventHandler = function(aEvent: TJSUIEvent) : Boolean;
  TJSPopStateEventHandler = function(aEvent: TJSPopStateEvent) : Boolean;
  TJSStorageEventHandler = function(aEvent: TJSStorageEvent) : Boolean;
  TJSProgressEventhandler = function(aEvent: TJSProgressEvent) : Boolean;

  TJSDocument = class(TJSNode)
  strict private
    fActiveElement: TJSElement;
    FCharacterSet: string;
    FChildElementCount: NativeInt;
    FCompatMode: string;
    FCurrentScript: TJSElement;
    FDefaultView: TJSWindow;
    FDocType: TJSDocumentType;
    FDocumentElement: TJSElement;
    FDocumentURI: string;
    FEmbeds: TJSHTMLCollection;
    FFirstElementChild: TJSElement;
    FForms: TJSHTMLCollection;
    FFullScreenElement: TJSElement;
    FFullscreenEnabled: Boolean;
    FHead: TJSElement;
    FHidden: Boolean;
    FImages: TJSHTMLCollection;
    FImplementation: TJSDOMImplementation;
    FLastElementChild: TJSElement;
    FLastModified: string;
    FLastStyleSheetSet: string;
    FLinks: TJSHTMLCollection;
    FLocation: TJSLocation;
    FLocationString: string;
    FPlugins: TJSHTMLCollection;
    FPointerLockElement: TJSElement;
    FPreferredStyleSheetSet: string;
    FReadyState: string;
    FReferrer: string;
    FScripts: TJSHTMLCollection;
    FStyleSheets: TJSStyleSheetList;
    FStyleSheetSets: TJSValueDynArray;
    FURL: string;
    FVisibilityState: string;
  public
    body: TJSElement;
    designMode: string;
    dir: string;
    domain: string;
    onabort: TJSEventHandler;
    onblur: TJSEventHandler;
    oncancel: TJSEventHandler;
    oncanplay: TJSEventHandler;
    oncanplaythrough: TJSEventHandler;
    onchange: TJSEventHandler;
    onclick: THTMLClickEventHandler;
    onclose: TJSEventHandler;
    oncontextmenu: TJSEventHandler;
    oncuechange: TJSEventHandler;
    ondblclick: THTMLClickEventHandler;
    ondrag: TJSDragDropEventHandler;
    ondragend: TJSDragDropEventHandler;
    ondragenter: TJSDragDropEventHandler;
    ondragexit: TJSDragDropEventHandler;
    ondragleave: TJSDragDropEventHandler;
    ondragover: TJSDragDropEventHandler;
    ondragstart: TJSDragDropEventHandler;
    ondrop: TJSDragDropEventHandler;
    ondurationchange: TJSEventHandler;
    onemptied: TJSEventHandler;
    onended: TJSEventHandler;
    onerror: TJSErrorEventHandler;
    onfocus: TJSFocusEventhandler;
    ongotpointercapture: TJSPointerEventHandler;
    oninput: TJSEventhandler;
    oninvalid: TJSEventhandler;
    onkeydown: TJSKeyEventhandler;
    onkeypress: TJSKeyEventhandler;
    onkeyup: TJSKeyEventhandler;
    onload: TJSEventhandler;
    onloadeddata: TJSEventhandler;
    onloadedmetadata: TJSEventhandler;
    onloadend: TJSProgressEventhandler;
    onloadstart: TJSProgressEventhandler;
    onlostpointercapture: TJSPointerEventHandler;
    onmessage: TJSEventHandler;
    onmousedown: TJSMouseEventHandler;
    onmouseenter: TJSMouseEventHandler;
    onmouseleave: TJSMouseEventHandler;
    onmousemove: TJSMouseEventHandler;
    onmouseout: TJSMouseEventHandler;
    onmouseover: TJSMouseEventHandler;
    onmouseup: TJSMouseEventHandler;
    onmousewheel: TJSMouseEventHandler;
    onpause: TJSEventHandler;
    onplay: TJSEventHandler;
    onplaying: TJSEventHandler;
    onpointercancel: TJSPointerEventHandler;
    onpointerdown: TJSPointerEventHandler;
    onpointerenter: TJSPointerEventHandler;
    onpointerleave: TJSPointerEventHandler;
    onpointerlockchange: TJSPointerEventHandler;
    onpointermove: TJSPointerEventHandler;
    onpointerout: TJSPointerEventHandler;
    onpointerover: TJSPointerEventHandler;
    onpointerup: TJSPointerEventHandler;
    onprogress: TJSProgressEventhandler;
    onratechange: TJSEventHandler;
    onreset: TJSUIEventHandler;
    onscroll: TJSUIEventHandler;
    onseekend: TJSEventHandler;
    onseeking: TJSEventHandler;
    onselect: TJSEventHandler;
    onselectionchange: TJSEventHandler;
    onsshow: TJSEventHandler;
    onsubmit: TJSEventHandler;
    onunload: TJSUIEventHandler;
    onwaiting: TJSEventHandler;
    selectedStyleSheetSet: string;
    title: string;
    function adoptNode(aExternalNode: TJSNode): TJSNode;
    procedure close;
    function createAttribute(aName: string): TJSAttr;
    function createCDATASection(S: string): TJSNode;
    function createComment(S: string): TJSNode;
    function createDocumentFragment: TJSDocumentFragment;
    function createElement(tagName: string): TJSElement; overload;
    function createElement(tagName: string; const options: TJSElementCreationOptions): TJSElement; overload;
    function createElementNS(aNameSpace: string; tagName: string): TJSElement; overload;
    function createElementNS(aNameSpace: string; tagName: string; const options: TJSElementCreationOptions): TJSElement; overload;
    function createEvent(aType: string): TJSEvent;
    function createExpression(xPathText: string; aNameSpaceMapper: TJSNameSpaceMapperCallback): TJSXPathExpression;
    function createNSResolver(aNode: TJSNode): TJSXPathNSResolver;
    function createProcessingInstruction(target: string; data: string): TJSProcessingInstruction;
    function createRange: TJSRange;
    function createTextNode(S: string): TJSNode;
    function createTreeWalker(root: TJSNode; whatToShow: NativeInt; filter: TJSNodeFilter): TJSTreeWalker;
    function elementFromPoint(x: Integer; y: Integer): TJSElement;
    procedure enableStyleSheetsForSet(aSet: string);
    function evaluate(xpathExpression: string; ContextNode: TJSNode; NameSpaceResolver: TJSNamespaceMapperCallBack; resultType: NativeInt; aResult: TJSXPathResult): TJSXPathResult;
    procedure exitFullScreen;
    function getElementById(aID: string): TJSElement;
    function getElementsByClassName(aNames: string): TJSHTMLCollection;
    function getElementsByName(aName: string): TJSNodeList;
    function getElementsByTagName(aName: string): TJSHTMLCollection;
    function getElementsByTagNameNS(aNameSpace: string; aName: string): TJSHTMLCollection;
    function getSelection: TJSSelection;
    function hasFocus: Boolean;
    function importNode(aExternalNode: TJSNode; Deep: Boolean): TJSNode;
    procedure open;
    function querySelector(aSelectors: string): TJSElement;
    function querySelectorAll(aSelectors: string): TJSNodeList;
    procedure releaseCapture;
    procedure write(aLine: string);
    procedure writeln(aLine: string);
    property activeElement: TJSElement read FActiveElement;
    property characterSet: string read FCharacterSet;
    property childElementCount: NativeInt read FChildElementCount;
    property compatMode: string read FCompatMode;
    property currentScript: TJSElement read FCurrentScript;
    property defaultView: TJSWindow read FDefaultView;
    property doctype: TJSDocumentType read FDocType;
    property documentElement: TJSElement read FDocumentElement;
    property documentURI: string read FDocumentURI;
    property embeds: TJSHTMLCollection read FEmbeds;
    property firstElementChild: TJSElement read FFirstElementChild;
    property Forms: TJSHTMLCollection read FForms;
    property fullscreenElement: TJSElement read FFullScreenElement;
    property fullscreenEnabled: Boolean read FFullscreenEnabled;
    property head: TJSElement read FHead;
    property hidden: Boolean read FHidden;
    property images: TJSHTMLCollection read FImages;
    property lastElementChild: TJSElement read FLastElementChild;
    property lastModified: string read FLastModified;
    property lastStyleSheetSet: string read FLastStyleSheetSet;
    property links: TJSHTMLCollection read FLinks;
    property location: TJSLocation read FLocation;
    property LocationString: string read FURL write FLocationString;
    property plugins: TJSHTMLCollection read FPlugins;
    property pointerLockElement: TJSElement read FPointerLockElement;
    property preferredStyleSheetSet: string read FPreferredStyleSheetSet;
    property readyState: string read FReadyState;
    property referrer: string read FReferrer;
    property scripts: TJSHTMLCollection read FScripts;
    property styleSheets: TJSStyleSheetList read FStyleSheets;
    property styleSheetSets: TJSValueDynArray read FStyleSheetSets;
    property URL: string read FURL;
    property visibilityState: string read FVisibilityState;
    property _implementation: TJSDOMImplementation read FImplementation;
  end;

  TJSConsole = class(TObject)
  public
    procedure assert(anAssertion: string; Obj1: JSValue);
    procedure clear;
    procedure count; overload;
    procedure count(aCounter: string); overload;
    procedure error(Obj1: JSValue);
    procedure group;
    procedure groupCollapsed;
    procedure groupEnd;
    procedure info(Obj1: JSValue);
    procedure log(Obj1: JSValue);
    procedure table(args: array of JSValue);
    procedure time(aName: string);
    procedure timeEnd(aName: string);
    procedure trace;
    procedure warn(Obj1: JSValue);
  end;

  TPromiseResolverFunc = procedure(aValue: JSValue);
  TPromiseResolverMethod = procedure(aValue: JSValue);
  TJSPromiseExecutorFunc = procedure(resolve: TPromiseResolverFunc; reject: TPromiseResolverFunc);
  TJSPromiseExecutorMethod = procedure(resolve: TPromiseResolverFunc; reject: TPromiseResolverFunc) of object;

  TJSPromise = class(TObject)
  public
    constructor new(Executor: TJSPromiseExecutorFunc); overload;
    constructor new(Executor: TJSPromiseExecutorMethod); overload;
    function catch(onRejected: TPromiseResolverFunc): TJSPromise; overload;
    function catch(onRejected: TPromiseResolverMethod): TJSPromise; overload;
    class function reject(reason: JSValue): TJSPromise;
    class function resolve(value: JSValue): TJSPromise;
    function _then(onRejected: TPromiseResolverFunc): TJSPromise; overload;
    function _then(onRejected: TPromiseResolverMethod): TJSPromise; overload;
  end;

  TJSCryptoKey = class(TObject)
  strict private
    FAlgorithm: JSValue;
    FExtractable: Boolean;
    FType: string;
    FUsages: TStringDynArray;
  public
    property algorithm: JSValue read FAlgorithm;
    property extractable: Boolean read FExtractable;
    property usages: TStringDynArray read FUsages;
    property _type: string read FType;
  end;

  TJSBufferSource = class(TObject)
  end;

  TJSTypedArray = class(TObject)
  end;

  TJSSubtleCrypto = class(TObject)
  public
    function decrypt(algorithm: JSValue; aKey: TJSCryptoKey; aData: TJSBufferSource): TJSPromise;
    function deriveKey(algorithm: JSValue; aMasterKey: TJSCryptoKey; aDerivedAlgo: JSValue; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
    function digest(algorithm: string; Buffer: TJSArrayBuffer): TJSPromise; overload;
    function digest(algorithm: string; Buffer: TJSTypedArray): TJSPromise; overload;
    function encrypt(algorithm: JSValue; aKey: TJSCryptoKey; aData: TJSBufferSource): TJSPromise;
    function exportKey(algorithm: string; AKey: TJSCryptoKey): TJSPromise;
    function generateKey(algorithm: JSValue; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
    function importKey(format: string; KeyData: TJSArrayBuffer; algorithm: string; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
    function sign(algorithm: string; Key: TJSCryptoKey; aText: TJSArrayBuffer): TJSPromise; overload;
    function unwrapKey(algorithm: string; wrappedKey: TJSArrayBuffer; Key: TJSCryptoKey; unwrapAlgo: string; unwrappedKeyAlgo: string; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
    function verify(algorithm: string; key: TJSCryptoKey; Signature: TJSArrayBuffer; textToVerify: TJSArrayBuffer): TJSPromise;
    function wrapKey(aFormat: string; Key: TJSCryptoKey; WrappingKey: TJSCryptoKey; WrapAlgorithm: string): TJSPromise;
  end;

  TJSCrypto = class(TObject)
  strict private
    FsubtleCrypto: TJSSubtleCrypto;
  public
    procedure getRandomValues(anArray: TJSTypedArray);
    property subtleCrypto: TJSSubtleCrypto read FsubtleCrypto;
  end;

  TJSHistory = class(TObject)
  strict private
    FLength: NativeInt;
  public
    procedure back;
    procedure forward;
    procedure go; overload;
    procedure go(aIndex: NativeInt); overload;
    procedure pushState(aState: jsValue; aTitle: string); overload;
    procedure pushState(aState: jsValue; aTitle: string; AURL: string); overload;
    procedure replaceState(aState: jsValue); overload;
    procedure replaceState(aState: jsValue; aTitle: string); overload;
    procedure replaceState(aState: jsValue; aTitle: string; AURL: string); overload;
    property length: NativeInt read FLength;
  end;

  TJSIDBTransactionMode = class(TObject)
  public const
    readonly = 'readonly';
    readwrite = 'readwrite';
    versionchange = 'versionchange';
  end;

  TJSIDBTransaction = class(TJSEventTarget)
  strict private
    FDB: TIDBDatabase;
    FError: JSValue;
    FMode: string;
    FObjectStoreNames: TStringDynArray;
  public
    procedure abort;
    function objectStore(aName: string): TJSIDBObjectStore;
    property db: TIDBDatabase read FDB;
    property error: JSValue read FError;
    property mode: string read FMode;
    property objectStoreNames: TStringDynArray read FObjectStoreNames;
  end;

  TJSIDBKeyRange = class(TObject)
  strict private
    FLower: JSValue;
    FLowerOpen: Boolean;
    FUpper: JSValue;
    FUpperOpen: Boolean;
  public
    class function bound(aLower: JSValue; aUpper: JSValue): TJSIDBKeyRange; overload;
    class function bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean): TJSIDBKeyRange; overload;
    class function bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean; aUpperOpen: Boolean): TJSIDBKeyRange; overload;
    function includes(aValue: JSValue): Boolean;
    class function lowerBound(aLower: JSValue): TJSIDBKeyRange; overload;
    class function lowerBound(aLower: JSValue; aOpen: Boolean): TJSIDBKeyRange; overload;
    class function only(aValue: JSValue): TJSIDBKeyRange;
    class function upperBound(aUpper: JSValue): TJSIDBKeyRange; overload;
    class function upperBound(aUpper: JSValue; aOpen: Boolean): TJSIDBKeyRange; overload;
    property lower: JSValue read FLower;
    property lowerOpen: Boolean read FLowerOpen;
    property upper: JSValue read FUpper;
    property upperOpen: Boolean read FUpperOpen;
  end;

  TJSIDBIndexParameters = record
  public
    locale : string;
    multiEntry : Boolean;
    unique : Boolean;
  end;

  TJSIDBIndex = class(TObject)
  strict private
    FKeyPath: JSValue;
    FMultiEntry: Boolean;
    FObjectStore: TJSIDBObjectStore;
    FUnique: Boolean;
  public
    name: string;
    function count: TJSIDBRequest;
    function get(aKey: jsValue): TJSIDBRequest; overload;
    function get(aKey: TJSIDBKeyRange): TJSIDBRequest; overload;
    function getAll(aKey: jsValue): TJSIDBRequest; overload;
    function getAll(aKey: jsValue; ACount: NativeInt): TJSIDBRequest; overload;
    function getAll(aKey: TJSIDBKeyRange): TJSIDBRequest; overload;
    function getAll(aKey: TJSIDBKeyRange; ACount: NativeInt): TJSIDBRequest; overload;
    function getAllKeys(aKey: jsValue): TJSIDBRequest; overload;
    function getAllKeys(aKey: jsValue; ACount: NativeInt): TJSIDBRequest; overload;
    function getAllKeys(aKey: TJSIDBKeyRange): TJSIDBRequest; overload;
    function getAllKeys(aKey: TJSIDBKeyRange; ACount: NativeInt): TJSIDBRequest; overload;
    function getKey(aKey: jsValue): TJSIDBRequest;
    function openCursor: TJSIDBRequest; overload;
    function openCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function openCursor(aKeyRange: TJSIDBKeyRange; ADirection: string): TJSIDBRequest; overload;
    function openKeyCursor: TJSIDBRequest; overload;
    function openKeyCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function openKeyCursor(aKeyRange: TJSIDBKeyRange; ADirection: string): TJSIDBRequest; overload;
    property keyPath: JSValue read FKeyPath;
    property multiEntry: Boolean read FMultiEntry;
    property objectStore: TJSIDBObjectStore read FObjectStore;
    property unique: Boolean read FUnique;
  end;

  TJSIDBCursorDirection = class(TObject)
  public const
    next = 'next';
    nextUnique = 'nextUnique';
    prev = 'prev';
    prevUnique = 'prevUnique';
  end;

  TJSIDBCursor = class(TObject)
  strict private
    FDirection: string;
    FKey: JSValue;
    FPrimaryKey: JSValue;
    FSource: JSValue;
    FSourceAsIndex: TJSIDBIndex;
    FSourceAsStore: TJSIDBObjectStore;
  public
    procedure advance(aKey: JSValue); overload;
    procedure advance(aCount: NativeInt); overload;
    procedure continue; overload;
    procedure continue(aKey: JSValue); overload;
    procedure continuePrimaryKey(aKey: JSValue); overload;
    procedure continuePrimaryKey(aKey: JSValue; aPrimaryKey: JSValue); overload;
    procedure delete;
    procedure update(aValue: JSValue);
    property direction: string read FDirection;
    property key: JSValue read FKey;
    property primaryKey: JSValue read FPrimaryKey;
    property source: JSValue read FSource;
    property sourceAsIndex: TJSIDBIndex read FSourceAsIndex;
    property sourceAsStore: TJSIDBObjectStore read FSourceAsStore;
  end;

  TJSIDBObjectStore = class(TJSEventTarget)
  public
    function add(aValue: JSValue): TJSIDBRequest; overload;
    function add(aValue: JSValue; aKey: string): TJSIDBRequest; overload;
    function clear: TJSIDBRequest;
    function count: TJSIDBRequest; overload;
    function count(aKey: string): TJSIDBRequest; overload;
    function count(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function createIndex(aIndexName: string; KeyPath: string): TJSIDBIndex; overload;
    function createIndex(aIndexName: string; KeyPath: string; Options: TJSIDBIndexParameters): TJSIDBIndex; overload;
    function createIndex(aIndexName: string; KeyPath: array of string): TJSIDBIndex; overload;
    function createIndex(aIndexName: string; KeyPath: array of string; Options: TJSIDBIndexParameters): TJSIDBIndex; overload;
    function delete(aKey: string): TJSIDBRequest; overload;
    function delete(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    procedure deleteIndex(aIndexName: string);
    function get(aKey: string): TJSIDBRequest; overload;
    function get(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function getAll: TJSIDBRequest; overload;
    function getAll(aKey: string): TJSIDBRequest; overload;
    function getAll(aKey: string; aCount: NativeInt): TJSIDBRequest; overload;
    function getAll(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function getAll(aKeyRange: TJSIDBKeyRange; aCount: NativeInt): TJSIDBRequest; overload;
    function getAllKeys(aKey: string): TJSIDBRequest; overload;
    function getAllKeys(aKey: string; aCount: NativeInt): TJSIDBRequest; overload;
    function getAllKeys(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function getAllKeys(aKeyRange: TJSIDBKeyRange; aCount: NativeInt): TJSIDBRequest; overload;
    function getKey(aKey: string): TJSIDBRequest; overload;
    function getKey(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function index(aIndexName: string): TJSIDBIndex;
    function openCursor: TJSIDBRequest; overload;
    function openCursor(aKey: string): TJSIDBRequest; overload;
    function openCursor(aKey: string; aDirection: string): TJSIDBRequest; overload;
    function openCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function openCursor(aKeyRange: TJSIDBKeyRange; aDirection: string): TJSIDBRequest; overload;
    function openKeyCursor: TJSIDBRequest; overload;
    function openKeyCursor(aKey: string): TJSIDBRequest; overload;
    function openKeyCursor(aKey: string; aDirection: string): TJSIDBRequest; overload;
    function openKeyCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest; overload;
    function openKeyCursor(aKeyRange: TJSIDBKeyRange; aDirection: string): TJSIDBRequest; overload;
    function put(aValue: JSValue): TJSIDBRequest; overload;
    function put(aValue: JSValue; aKey: string): TJSIDBRequest; overload;
    property Indexes[aIndexName : string]: TJSIDBIndex read index;
  end;

  TJSIDBRequest = class(TJSEventTarget)
  strict private
    Ferror: JSValue;
    FReadyState: string;
    FResult: JSValue;
    FResultCursor: TJSIDBCursor;
    FResultDatabase: TIDBDatabase;
    FResultIndex: TJSIDBIndex;
    FResultObjectStore: TJSIDBObjectStore;
    FSource: JSValue;
    FSourceCursor: TJSIDBCursor;
    FSourceDatabase: TIDBDatabase;
    FSourceIndex: TJSIDBIndex;
    FSourceObjectStore: TJSIDBObjectStore;
    FTransaction: TJSIDBTransaction;
  public
    onerror: TJSEventHandler;
    onsuccess: TJSEventHandler;
    property error: JSValue read FError;
    property readyState: string read FReadyState;
    property result: JSValue read FResult;
    property resultAsCursor: TJSIDBCursor read FResultCursor;
    property resultAsDatabase: TIDBDatabase read FResultDatabase;
    property resultAsIndex: TJSIDBIndex read FResultIndex;
    property resultAsObjectStore: TJSIDBObjectStore read FResultObjectStore;
    property source: JSValue read FSource;
    property sourceAsCursor: TJSIDBCursor read FSourceCursor;
    property sourceAsDatabase: TIDBDatabase read FSourceDatabase;
    property sourceAsIndex: TJSIDBIndex read FSourceIndex;
    property sourceAsObjectStore: TJSIDBObjectStore read FSourceObjectStore;
    property transaction: TJSIDBTransaction read FTransaction;
  end;

  TJSIDBOpenDBRequest = class(TJSIDBRequest)
  public
    onblocked: TJSEventHandler;
    onupgradeneeded: TJSEventHandler;
  end;

  TJSCreateObjectStoreOptions = record
  public
    autoIncrement : Boolean;
    keyPath : jsValue;
  end;

  TIDBDatabase = class(TJSEventTarget)
  strict private
    FName: string;
    FobjectStoreNames: TStringDynArray;
    FVersion: Integer;
  public
    procedure close;
    function createObjectStore(aName: string): TJSIDBObjectStore; overload;
    function createObjectStore(aName: string; Options: TJSCreateObjectStoreOptions): TJSIDBObjectStore; overload;
    procedure deleteObjectStore(aName: string);
    function transaction(aStoreNames: array of string): TJSIDBTransaction; overload;
    function transaction(aStoreNames: array of string; aMode: string): TJSIDBTransaction; overload;
    property name: string read FName;
    property objectStoreNames: TStringDynArray read FobjectStoreNames;
    property version: Integer read FVersion;
  end;

  TJSIDBFactory = class(TJSEventTarget)
  public
    function cmp(a: jsValue; b: jsValue): NativeInt;
    function deleteDatabase(aName: string): TJSIDBOpenDBRequest;
    function open(aName: string): TJSIDBOpenDBRequest; overload;
    function open(aName: string; aVersion: Integer): TJSIDBOpenDBRequest; overload;
  end;

  TJSStorage = class(TJSEventTarget)
  strict private
    FLength: NativeInt;
  public
    procedure clear;
    function getItem(aKeyName: string): string;
    function key(aIndex: Integer): string;
    procedure removeItem(aKeyName: string);
    procedure setItem(aKeyName: string; aValue: string);
    property Items[aKeyName : string]: string read getItem write setItem; default;
    property Keys[AIndex : Integer]: string read key;
    property length: NativeInt read FLength;
  end;

  TJSVisibleItem = class(TObject)
  strict private
    FVisible: Boolean;
  public
    property visible: Boolean read FVisible;
  end;

  TJSLocationBar = class(TJSVisibleItem)
  end;

  TJSMenuBar = class(TJSVisibleItem)
  end;

  TJSToolBar = class(TJSVisibleItem)
  end;

  TJSPersonalBar = class(TJSVisibleItem)
  end;

  TJSScrollBars = class(TJSVisibleItem)
  end;

  TJSPositionError = record
  public
    code : Integer;
    message : string;
  end;

  TJSPositionOptions = record
  public
    enableHighAccuracy : Boolean;
    maximumAge : Integer;
    timeout : Integer;
  end;

  TJSCoordinates = record
  public
    accuracy : Double;
    altitude : Double;
    altitudeAccuracy : Double;
    heading : Double;
    latitude : Double;
    longitude : Double;
    speed : Double;
  end;

  TJSPosition = record
  public
    coords : TJSCoordinates;
    timestamp : string;
  end;

  TJSGeoLocationCallback = procedure(aPosition: TJSPosition);
  TJSGeoLocationEvent = procedure(aPosition: TJSPosition) of object;
  TJSGeoLocationErrorCallback = procedure(aValue: TJSPositionError);
  TJSGeoLocationErrorEvent = procedure(aValue: TJSPositionError) of object;

  TJSGeoLocation = class(TObject)
  public
    procedure clearWatch(AID: NativeInt);
    procedure getCurrentPosition(ASuccess: TJSGeoLocationCallback); overload;
    procedure getCurrentPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback); overload;
    procedure getCurrentPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback; AOptions: TJSPositionOptions); overload;
    function watchPosition(ASuccess: TJSGeoLocationCallback): NativeInt; overload;
    function watchPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback): NativeInt; overload;
    function watchPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback; AOptions: TJSPositionOptions): NativeInt; overload;
  end;

  TJSMediaDevices = class(TJSEventTarget)
  end;

  TJSWorker = class(TJSEventTarget)
  public
    constructor new(aURL: string);
    procedure postMessage(aValue: JSValue); overload;
    procedure postMessage(aValue: JSValue; aList: TJSValueDynArray); overload;
    procedure terminate;
  end;

  TJSMessagePort = class(TJSEventTarget)
  public
    procedure close;
    procedure postMessage(aValue: JSValue); overload;
    procedure postMessage(aValue: JSValue; aList: TJSValueDynArray); overload;
    procedure start;
  end;

  TJSSharedWorker = class(TJSEventTarget)
  strict private
    FPort: TJSMessagePort;
  public
    constructor new(aURL: string); overload;
    constructor new(aURL: string; aName: string); overload;
    property port: TJSMessagePort read FPort;
  end;

  TJSServiceWorker = class(TJSWorker)
  strict private
    FscriptURL: string;
    FState: string;
  public
    property scriptURL: string read FscriptURL;
    property state: string read FState;
  end;

  TJSServiceWorkerRegistration = class(TObject)
  strict private
    FActive: TJSServiceWorker;
    FInstalling: TJSServiceWorker;
    FScope: string;
    FWaiting: TJSServiceWorker;
  public
    function unregister: TJSPromise;
    procedure update;
    property active: TJSServiceWorker read FActive;
    property installing: TJSServiceWorker read FInstalling;
    property scope: string read FScope;
    property waiting: TJSServiceWorker read FWaiting;
  end;

  TJSServiceWorkerContainerOptions = record
  public
    scope : string;
  end;

  TJSServiceWorkerContainer = class(TObject)
  strict private
    FController: TJSServiceWorker;
    FReady: TJSPromise;
  public
    function getRegistration: TJSPromise; overload;
    function getRegistration(aURL: string): TJSPromise; overload;
    function getRegistrations: TJSPromise;
    function register(aURL: string): TJSPromise; overload;
    function register(aURL: string; aOptions: TJSServiceWorkerContainerOptions): TJSPromise; overload;
    property controller: TJSServiceWorker read FController;
    property ready: TJSPromise read FReady;
  end;

  TJSNavigator = class(TObject)
  strict private
    FCookieEnabled: Boolean;
    FGeoLocation: TJSGeoLocation;
    FLanguage: string;
    FMaxTouchPoints: NativeInt;
    FMediaDevices: TJSMediaDevices;
    FOnline: Boolean;
    FPlatform: string;
    FServiceWorker: TJSServiceWorkerContainer;
    FUserAgent: string;
  public
    function getBattery: TJSPromise;
    procedure registerContentHandler(aMimeType: string; aURI: string; aTitle: string);
    procedure registerProtocolHandler(aProtocol: string; aURI: string; aTitle: string);
    function requestMediaKeySystemAccess(aKeySystem: string; supportedConfigurations: TJSValueDynArray): TJSPromise;
    procedure vibrate(aPattern: array of NativeInt); overload;
    procedure vibrate(aPattern: NativeInt); overload;
    property cookieEnabled: Boolean read FCookieEnabled;
    property geoLocation: TJSGeoLocation read FGeoLocation;
    property language: string read FLanguage;
    property maxTouchPoints: NativeInt read FMaxTouchPoints;
    property mediaDevices: TJSMediaDevices read FMediaDevices;
    property onLine: Boolean read FOnline;
    property platform: string read FPlatform;
    property serviceWorker: TJSServiceWorkerContainer read FServiceWorker;
    property userAgent: string read FUserAgent;
  end;

  TJSPerformance = class(TJSObject)
  end;

  TJSScreen = class(TJSObject)
  end;

  TJSURL = class(TJSObject)
  end;

  TJSTimerCallBack = procedure;

  TJSMediaQueryList = class(TJSObject)
  strict private
    FMatches: Boolean;
    FMedia: string;
  public
    property matches: Boolean read FMatches;
    property media: string read FMedia;
  end;

  TJSWindowArray = array of TJSWindow;

  TJSWindow = class(TJSObject)
  strict private
    FClosed: Boolean;
    FConsole: TJSConsole;
    FCrypto: TJSCrypto;
    FDevicePixelRatio: Double;
    FDocument: TJSDocument;
    FFrameElement: TJSElement;
    FFrames: TJSWindowArray;
    FHistory: TJSHistory;
    FIndexedDB: TJSIDBFactory;
    FInnerheight: NativeInt;
    FInnerWidth: NativeInt;
    FLength: NativeInt;
    FLocalStorage: TJSStorage;
    FLocation: TJSLocation;
    FLocationBar: TJSLocationBar;
    FLocationString: TJSLocation;
    FMenuBar: TJSMenuBar;
    FNavigator: TJSNavigator;
    FOpener: TJSWindow;
    FOuterheight: NativeInt;
    FOuterWidth: NativeInt;
    FParent: TJSWindow;
    FPerformance: TJSPerformance;
    FPersonalBar: TJSPersonalBar;
    FScreen: TJSScreen;
    FScreenX: NativeInt;
    FScreenY: NativeInt;
    FScrollbar: TJSScrollBars;
    FScrollX: NativeInt;
    FScrollY: NativeInt;
    FSelf: TJSWindow;
    FSessionStorage: TJSStorage;
    FToolBar: TJSToolBar;
    FTop: TJSWindow;
    FURL: TJSURL;
  public
    fullSreen: Boolean;
    name: string;
    onabort: TJSEventHandler;
    onafterprint: TJSEventHandler;
    onbeforeinstallprompt: TJSEventHandler;
    onbeforeprint: TJSEventHandler;
    onbeforeunloadprompt: TJSEventHandler;
    onblur: TJSEventHandler;
    onchange: TJSEventHandler;
    onclick: THTMLClickEventHandler;
    onclose: TJSEventHandler;
    oncontextmenu: TJSEventHandler;
    ondblclick: THTMLClickEventHandler;
    onerror: TJSErrorEventHandler;
    onfocus: TJSFocusEventhandler;
    onhashchange: TJSHashChangeEventhandler;
    oninput: TJSEventhandler;
    onkeydown: TJSKeyEventhandler;
    onkeypress: TJSKeyEventhandler;
    onkeyup: TJSKeyEventhandler;
    onlanguagechange: TJSEventhandler;
    onload: TJSEventhandler;
    onloadend: TJSLoadEventhandler;
    onloadstart: TJSLoadEventhandler;
    onmessage: TJSEventHandler;
    onmousedown: TJSMouseEventHandler;
    onmouseenter: TJSMouseEventHandler;
    onmouseleave: TJSMouseEventHandler;
    onmousemove: TJSMouseEventHandler;
    onmouseout: TJSMouseEventHandler;
    onmouseover: TJSMouseEventHandler;
    onmouseup: TJSMouseEventHandler;
    onmousewheel: TJSMouseEventHandler;
    onoffline: TJSEventHandler;
    ononline: TJSEventHandler;
    onpagehide: TJSPageTransitionEventHandler;
    onpageshow: TJSPageTransitionEventHandler;
    onpaint: TJSEventHandler;
    onpointercancel: TJSPointerEventHandler;
    onpointerdown: TJSPointerEventHandler;
    onpointerenter: TJSPointerEventHandler;
    onpointerleave: TJSPointerEventHandler;
    onpointerlockchange: TJSPointerEventHandler;
    onpointermove: TJSPointerEventHandler;
    onpointerout: TJSPointerEventHandler;
    onpointerover: TJSPointerEventHandler;
    onpointerup: TJSPointerEventHandler;
    onpopstate: TJSPopStateEventHandler;
    onprogress: TJSProgressEventhandler;
    onreset: TJSUIEventHandler;
    onscroll: TJSUIEventHandler;
    onselect: TJSEventHandler;
    onselectionchange: TJSEventHandler;
    onstorage: TJSStorageEventHandler;
    onsubmit: TJSEventHandler;
    onunload: TJSUIEventHandler;
    status: string;
    procedure alert(const Msg: string);
    function atob(const aValue: string): string;
    procedure blur;
    function btoa(const aValue: string): string;
    procedure clearInterval(aID: NativeInt);
    procedure clearTimeout(aID: NativeInt);
    procedure close;
    function confirm(const aMsg: string): Boolean;
    procedure focus;
    function getComputedStyle(aElement: TJSElement): TJSCSSStyleDeclaration; overload;
    function getComputedStyle(aElement: TJSElement; aPseudoElement: TJSElement): TJSCSSStyleDeclaration; overload;
    function matchMedia(aQuery: string): TJSMediaQueryList;
    procedure moveBy(x: NativeInt; y: NativeInt);
    procedure moveTo(x: NativeInt; y: NativeInt);
    function open: TJSWindow; overload;
    function open(const aURL: string): TJSWindow; overload;
    function open(const aURL: string; const aTarget: string): TJSWindow; overload;
    function open(const aURL: string; const aTarget: string; AOptions: TJSObject): TJSWindow; overload;
    procedure print;
    function prompt(const aMessage: string): string; overload;
    function prompt(const aMessage: string; const aDefault: string): string; overload;
    procedure resizeBy(aWidth: NativeInt; aHeight: NativeInt);
    procedure resizeTo(aWidth: NativeInt; aHeight: NativeInt);
    procedure scrollBy(x: NativeInt; y: NativeInt);
    procedure scrollTo(x: NativeInt; y: NativeInt);
    function setInterval(ahandler: TJSTimerCallBack; aInterval: NativeUInt): NativeInt;
    function setTimeout(ahandler: TJSTimerCallBack; aTimeout: NativeUInt): NativeInt;
    procedure stop;
    property closed: Boolean read FClosed;
    property console: TJSConsole read FConsole;
    property crypto: TJSCrypto read FCrypto;
    property devicePixelRatio: Double read FDevicePixelRatio;
    property document: TJSDocument read FDocument;
    property frameElement: TJSElement read FFrameElement;
    property frames: TJSWindowArray read FFrames;
    property history: TJSHistory read FHistory;
    property indexedDB: TJSIDBFactory read FIndexedDB;
    property innerHeight: NativeInt read FInnerheight;
    property innerWidth: NativeInt read FInnerWidth;
    property length: NativeInt read FLength;
    property localStorage: TJSStorage read FLocalStorage;
    property location: TJSLocation read FLocation;
    property locationbar: TJSLocationBar read FLocationBar;
    property locationString: TJSLocation read FLocationString write FLocationString;
    property menubar: TJSMenuBar read FMenuBar;
    property navigator: TJSNavigator read FNavigator;
    property opener: TJSWindow read FOpener;
    property outerHeight: NativeInt read FOuterheight;
    property outerWidth: NativeInt read FOuterWidth;
    property parent: TJSWindow read FParent;
    property Performance: TJSPerformance read FPerformance;
    property personalbar: TJSPersonalBar read FPersonalBar;
    property screen: TJSScreen read FScreen;
    property screenX: NativeInt read FScreenX;
    property screenY: NativeInt read FScreenY;
    property scrollbar: TJSScrollBars read FScrollbar;
    property scrollX: NativeInt read FScrollX;
    property scrollY: NativeInt read FScrollY;
    property sessionStorage: TJSStorage read FSessionStorage;
    property toolbar: TJSToolBar read FToolBar;
    property top: TJSWindow read FTop;
    property URL: TJSURL read FURL;
    property _Self: TJSWindow read FSelf;
  end;

  TJSCSSStyleDeclaration = class
  strict private
    FLength: NativeInt;
    FParentRule: TJSCSSRule;
  public
    cssText: string;
    function getPropertyPriority(const aProperty: string): string;
    function getPropertyValue(const aProperty: string): string;
    function item(aIndex: Integer): string;
    function removeProperty(const aProperty: string): string;
    procedure setProperty(const aProperty: string; const aValue: string); overload;
    procedure setProperty(const aProperty: string; const aValue: string; const aPriority: string); overload;
    property length: NativeInt read FLength;
    property parentRule: TJSCSSRule read FParentRule;
  end;

  TJSHTMLElement = class(TJSElement)
  strict private
    FDataset: TJSObject;
    FIsContentEditable: Boolean;
    FOffsetHeight: Double;
    FOffsetLeft: Double;
    FOffsetParent: TJSElement;
    FOffsetTop: Double;
    FOffsetWidth: Double;
  public
    accessKey: string;
    contentEditable: string;
    dir: string;
    draggable: Boolean;
    hidden: Boolean;
    lang: string;
    onabort: TJSEventHandler;
    onanimationcancel: THTMLAnimationEventHandler;
    onanimationend: THTMLAnimationEventHandler;
    onblur: TJSEventHandler;
    onchange: TJSEventHandler;
    onclick: THTMLClickEventHandler;
    onclose: TJSEventHandler;
    oncontextmenu: TJSEventHandler;
    ondblclick: THTMLClickEventHandler;
    onerror: TJSErrorEventHandler;
    onfocus: TJSFocusEventhandler;
    ongotpointercapture: TJSPointerEventHandler;
    oninput: TJSEventhandler;
    onkeydown: TJSKeyEventhandler;
    onkeypress: TJSKeyEventhandler;
    onkeyup: TJSKeyEventhandler;
    onload: TJSEventhandler;
    onloadend: TJSLoadEventhandler;
    onloadstart: TJSLoadEventhandler;
    onlostpointercapture: TJSPointerEventHandler;
    onmousedown: TJSMouseEventHandler;
    onmouseenter: TJSMouseEventHandler;
    onmouseleave: TJSMouseEventHandler;
    onmousemove: TJSMouseEventHandler;
    onmouseout: TJSMouseEventHandler;
    onmouseover: TJSMouseEventHandler;
    onmouseup: TJSMouseEventHandler;
    onmousewheel: TJSMouseEventHandler;
    onpointercancel: TJSPointerEventHandler;
    onpointerdown: TJSPointerEventHandler;
    onpointerenter: TJSPointerEventHandler;
    onpointerleave: TJSPointerEventHandler;
    onpointerlockchange: TJSPointerEventHandler;
    onpointermove: TJSPointerEventHandler;
    onpointerout: TJSPointerEventHandler;
    onpointerover: TJSPointerEventHandler;
    onpointerup: TJSPointerEventHandler;
    onprogress: TJSProgressEventhandler;
    onreset: TJSUIEventHandler;
    onscroll: TJSUIEventHandler;
    onselect: TJSEventHandler;
    onsubmit: TJSEventHandler;
    spellcheck: Boolean;
    style: TJSCSSStyleDeclaration;
    tabIndex: Integer;
    title: string;
    procedure blur;
    procedure click;
    procedure focus;
    property dataset: TJSObject read FDataset;
    property isContentEditable: Boolean read FIsContentEditable;
    property offsetHeight: Double read FOffsetHeight;
    property offsetLeft: Double read FOffsetLeft;
    property offsetParent: TJSElement read FOffsetParent;
    property offsetTop: Double read FOffsetTop;
    property offsetWidth: Double read FOffsetWidth;
  end;

  TJSHTMLFormControlsCollection = class(TJSHTMLCollection)
  public
    function namedItem(S: string): TJSElement;
    property Items[S : string]: TJSElement read namedItem; default;
  end;

  TJSHTMLFormElement = class(TJSHTMLElement)
  strict private
    FElements: TJSHTMLFormControlsCollection;
    FLength: NativeInt;
  public
    acceptCharset: string;
    action: string;
    autocomplete: string;
    encoding: string;
    enctype: string;
    method: string;
    noValidate: Boolean;
    target: string;
    property elements: TJSHTMLFormControlsCollection read FElements;
    property length: NativeInt read FLength;
  end;

  TJSValidityState = class(TObject)
  strict private
    FBadInput: Boolean;
    FCustomError: Boolean;
    FPatternMismatch: Boolean;
    FRangeOverflow: Boolean;
    FRangeUnderflow: Boolean;
    FStepMismatch: Boolean;
    FTooLong: Boolean;
    FTooShort: Boolean;
    FTypeMismatch: Boolean;
    FValid: Boolean;
    FValueMissing: Boolean;
  public
    property badInput: Boolean read FBadInput;
    property customError: Boolean read FCustomError;
    property patternMisMatch: Boolean read FPatternMismatch;
    property rangeOverflow: Boolean read FRangeOverflow;
    property rangeUnderflow: Boolean read FRangeUnderflow;
    property stepMismatch: Boolean read FStepMismatch;
    property tooLong: Boolean read FTooLong;
    property tooShort: Boolean read FTooShort;
    property typeMisMatch: Boolean read FTypeMismatch;
    property valid: Boolean read FValid;
    property valueMissing: Boolean read FValueMissing;
  end;

  TJSBlob = class(TJSEventTarget)
  strict private
    FSize: NativeInt;
    FType: string;
  public
    procedure close;
    function slice: TJSBlob; overload;
    function slice(aStart: NativeInt): TJSBlob; overload;
    function slice(aStart: NativeInt; aEnd: NativeInt): TJSBlob; overload;
    function slice(aStart: NativeInt; aEnd: NativeInt; AContentType: string): TJSBlob; overload;
    property size: NativeInt read FSize;
    property _type: string read FType;
  end;

  TJSHTMLFile = class(TJSBlob)
  strict private
    FLastModified: NativeInt;
    FLastModifiedDate: TJSDate;
    FName: string;
  public
    property lastModified: NativeInt read FLastModified;
    property lastModifiedDate: TJSDate read FLastModifiedDate;
    property name: string read FName;
  end;

  TJSHTMLFileList = class(TJSEventTarget)
  strict private
    FLength: NativeInt;
  public
    function item(aIndex: NativeInt): TJSHTMLFile;
    property Files[aIndex : NativeInt]: TJSHTMLFile read item; default;
    property length: NativeInt read FLength;
  end;

  TJSHTMLInputElement = class(TJSHTMLElement)
  strict private
    FFiles: TJSHTMLFileList;
    FForm: TJSHTMLFormElement;
    FLabels: TJSNodeList;
    FList: TJSHTMLElement;
    FValidationmMessage: string;
    FValidity: TJSValidityState;
    FWillValidate: Boolean;
  public
    accept: string;
    align: string;
    allowDirs: Boolean;
    alt: string;
    autocapitalize: string;
    autocomplete: string;
    autofocus: Boolean;
    checked: Boolean;
    defaultChecked: string;
    defaultValue: string;
    dirName: string;
    disabled: Boolean;
    formAction: string;
    formEncType: string;
    formMethod: string;
    formNoValidate: Boolean;
    formTarget: string;
    height: Boolean;
    indeterminate: Boolean;
    inputMode: string;
    max: string;
    maxLength: NativeInt;
    min: string;
    minLength: NativeInt;
    multiple: Boolean;
    pattern: string;
    placeholder: string;
    readOnly: Boolean;
    required: Boolean;
    selectionDirection: string;
    selectionEnd: NativeInt;
    selectionStart: NativeInt;
    size: NativeInt;
    src: string;
    step: string;
    useMap: string;
    value: string;
    width: NativeInt;
    _type: string;
    function checkValidity: Boolean;
    function reportValidity: Boolean;
    procedure select;
    procedure setCustomValidity(aText: string);
    procedure setRangeText(aText: string; selectionStart: NativeInt; selectionEnd: NativeInt); overload;
    procedure setRangeText(aText: string; selectionStart: NativeInt; selectionEnd: NativeInt; Direction: string); overload;
    procedure setSelectionRange(selectionStart: NativeInt; selectionEnd: NativeInt); overload;
    procedure setSelectionRange(selectionStart: NativeInt; selectionEnd: NativeInt; Direction: string); overload;
    procedure stepUp; overload;
    procedure stepUp(n: Integer); overload;
    property files: TJSHTMLFileList read FFiles;
    property form: TJSHTMLFormElement read FForm;
    property labels: TJSNodeList read FLabels;
    property list: TJSHTMLElement read FList;
    property validationMessage: string read FValidationmMessage;
    property validity: TJSValidityState read FValidity;
    property willValidate: Boolean read FWillValidate;
  end;

  TJSHTMLOptionElement = class(TJSHTMLElement)
  strict private
    FForm: TJSHTMLFormElement;
    FIndex: NativeInt;
  public
    defaultSelected: Boolean;
    disabled: Boolean;
    selected: Boolean;
    text: string;
    value: string;
    _label: string;
    constructor New; overload;
    constructor New(aText: string); overload;
    constructor New(aText: string; aValue: string); overload;
    constructor New(aText: string; aValue: string; aDefaultSelected: Boolean); overload;
    constructor New(aText: string; aValue: string; aDefaultSelected: Boolean; Selected: Boolean); overload;
    property form: TJSHTMLFormElement read FForm;
    property index: NativeInt read FIndex;
  end;

  TJSHTMLOptGroupElement = class(TJSHTMLElement)
  end;

  TJSHTMLOptionsCollection = class(TJSHTMLCollection)
  end;

  TJSHTMLSelectElement = class(TObject)
  strict private
    FLength: NativeInt;
    FOptions: TJSHTMLOptionsCollection;
    FSelectedOptions: TJSHTMLCollection;
  public
    multiple: Boolean;
    name: string;
    selectedIndex: NativeInt;
    size: NativeInt;
    procedure add(anItem: TJSHTMLOptionElement); overload;
    procedure add(anItem: TJSHTMLOptionElement; before: TJSHTMLOptionElement); overload;
    function item(aIndex: NativeInt): TJSHTMLOptionElement;
    function namedItem(aName: string): TJSHTMLOptionElement;
    procedure remove(aIndex: NativeInt);
    property length: NativeInt read FLength;
    property options: TJSHTMLOptionsCollection read FOptions;
    property selectedOptions: TJSHTMLCollection read FSelectedOptions;
  end;

  THTMLCanvasToBlobCallback = function(aBlob: TJSBlob) : Boolean;

  TJSHTMLCanvasElement = class(TJSHTMLElement)
  public
    height: Integer;
    width: Integer;
    function getContext(contextType: string): TJSObject; overload;
    function getContext(contextType: string; contextAttributes: TJSObject): TJSObject; overload;
    function getContextAs2DContext(contextType: string): TJSCanvasRenderingContext2D; overload;
    function getContextAs2DContext(contextType: string; contextAttributes: TJSObject): TJSCanvasRenderingContext2D; overload;
    procedure toBlob(aCallBack: THTMLCanvasToBlobCallback; aMimeType: string); overload;
    procedure toBlob(aCallBack: THTMLCanvasToBlobCallback; aMimeType: string; aQuality: Double); overload;
    function toDataURL: string; overload;
    function toDataURL(aMimeType: string): string; overload;
    function toDataURL(aMimeType: string; aQuality: Double): string; overload;
  end;

  TJSCanvasGradient = class(TObject)
  end;

  TJSCanvasPattern = class(TObject)
  end;

  TJSPath2D = class(TObject)
  end;

  TJSImageData = class(TObject)
  strict private
    FData: TJSUint8ClampedArray;
    FHeight: Integer;
    FWidth: Integer;
  public
    constructor new(awidth: Integer; aheight: Integer); overload;
    constructor new(anArray: TJSUint8ClampedArray; awidth: Integer; aheight: Integer); overload;
    property data: TJSUint8ClampedArray read FData;
    property height: Integer read FHeight;
    property width: Integer read FWidth;
  end;

  TCanvasCoordType = Double;

  TJSTextMetrics = record
  public
    actualBoundingBoxAscent : TCanvasCoordType;
    actualBoundingBoxDescent : TCanvasCoordType;
    actualBoundingBoxLeft : TCanvasCoordType;
    actualBoundingBoxRight : TCanvasCoordType;
    alphabeticBaseline : TCanvasCoordType;
    emHeightAscent : TCanvasCoordType;
    emHeightDescent : TCanvasCoordType;
    fontBoundingBoxAscent : TCanvasCoordType;
    fontBoundingBoxDescent : TCanvasCoordType;
    hangingBaseline : TCanvasCoordType;
    ideographicBaseline : TCanvasCoordType;
    width : TCanvasCoordType;
  end;

  TJSCanvasRenderingContext2D = class(TObject)
  strict private
    FCanvas: TJSHTMLCanvasElement;
    FfillStyleColor: string;
    FfillStyleGradient: TJSCanvasGradient;
    FfillStylePattern: TJSCanvasPattern;
    FstrokeStyleColor: string;
    FstrokeStyleGradient: TJSCanvasGradient;
    FstrokeStylePattern: TJSCanvasPattern;
  public
    fillStyle: JSValue;
    font: string;
    globalAlpha: Double;
    globalCompositeOperation: string;
    lineCap: string;
    lineDashOffset: Double;
    lineJoin: string;
    lineWidth: Double;
    miterLimit: Double;
    shadowBlur: Double;
    shadowColor: string;
    shadowOffsetX: Double;
    shadowOffsetY: Double;
    strokeStye: JSValue;
    textAlign: string;
    textBaseline: string;
    procedure arc(x: TCanvasCoordType; y: TCanvasCoordType; radius: TCanvasCoordType; startAngle: TCanvasCoordType; endAngle: TCanvasCoordType); overload;
    procedure arc(x: TCanvasCoordType; y: TCanvasCoordType; radius: TCanvasCoordType; startAngle: TCanvasCoordType; endAngle: TCanvasCoordType; antiClockWise: Boolean); overload;
    procedure arcTo(x1: TCanvasCoordType; y1: TCanvasCoordType; x2: TCanvasCoordType; y2: TCanvasCoordType; radius: TCanvasCoordType); overload;
    procedure beginPath;
    procedure bezierCurveTo(cp1x: TCanvasCoordType; cp1y: TCanvasCoordType; cp2x: TCanvasCoordType; cp2y: TCanvasCoordType; x: TCanvasCoordType; y: TCanvasCoordType); overload;
    procedure clearRect(x: TCanvasCoordType; y: TCanvasCoordType; width: TCanvasCoordType; height: TCanvasCoordType);
    procedure clip; overload;
    procedure clip(aFillRule: string); overload;
    procedure clip(aPath: TJSPath2D); overload;
    procedure closePath;
    function createImageData(aWidth: Integer; aHeight: Integer): TJSImageData; overload;
    function createImageData(aImage: TJSImageData): TJSImageData; overload;
    function createLinearGradient(x0: TCanvasCoordType; y0: TCanvasCoordType; x1: TCanvasCoordType; y1: TCanvasCoordType): TJSCanvasGradient;
    function createPattern(aImage: TJSObject; repetition: string): TJSCanvasPattern;
    function createRadialGradient(x0: TCanvasCoordType; y0: TCanvasCoordType; r0: TCanvasCoordType; x1: TCanvasCoordType; y1: TCanvasCoordType; r1: TCanvasCoordType): TJSCanvasGradient;
    procedure drawFocusIfNeeded(aElement: TJSElement); overload;
    procedure drawFocusIfNeeded(aPath: TJSPath2D; aElement: TJSElement); overload;
    procedure drawImage(image: TJSObject; dx: TCanvasCoordType; dy: TCanvasCoordType); overload;
    procedure drawImage(image: TJSObject; dx: TCanvasCoordType; dy: TCanvasCoordType; dwidth: TCanvasCoordType; dheight: TCanvasCoordType); overload;
    procedure drawImage(image: TJSObject; sx: TCanvasCoordType; sy: TCanvasCoordType; sWidth: TCanvasCoordType; sHeight: TCanvasCoordType; dx: TCanvasCoordType; dy: TCanvasCoordType; dwidth: TCanvasCoordType; dheight: TCanvasCoordType); overload;
    procedure fill; overload;
    procedure fill(aRule: string); overload;
    procedure fill(aPath: TJSPath2D); overload;
    procedure fill(aPath: TJSPath2D; aRule: string); overload;
    procedure fillRect(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType); overload;
    procedure fillText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType); overload;
    procedure fillText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType; aMaxWidth: TCanvasCoordType); overload;
    function getImageData(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType): TJSImageData; overload;
    function getLineDash: TJSArray;
    function isPointInPath(x: TCanvasCoordType; y: TCanvasCoordType): Boolean; overload;
    function isPointInPath(x: TCanvasCoordType; y: TCanvasCoordType; aFillRule: string): Boolean; overload;
    function isPointInPath(aPath: TJSPath2D; x: TCanvasCoordType; y: TCanvasCoordType): Boolean; overload;
    function isPointInPath(aPath: TJSPath2D; x: TCanvasCoordType; y: TCanvasCoordType; aFillRule: string): Boolean; overload;
    function isPointInStroke(x: TCanvasCoordType; y: TCanvasCoordType): Boolean; overload;
    function isPointInStroke(aPath: TJSPath2D; x: TCanvasCoordType; y: TCanvasCoordType): Boolean; overload;
    procedure lineTo(x: TCanvasCoordType; y: TCanvasCoordType);
    function measureText(S: string): TJSTextMetrics;
    procedure moveTo(x: TCanvasCoordType; y: TCanvasCoordType);
    procedure putImageData(aData: TJSImageData; x: TCanvasCoordType; y: TCanvasCoordType); overload;
    procedure putImageData(aData: TJSImageData; x: TCanvasCoordType; y: TCanvasCoordType; dityX: TCanvasCoordType; dirtyY: TCanvasCoordType; dirtyWidth: TCanvasCoordType; dirtyHeight: TCanvasCoordType); overload;
    procedure quadraticCurveTo(cpx: TCanvasCoordType; cpy: TCanvasCoordType; x: TCanvasCoordType; y: TCanvasCoordType);
    procedure rect(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType); overload;
    procedure restore;
    procedure rotate(anAngle: Double);
    procedure save;
    procedure scale(x: Double; y: Double);
    procedure setLineDash(segments: array of Integer); overload;
    procedure setLineDash(segments: TJSArray); overload;
    procedure setTransform(a: Double; b: Double; c: Double; d: Double; e: Double; f: Double);
    procedure stroke; overload;
    procedure stroke(aPath: TJSPath2D); overload;
    procedure strokeRect(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType);
    procedure strokeText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType); overload;
    procedure strokeText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType; aMaxWidth: TCanvasCoordType); overload;
    procedure transform(a: Double; b: Double; c: Double; d: Double; e: Double; f: Double);
    procedure translate(x: TCanvasCoordType; y: TCanvasCoordType);
    property canvas: TJSHTMLCanvasElement read FCanvas;
    property fillStyleAsColor: string read FfillStyleColor write FfillStyleColor;
    property fillStyleAsGradient: TJSCanvasGradient read FfillStyleGradient write FfillStyleGradient;
    property fillStyleAsPattern: TJSCanvasPattern read FfillStylePattern write FfillStylePattern;
    property strokeStyleAsColor: string read FstrokeStyleColor write FstrokeStyleColor;
    property strokeStyleAsGradient: TJSCanvasGradient read FstrokeStyleGradient write FstrokeStyleGradient;
    property strokeStyleAsPattern: TJSCanvasPattern read FstrokeStylePattern write FstrokeStylePattern;
  end;

  TJSXMLHttpRequestEventTarget = class(TJSEventTarget)
  end;

  TJSXMLHttpRequestUpload = class(TJSXMLHttpRequestEventTarget)
  end;

  TJSOnReadyStateChangeHandler = procedure;

  TJSXMLHttpRequest = class(TJSXMLHttpRequestEventTarget)
  strict private
    FReadyState: NativeInt;
    FResponse: JSValue;
    FResponseText: string;
    FResponseType: string;
    FresponseURL: string;
    FresponseXML: TJSDocument;
    FStatus: integer;
    FUpload: TJSXMLHttpRequestUpload;
  public const
    DONE = 4;
    HEADERS_RECEIVED = 3;
    LOADING = 3;
    OPENED = 1;
    UNSENT = 0;
  public
    onreadystatechange: TJSOnReadyStateChangeHandler;
    timeout: LongWord;
    withCredentials: Boolean;
    constructor new;
    procedure abort;
    function getAllResponseHeaders: string;
    function getResponseHeader(aName: string): string;
    procedure open(aMethod: string; aURL: string); overload;
    procedure open(aMethod: string; aURL: string; Async: Boolean); overload;
    procedure open(aMethod: string; aURL: string; Async: Boolean; AUserame: string); overload;
    procedure open(aMethod: string; aURL: string; Async: Boolean; AUserame: string; APassword: string); overload;
    procedure overrideMimeType(aType: string);
    procedure send; overload;
    procedure send(aBody: jsValue); overload;
    procedure setRequestHeader(aName: string; AValue: string);
    property readyState: NativeInt read FReadyState;
    property response: JSValue read FResponse;
    property ResponseHeaders[aName : string]: string read getResponseHeader;
    property responseText: string read FResponseText;
    property responseType: string read FResponseType;
    property responseURL: string read FresponseURL;
    property responseXML: TJSDocument read FresponseXML;
    property Status: integer read FStatus;
    property upload: TJSXMLHttpRequestUpload read FUpload;
  end;

  TJSUIEvent = class(TJSEvent)
  strict private
    FDetail: NativeInt;
    FView: TJSWindow;
  public
    property detail: NativeInt read FDetail;
    property view: TJSWindow read FView;
  end;

  TJSMouseEvent = class(TJSUIevent)
  strict private
    FAltKey: Boolean;
    FBUtton: NativeInt;
    FBUttons: NativeInt;
    FClientX: Double;
    FClientY: Double;
    FCtrlKey: Boolean;
    FMetaKey: Boolean;
    FmovementX: Double;
    FmovementY: Double;
    FoffsetX: Double;
    FoffsetY: Double;
    FRegion: string;
    FRelatedTarget: TJSEventTarget;
    FscreenX: Double;
    FscreenY: Double;
    FShiftKey: Boolean;
  public
    property altKey: Boolean read FAltKey;
    property button: NativeInt read FBUtton;
    property buttons: NativeInt read FBUttons;
    property clientX: Double read FClientX;
    property clientY: Double read FClientY;
    property ctrlKey: Boolean read FCtrlKey;
    property metaKey: Boolean read FMetaKey;
    property movementX: Double read FmovementX;
    property movementY: Double read FmovementY;
    property offsetX: Double read FoffsetX;
    property offsetY: Double read FoffsetY;
    property region: string read FRegion;
    property relatedTarget: TJSEventTarget read FRelatedTarget;
    property screenX: Double read FscreenX;
    property screenY: Double read FscreenY;
    property shiftKey: Boolean read FShiftKey;
    property x: Double read FClientX;
    property y: Double read FClientY;
  end;

  TJSWheelEventInit = record
  public
    deltaMode : NativeInt;
    deltaX : Double;
    deltaY : Double;
    deltaZ : Double;
  end;

  TJSWheelEvent = class(TJSMouseEvent)
  strict private
    FDeltaMode: NativeInt;
    FDeltaX: Double;
    FDeltaY: Double;
    FDeltaZ: Double;
  public
    constructor new(atype: string); overload;
    constructor new(atype: string; aInit: TJSWheelEventInit); overload;
    property deltaMode: NativeInt read FDeltaMode;
    property deltaX: Double read FDeltaX;
    property deltaY: Double read FDeltaY;
    property deltaZ: Double read FDeltaZ;
  end;

  TJSPointerEvent = class(TJSMouseEvent)
  end;

  TJSKeyNames = class(TObject)
  public const
    Accept = 'Accept';
    Add = 'Add';
    Again = 'Again';
    AllCandidates = 'AllCandidates';
    Alphanumeric = 'Alphanumeric';
    Alt = 'Alt';
    AltGraph = 'AltGraph';
    AppSwitch = 'AppSwitch';
    ArrowDown = 'ArrowDown';
    ArrowLeft = 'ArrowLeft';
    ArrowRight = 'ArrowRight';
    ArrowUp = 'ArrowUp';
    Attn = 'Attn';
    BackSpace = 'BackSpace';
    BrightnessDown = 'BrightnessDown';
    BrightnessUp = 'BrightnessUp';
    Call = 'Call';
    Camera = 'Camera';
    CameraFocus = 'CameraFocus';
    Cancel = 'Cancel';
    CapsLock = 'CapsLock';
    Clear = 'Clear';
    CodeInput = 'CodeInput';
    Compose = 'Compose';
    ContextMenu = 'Contextmenu';
    Control = 'Control';
    Convert = 'Convert';
    Copy = 'Copy';
    CrSel = 'CrSel';
    Cut = 'Cut';
    Dead = 'Dead';
    Decimal = 'Decimal';
    Delete = 'Delete';
    Divide = 'Divide';
    Eisu = 'Eisu';
    Eject = 'Eject';
    EndCall = 'EndCall';
    Enter = 'Enter';
    EraseEof = 'EraseEof';
    Escape = 'Escape';
    Execute = 'Execute';
    ExSel = 'ExSel';
    F1 = 'F1';
    F10 = 'F10';
    F11 = 'F11';
    F12 = 'F12';
    F13 = 'F13';
    F14 = 'F14';
    F15 = 'F15';
    F16 = 'F16';
    F17 = 'F17';
    F18 = 'F18';
    F19 = 'F19';
    F2 = 'F2';
    F20 = 'F20';
    F3 = 'F3';
    F4 = 'F4';
    F5 = 'F5';
    F6 = 'F6';
    F7 = 'F7';
    F8 = 'F8';
    F9 = 'F9';
    FinalMode = 'FinalMode';
    Find = 'Find';
    Finish = 'Finish';
    Fn = 'Fn';
    FnLock = 'FnLock';
    GoBack = 'GoBack';
    GoHome = 'GoHome';
    GroupFirst = 'GroupFirst';
    GroupLast = 'GroupLast';
    GroupNext = 'GroupNext';
    GroupPrevious = 'GroupPrevious';
    HangulMode = 'HangulMode';
    HanjaMode = 'HanjaMode';
    Hankaku = 'Hankaku';
    HeadsetHook = 'HeadsetHook';
    Help = 'Help';
    Hibernate = 'Hibernate';
    Hiranga = 'Hiranga';
    HirangaKatakana = 'HirangaKatakana';
    Home = 'Home';
    Hyper = 'Hyper';
    Insert = 'Insert';
    JunjaMode = 'JunjaMode';
    KanaMode = 'KanaMode';
    Katakana = 'Katakana';
    Key11 = 'Key11';
    Key12 = 'Key12';
    LastNumberRedial = 'LastNumberRedial';
    LogOff = 'LogOff';
    MannerMode = 'MannerMode';
    Meta = 'Meta';
    ModelChange = 'ModelChange';
    Multiply = 'Multiply';
    NextCandidate = 'NextCandidate';
    NonConvert = 'NonConvert';
    Notification = 'Notification';
    NumClear = 'Clear';
    NumLock = 'NumLock';
    PageDown = 'PageDown';
    PageUp = 'PageUp';
    Paste = 'Paste';
    Pause = 'Pause';
    Play = 'Play';
    Power = 'Power';
    PowerOff = 'PowerOff';
    PreviousCandidate = 'PreviousCandidate';
    PrintScreen = 'PrintScreen';
    Process = 'Process';
    Props = 'Props';
    Redo = 'Redo';
    Romaji = 'Romaji';
    ScrollLock = 'ScrollLock';
    Select = 'Select';
    Separator = 'Separator';
    Shift = 'Shift';
    SingleCandidate = 'SingleCandidate';
    Soft1 = 'Soft1';
    Soft2 = 'Soft2';
    Soft3 = 'Soft3';
    Soft4 = 'Soft4';
    Space = ' ';
    Standby = 'Standby';
    Subtract = 'Subtract';
    Super = 'Super';
    Symbol = 'Symbol';
    SymbolLock = 'SymbolLock';
    Tab = 'Tab';
    Undo = 'Undo';
    VoiceDial = 'VoiceDial';
    WakeUp = 'WakeUp';
    Zenkaku = 'Zenkaku';
    ZenkakuHanaku = 'ZenkakuHanaku';
    ZoomIn = 'ZoomIn';
    ZoomOut = 'ZoomOut';
    _End = 'End';
  end;

  TJSKeyboardEvent = class(TJSUIEvent)
  strict private
    FAltKey: Boolean;
    FCode: string;
    FCtrlKey: Boolean;
    FIsComposing: Boolean;
    FKey: string;
    FLocale: string;
    FLocation: NativeInt;
    FMetaKey: Boolean;
    FRepeat: Boolean;
    FShiftKey: Boolean;
  public
    function getModifierState(aKey: string): Boolean;
    property altKey: Boolean read FAltKey;
    property code: string read FCode;
    property ctrlKey: Boolean read FCtrlKey;
    property isComposing: Boolean read FIsComposing;
    property Key: string read FKey;
    property locale: string read FLocale;
    property location: NativeInt read FLocation;
    property metaKey: Boolean read FMetaKey;
    property shiftKey: Boolean read FShiftKey;
    property _repeat: Boolean read FRepeat;
  end;

  TJSMessageEvent = class(TObject)
  strict private
    FData: TJSObject;
    FOrigin: string;
  public
    property data: TJSObject read FData;
    property origin: string read FOrigin;
  end;

  TJSInputEvent = class(TObject)
  strict private
    FData: TJSDOMString;
    FDataTransfer: TJSObject;
    FInputType: TJSDOMString;
    FIsComposing: Boolean;
  public
    property data: TJSDOMString read FData;
    property dataTransfer: TJSObject read FDataTransfer;
    property inputType: TJSDOMString read FInputType;
    property isComposing: Boolean read FIsComposing;
  end;

  TJSHTMLMenuElement = class(TJSHTMLElement)
  end;

  TJSHTMLButtonElement = class(TJSHTMLElement)
  strict private
    FForm: TJSHTMLFormElement;
    FLabels: TJSNodeList;
    FValidationmMessage: string;
    FValidity: TJSValidityState;
    FWillValidate: Boolean;
  public
    autofocus: Boolean;
    disabled: Boolean;
    formAction: string;
    formEnctype: string;
    formMethod: string;
    formNoValidate: Boolean;
    formTarget: string;
    menu: TJSHTMLMenuElement;
    value: string;
    _type: string;
    property form: TJSHTMLFormElement read FForm;
    property labels: TJSNodeList read FLabels;
    property validationMessage: string read FValidationmMessage;
    property validity: TJSValidityState read FValidity;
    property willValidate: Boolean read FWillValidate;
  end;

  TJSHTMLTableCellElement = class(TJSHTMLElement)
  strict private
    Fabbr: string;
    Falign: string;
    Faxis: string;
    FbgColor: string;
    FcellIndex: Integer;
    Fch: string;
    FchOff: string;
    FcolSpan: Integer;
    Fheaders: string;
    Fheight: string;
    FnoWrap: Boolean;
    FrowSpan: Integer;
    Fscope: string;
    FvAlign: string;
    Fwidth: string;
  public
    property abbr: string read Fabbr write Fabbr;
    property align: string read Falign write Falign;
    property axis: string read Faxis write Faxis;
    property bgColor: string read FbgColor write FbgColor;
    property cellIndex: Integer read FcellIndex;
    property ch: string read Fch write Fch;
    property chOff: string read FchOff write FchOff;
    property colSpan: Integer read FcolSpan write FcolSpan;
    property headers: string read Fheaders write Fheaders;
    property height: string read Fheight write Fheight;
    property noWrap: Boolean read FnoWrap write FnoWrap;
    property rowSpan: Integer read FrowSpan write FrowSpan;
    property scope: string read Fscope write Fscope;
    property vAlign: string read FvAlign write FvAlign;
    property width: string read Fwidth write Fwidth;
  end;

  TJSHTMLTableRowElement = class(TJSHTMLElement)
  strict private
    Falign: string;
    FbgColor: string;
    Fcells: TJSHTMLCollection;
    Fch: string;
    FchOff: string;
    FrowIndex: Integer;
    FsectionRowIndex: Integer;
    FvAlign: string;
  public
    procedure deleteCell(index: Integer);
    function insertCell(index: Integer): TJSHTMLTableCellElement;
    property align: string read Falign write Falign;
    property bgColor: string read FbgColor write FbgColor;
    property cells: TJSHTMLCollection read Fcells;
    property ch: string read Fch write Fch;
    property chOff: string read FchOff write FchOff;
    property rowIndex: Integer read FrowIndex;
    property sectionRowIndex: Integer read FsectionRowIndex;
    property vAlign: string read FvAlign write FvAlign;
  end;

  TJSHTMLTableSectionElement = class(TJSHTMLElement)
  strict private
    Falign: string;
    Fch: string;
    FchOff: string;
    Frows: TJSHTMLCollection;
    FvAlign: string;
  public
    procedure deleteRow(index: Integer);
    function insertRow(index: Integer): TJSHTMLTableRowElement;
    property align: string read Falign write Falign;
    property ch: string read Fch write Fch;
    property chOff: string read FchOff write FchOff;
    property rows: TJSHTMLCollection read Frows;
    property vAlign: string read FvAlign write FvAlign;
  end;

  TJSHTMLTableElement = class(TJSHTMLElement)
  strict private
    FAlign: string;
    FBGColor: string;
    FBorder: string;
    FCaption: TJSHTMLElement;
    FCellPadding: string;
    FCellSpacing: string;
    FFrame: string;
    FRows: TJSHTMLCollection;
    FRules: string;
    FSummary: string;
    FTBodies: TJSHTMLCollection;
    FTfoot: TJSHTMLTableSectionElement;
    FTHead: TJSHTMLTableSectionElement;
    FWidth: string;
  public
    function createCaption: TJSHTMLElement;
    function createTFoot: TJSHTMLTableSectionElement;
    function createTHead: TJSHTMLTableSectionElement;
    procedure deleteCaption;
    procedure deleteRow(index: Integer);
    procedure deleteTFoot;
    procedure deleteTHead;
    function insertRow(index: Integer): TJSHTMLTableRowElement;
    property align: string read FAlign write FAlign;
    property bgColor: string read FBGColor write FBGColor;
    property border: string read FBorder write FBorder;
    property caption: TJSHTMLElement read FCaption;
    property cellPadding: string read FCellPadding write FCellPadding;
    property cellSpacing: string read FCellSpacing write FCellSpacing;
    property frame: string read FFrame write FFrame;
    property rows: TJSHTMLCollection read FRows;
    property rules: string read FRules write FRules;
    property summary: string read FSummary write FSummary;
    property tBodies: TJSHTMLCollection read FTBodies;
    property tfoot: TJSHTMLTableSectionElement read FTfoot;
    property tHead: TJSHTMLTableSectionElement read FTHead;
    property width: string read FWidth write FWidth;
  end;

var
  document: TJSDocument;
  window: TJSWindow;
  console: TJSConsole;

implementation

{ TJSEventTarget }

procedure TJSEventTarget.addEventListener(aname: string; aListener: JSValue);
begin
end;

procedure TJSEventTarget.addEventListener(aname: string; aListener: TJSEventHandler);
begin
end;

procedure TJSEventTarget.addEventListener(aname: string; aListener: TJSEventHandlerMethod);
begin
end;

function TJSEventTarget.dispatchEvent(event: JSValue): Boolean;
begin
  Result := False;
end;

procedure TJSEventTarget.removeEventListener(aname: string; aListener: JSValue);
begin
end;

procedure TJSEventTarget.removeEventListener(aname: string; aListener: TJSEventHandler);
begin
end;

{ TJSNode }

function TJSNode.appendChild(aChild: TJSNode): TJSNode;
begin
  Result := nil;
end;

function TJSNode.cloneNode(deep: Boolean): TJSNode;
begin
  Result := nil;
end;

function TJSNode.compareDocumentPosition(aNode: TJSNode): NativeInt;
begin
  Result := 0;
end;

function TJSNode.contains(aNode: TJSNode): Boolean;
begin
  Result := False;
end;

function TJSNode.hasChildNodes: Boolean;
begin
  Result := False;
end;

function TJSNode.insertBefore(newNode: TJSNode; referenceNode: TJSNode): TJSNode;
begin
  Result := nil;
end;

function TJSNode.isDefaultNameSpace(aNameSpaceURI: string): Boolean;
begin
  Result := False;
end;

function TJSNode.isEqualNode(aNode: TJSNode): Boolean;
begin
  Result := False;
end;

function TJSNode.isSameNode(aNode: TJSNode): Boolean;
begin
  Result := False;
end;

function TJSNode.lookupNamespaceURI(aPrefix: string): string;
begin
  Result := '';
end;

function TJSNode.lookupPrefix(aPrefix: string): string;
begin
  Result := '';
end;

procedure TJSNode.normalize;
begin
end;

function TJSNode.removeChild(aChild: TJSNode): TJSNode;
begin
  Result := nil;
end;

function TJSNode.replaceChild(aNewChild: TJSNode; aOldChild: TJSNode): TJSNode;
begin
  Result := nil;
end;

{ TJSNodeList }

procedure TJSNodeList.forEach(const aCallBack: TJSNodeListCallBack);
begin
end;

procedure TJSNodeList.forEach(const aCallBack: TJSNodeListEvent);
begin
end;

function TJSNodeList.item(aIndex: NativeInt): TJSNode;
begin
  Result := nil;
end;

{ TJSNamedNodeMap }

function TJSNamedNodeMap.getNamedItem(aName: string): TJSAttr;
begin
  Result := nil;
end;

{ TJSHTMLCollection }

function TJSHTMLCollection.item(aIndex: Integer): TJSNode;
begin
  Result := nil;
end;

function TJSHTMLCollection.namedItem(aName: string): TJSNode;
begin
  Result := nil;
end;

{ TJSDOMTokenList }

procedure TJSDOMTokenList.add(aToken: TJSDOMString);
begin
end;

function TJSDOMTokenList.contains(aToken: TJSDOMString): Boolean;
begin
  Result := False;
end;

procedure TJSDOMTokenList.forEach(const callback: TDOMTokenlistCallBack);
begin
end;

function TJSDOMTokenList.item(aIndex: NativeInt): string;
begin
  Result := '';
end;

procedure TJSDOMTokenList.remove(aToken: TJSDOMString);
begin
end;

procedure TJSDOMTokenList.replace(aOldToken: TJSDOMString; ANewToken: TJSDOMString);
begin
end;

function TJSDOMTokenList.supports(aToken: TJSDOMString): Boolean;
begin
  Result := False;
end;

function TJSDOMTokenList.toggle(aToken: TJSDOMString): Boolean;
begin
  Result := False;
end;

{ TJSElement }

procedure TJSElement.append(aText: string);
begin
end;

procedure TJSElement.append(aText: string; aNode: TJSElement);
begin
end;

procedure TJSElement.append(aNode: TJSElement);
begin
end;

procedure TJSElement.append(aNode1: TJSElement; aNode2: TJSElement);
begin
end;

function TJSElement.getAttribute(aName: string): string;
begin
  Result := '';
end;

function TJSElement.getAttributeNode(aName: string): TJSAttr;
begin
  Result := nil;
end;

function TJSElement.getAttributeNodeNS(aNameSpace: string; aName: string): TJSAttr;
begin
  Result := nil;
end;

function TJSElement.getAttributeNS(aNameSpace: string; aName: string): string;
begin
  Result := '';
end;

function TJSElement.getBoundingClientRect: TJSDOMRect;
begin
  Result := nil;
end;

function TJSElement.getClientRects: TJSClientRectArray;
begin
  Result := nil;
end;

function TJSElement.getElementsByClassName(aClassName: string): TJSHTMLCollection;
begin
  Result := nil;
end;

function TJSElement.getElementsByTagName(aTagName: string): TJSHTMLCollection;
begin
  Result := nil;
end;

function TJSElement.getElementsByTagNameNS(aNameSpace: string; aTagName: string): TJSHTMLCollection;
begin
  Result := nil;
end;

function TJSElement.hasAttribute(aName: string): Boolean;
begin
  Result := False;
end;

function TJSElement.hasAttributeNS(aNameSpace: string; aName: string): Boolean;
begin
  Result := False;
end;

function TJSElement.hasAttributes: Boolean;
begin
  Result := False;
end;

function TJSElement.insertAdjacentElement(aPosition: string; aElement: TJSElement): TJSElement;
begin
  Result := nil;
end;

procedure TJSElement.insertAdjacentHTML(aPosition: string; aHTML: string);
begin
end;

procedure TJSElement.insertAdjacentText(aPosition: string; aText: string);
begin
end;

function TJSElement.matches(aSelectorString: string): Boolean;
begin
  Result := False;
end;

function TJSElement.querySelector(aSelectors: string): TJSElement;
begin
  Result := nil;
end;

function TJSElement.querySelectorAll(aSelectors: string): TJSNodeList;
begin
  Result := nil;
end;

procedure TJSElement.releasePointerCapture(evID: JSValue);
begin
end;

procedure TJSElement.removeAttribute(aName: string);
begin
end;

function TJSElement.removeAttributeNode(aAttr: TJSAttr): TJSAttr;
begin
  Result := nil;
end;

procedure TJSElement.removeAttributeNS(aNameSpace: string; aName: string);
begin
end;

procedure TJSElement.requestFullScreen;
begin
end;

procedure TJSElement.setAttribute(aName: string; AValue: string);
begin
end;

function TJSElement.setAttributeNode(aNode: TJSAttr): TJSAttr;
begin
  Result := nil;
end;

function TJSElement.setAttributeNodeNS(aNode: TJSAttr): TJSAttr;
begin
  Result := nil;
end;

procedure TJSElement.setAttributeNS(aNameSpace: string; aName: string; AValue: string);
begin
end;

procedure TJSElement.setCapture(retargetToElement: Boolean);
begin
end;

procedure TJSElement.setPointerCapture(pointerID: JSValue);
begin
end;

{ TJSDOMImplementation }

function TJSDOMImplementation.createDocument(aNamespaceURI: string; aQualifiedNameStr: string; aDocumentType: TJSDocumentType): TJSDocument;
begin
  Result := nil;
end;

function TJSDOMImplementation.createDocumentType(aQualifiedNameStr: string; aPublicId: string; aSystemId: string): TJSDocumentType;
begin
  Result := nil;
end;

function TJSDOMImplementation.createHTMLDocument(aTitle: string): TJSDocument;
begin
  Result := nil;
end;

{ TJSLocation }

procedure TJSLocation.assign(aURL: string);
begin
end;

procedure TJSLocation.reload(aForce: Boolean);
begin
end;

procedure TJSLocation.replace(aURL: string);
begin
end;

function TJSLocation.toString: string;
begin
  Result := '';
end;

{ TJSCSSRuleList }

function TJSCSSRuleList.item(index: NativeInt): TJSCSSRule;
begin
  Result := nil;
end;

{ TJSCSSStyleSheet }

procedure TJSCSSStyleSheet.deleteRule(aIndex: NativeInt);
begin
end;

function TJSCSSStyleSheet.insertRule(aRule: string; aIndex: NativeInt): NativeInt;
begin
  Result := 0;
end;

function TJSStyleSheetList.item(index: NativeInt): TJSStyleSheet;
begin
  Result := nil;
end;

{ TJSDocumentFragment }

constructor TJSDocumentFragment.new;
begin
end;

function TJSDocumentFragment.querySelector(aSelector: string): TJSElement;
begin
  Result := nil;
end;

function TJSDocumentFragment.querySelectorAll(aSelector: string): TJSNodeList;
begin
  Result := nil;
end;

{ TJSEvent }

constructor TJSEvent.new(aType: string; const aInit: TJSEventInit);
begin
end;

procedure TJSEvent.preventDefault;
begin
end;

procedure TJSEvent.stopImmediatePropagation;
begin
end;

procedure TJSEvent.stopPropagation;
begin
end;

{ TJSXPathExpression }

function TJSXPathExpression.evaluate(contextNode: TJSNode; aType: NativeInt; aResult: TJSXPathResult): TJSXPathResult;
begin
  Result := nil;
end;

function TJSXPathExpression.evaluateWithContext(contextNode: TJSNode; aPosition: NativeInt; aSize: NativeInt; aType: NativeInt; aResult: TJSXPathResult): TJSXPathResult;
begin
  Result := nil;
end;

{ TJSXPathNSResolver }

function TJSXPathNSResolver.lookupNamespaceURI(prefix: string): string;
begin
  Result := '';
end;

{ TJSRange }

constructor TJSRange.new;
begin
end;

function TJSRange.cloneContents: TJSDocumentFragment;
begin
  Result := nil;
end;

function TJSRange.cloneRange: TJSRange;
begin
  Result := nil;
end;

procedure TJSRange.collapse;
begin
end;

function TJSRange.compareBoundaryPoints(aHow: NativeInt): NativeInt;
begin
  Result := 0;
end;

function TJSRange.createContextualFragment(aTagstring: string): TJSDocumentFragment;
begin
  Result := nil;
end;

procedure TJSRange.deleteContents;
begin
end;

procedure TJSRange.detach;
begin
end;

function TJSRange.extractContents: TJSDocumentFragment;
begin
  Result := nil;
end;

procedure TJSRange.insertNode(aNode: TJSNode);
begin
end;

procedure TJSRange.selectNode(aNode: TJSNode);
begin
end;

procedure TJSRange.selectNodeContents(aNode: TJSNode);
begin
end;

procedure TJSRange.setEnd(aEndNode: TJSNode; aEndOffset: NativeInt);
begin
end;

procedure TJSRange.setEndAfter(aEndNode: TJSNode);
begin
end;

procedure TJSRange.setEndBefore(aEndNode: TJSNode);
begin
end;

procedure TJSRange.setStart(aStartNode: TJSNode; aStartOffset: NativeInt);
begin
end;

procedure TJSRange.setStartAfter(aStartNode: TJSNode);
begin
end;

procedure TJSRange.setStartBefore(aStartNode: TJSNode);
begin
end;

procedure TJSRange.surroundContents(aNode: TJSNode);
begin
end;

function TJSRange.toString: string;
begin
  Result := '';
end;

{ TJSTreeWalker }

function TJSTreeWalker.firstChild: TJSNode;
begin
  Result := nil;
end;

function TJSTreeWalker.lastChild: TJSNode;
begin
  Result := nil;
end;

function TJSTreeWalker.nextNode: TJSNode;
begin
  Result := nil;
end;

function TJSTreeWalker.nextSibling: TJSNode;
begin
  Result := nil;
end;

function TJSTreeWalker.parentNode: TJSNode;
begin
  Result := nil;
end;

function TJSTreeWalker.previousNode: TJSNode;
begin
  Result := nil;
end;

function TJSTreeWalker.previousSibling: TJSNode;
begin
  Result := nil;
end;

{ TJSNodeFilter }

function TJSNodeFilter.acceptNode(aNode: TJSNode): NativeInt;
begin
  Result := 0;
end;

{ TJSXPathResult }

function TJSXPathResult.iterateNext: TJSNode;
begin
  Result := nil;
end;

function TJSXPathResult.snapshotItem(Index: NativeInt): TJSNode;
begin
  Result := nil;
end;

{ TJSSelection }

procedure TJSSelection.addRange(aRange: TJSRange);
begin
end;

procedure TJSSelection.collapse(aParentNode: TJSNode; Offset: NativeInt);
begin
end;

procedure TJSSelection.collapseToEnd;
begin
end;

procedure TJSSelection.collapseToStart;
begin
end;

function TJSSelection.containsNode(aNode: TJSNode; aPartlyContained: Boolean): Boolean;
begin
  Result := False;
end;

procedure TJSSelection.deleteFromDocument;
begin
end;

procedure TJSSelection.extend(aParentNode: TJSNode; Offset: NativeInt);
begin
end;

function TJSSelection.getRangeAt(aIndex: NativeInt): TJSRange;
begin
  Result := nil;
end;

procedure TJSSelection.removeAllRanges;
begin
end;

procedure TJSSelection.removeRange(aRange: TJSRange);
begin
end;

procedure TJSSelection.selectAllChildren(aParentNode: TJSNode);
begin
end;

procedure TJSSelection.setBaseAndExtent(aAnchorNode: TJSNode; aAnchorOffset: NativeInt; aFocusNode: TJSNode; aFocusOffset: NativeInt);
begin
end;

function TJSSelection.toString: string;
begin
  Result := '';
end;

{ TJSDocument }

function TJSDocument.adoptNode(aExternalNode: TJSNode): TJSNode;
begin
  Result := nil;
end;

procedure TJSDocument.close;
begin
end;

function TJSDocument.createAttribute(aName: string): TJSAttr;
begin
  Result := nil;
end;

function TJSDocument.createCDATASection(S: string): TJSNode;
begin
  Result := nil;
end;

function TJSDocument.createComment(S: string): TJSNode;
begin
  Result := nil;
end;

function TJSDocument.createDocumentFragment: TJSDocumentFragment;
begin
  Result := nil;
end;

function TJSDocument.createElement(tagName: string): TJSElement;
begin
  Result := nil;
end;

function TJSDocument.createElement(tagName: string; const options: TJSElementCreationOptions): TJSElement;
begin
  Result := nil;
end;

function TJSDocument.createElementNS(aNameSpace: string; tagName: string): TJSElement;
begin
  Result := nil;
end;

function TJSDocument.createElementNS(aNameSpace: string; tagName: string; const options: TJSElementCreationOptions): TJSElement;
begin
  Result := nil;
end;

function TJSDocument.createEvent(aType: string): TJSEvent;
begin
  Result := nil;
end;

function TJSDocument.createExpression(xPathText: string; aNameSpaceMapper: TJSNameSpaceMapperCallback): TJSXPathExpression;
begin
  Result := nil;
end;

function TJSDocument.createNSResolver(aNode: TJSNode): TJSXPathNSResolver;
begin
  Result := nil;
end;

function TJSDocument.createProcessingInstruction(target: string; data: string): TJSProcessingInstruction;
begin
  Result := nil;
end;

function TJSDocument.createRange: TJSRange;
begin
  Result := nil;
end;

function TJSDocument.createTextNode(S: string): TJSNode;
begin
  Result := nil;
end;

function TJSDocument.createTreeWalker(root: TJSNode; whatToShow: NativeInt; filter: TJSNodeFilter): TJSTreeWalker;
begin
  Result := nil;
end;

function TJSDocument.elementFromPoint(x: Integer; y: Integer): TJSElement;
begin
  Result := nil;
end;

procedure TJSDocument.enableStyleSheetsForSet(aSet: string);
begin
end;

function TJSDocument.evaluate(xpathExpression: string; ContextNode: TJSNode; NameSpaceResolver: TJSNamespaceMapperCallBack; resultType: NativeInt; aResult: TJSXPathResult): TJSXPathResult;
begin
  Result := nil;
end;

procedure TJSDocument.exitFullScreen;
begin
end;

function TJSDocument.getElementById(aID: string): TJSElement;
begin
  Result := nil;
end;

function TJSDocument.getElementsByClassName(aNames: string): TJSHTMLCollection;
begin
  Result := nil;
end;

function TJSDocument.getElementsByName(aName: string): TJSNodeList;
begin
  Result := nil;
end;

function TJSDocument.getElementsByTagName(aName: string): TJSHTMLCollection;
begin
  Result := nil;
end;

function TJSDocument.getElementsByTagNameNS(aNameSpace: string; aName: string): TJSHTMLCollection;
begin
  Result := nil;
end;

function TJSDocument.getSelection: TJSSelection;
begin
  Result := nil;
end;

function TJSDocument.hasFocus: Boolean;
begin
  Result := False;
end;

function TJSDocument.importNode(aExternalNode: TJSNode; Deep: Boolean): TJSNode;
begin
  Result := nil;
end;

procedure TJSDocument.open;
begin
end;

function TJSDocument.querySelector(aSelectors: string): TJSElement;
begin
  Result := nil;
end;

function TJSDocument.querySelectorAll(aSelectors: string): TJSNodeList;
begin
  Result := nil;
end;

procedure TJSDocument.releaseCapture;
begin
end;

procedure TJSDocument.write(aLine: string);
begin
end;

procedure TJSDocument.writeln(aLine: string);
begin
end;

{ TJSConsole }

procedure TJSConsole.assert(anAssertion: string; Obj1: JSValue);
begin
end;

procedure TJSConsole.clear;
begin
end;

procedure TJSConsole.count;
begin
end;

procedure TJSConsole.count(aCounter: string);
begin
end;

procedure TJSConsole.error(Obj1: JSValue);
begin
end;

procedure TJSConsole.group;
begin
end;

procedure TJSConsole.groupCollapsed;
begin
end;

procedure TJSConsole.groupEnd;
begin
end;

procedure TJSConsole.info(Obj1: JSValue);
begin
end;

procedure TJSConsole.log(Obj1: JSValue);
begin
end;

procedure TJSConsole.table(args: array of JSValue);
begin
end;

procedure TJSConsole.time(aName: string);
begin
end;

procedure TJSConsole.timeEnd(aName: string);
begin
end;

procedure TJSConsole.trace;
begin
end;

procedure TJSConsole.warn(Obj1: JSValue);
begin
end;

{ TJSPromise }

constructor TJSPromise.new(Executor: TJSPromiseExecutorFunc);
begin
end;

constructor TJSPromise.new(Executor: TJSPromiseExecutorMethod);
begin
end;

function TJSPromise.catch(onRejected: TPromiseResolverFunc): TJSPromise;
begin
  Result := nil;
end;

function TJSPromise.catch(onRejected: TPromiseResolverMethod): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.reject(reason: JSValue): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.resolve(value: JSValue): TJSPromise;
begin
  Result := nil;
end;

function TJSPromise._then(onRejected: TPromiseResolverFunc): TJSPromise;
begin
  Result := nil;
end;

function TJSPromise._then(onRejected: TPromiseResolverMethod): TJSPromise;
begin
  Result := nil;
end;

{ TJSSubtleCrypto }

function TJSSubtleCrypto.decrypt(algorithm: JSValue; aKey: TJSCryptoKey; aData: TJSBufferSource): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.deriveKey(algorithm: JSValue; aMasterKey: TJSCryptoKey; aDerivedAlgo: JSValue; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.digest(algorithm: string; Buffer: TJSArrayBuffer): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.digest(algorithm: string; Buffer: TJSTypedArray): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.encrypt(algorithm: JSValue; aKey: TJSCryptoKey; aData: TJSBufferSource): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.exportKey(algorithm: string; AKey: TJSCryptoKey): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.generateKey(algorithm: JSValue; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.importKey(format: string; KeyData: TJSArrayBuffer; algorithm: string; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.sign(algorithm: string; Key: TJSCryptoKey; aText: TJSArrayBuffer): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.unwrapKey(algorithm: string; wrappedKey: TJSArrayBuffer; Key: TJSCryptoKey; unwrapAlgo: string; unwrappedKeyAlgo: string; extractable: Boolean; Usages: TStringDynArray): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.verify(algorithm: string; key: TJSCryptoKey; Signature: TJSArrayBuffer; textToVerify: TJSArrayBuffer): TJSPromise;
begin
  Result := nil;
end;

function TJSSubtleCrypto.wrapKey(aFormat: string; Key: TJSCryptoKey; WrappingKey: TJSCryptoKey; WrapAlgorithm: string): TJSPromise;
begin
  Result := nil;
end;

{ TJSCrypto }

procedure TJSCrypto.getRandomValues(anArray: TJSTypedArray);
begin
end;

{ TJSHistory }

procedure TJSHistory.back;
begin
end;

procedure TJSHistory.forward;
begin
end;

procedure TJSHistory.go;
begin
end;

procedure TJSHistory.go(aIndex: NativeInt);
begin
end;

procedure TJSHistory.pushState(aState: jsValue; aTitle: string);
begin
end;

procedure TJSHistory.pushState(aState: jsValue; aTitle: string; AURL: string);
begin
end;

procedure TJSHistory.replaceState(aState: jsValue);
begin
end;

procedure TJSHistory.replaceState(aState: jsValue; aTitle: string);
begin
end;

procedure TJSHistory.replaceState(aState: jsValue; aTitle: string; AURL: string);
begin
end;

{ TJSIDBTransaction }

procedure TJSIDBTransaction.abort;
begin
end;

function TJSIDBTransaction.objectStore(aName: string): TJSIDBObjectStore;
begin
  Result := nil;
end;

{ TJSIDBKeyRange }

class function TJSIDBKeyRange.bound(aLower: JSValue; aUpper: JSValue): TJSIDBKeyRange;
begin
  Result := nil;
end;

class function TJSIDBKeyRange.bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean): TJSIDBKeyRange;
begin
  Result := nil;
end;

class function TJSIDBKeyRange.bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean; aUpperOpen: Boolean): TJSIDBKeyRange;
begin
  Result := nil;
end;

function TJSIDBKeyRange.includes(aValue: JSValue): Boolean;
begin
  Result := False;
end;

class function TJSIDBKeyRange.lowerBound(aLower: JSValue): TJSIDBKeyRange;
begin
  Result := nil;
end;

class function TJSIDBKeyRange.lowerBound(aLower: JSValue; aOpen: Boolean): TJSIDBKeyRange;
begin
  Result := nil;
end;

class function TJSIDBKeyRange.only(aValue: JSValue): TJSIDBKeyRange;
begin
  Result := nil;
end;

class function TJSIDBKeyRange.upperBound(aUpper: JSValue): TJSIDBKeyRange;
begin
  Result := nil;
end;

class function TJSIDBKeyRange.upperBound(aUpper: JSValue; aOpen: Boolean): TJSIDBKeyRange;
begin
  Result := nil;
end;

{ TJSIDBIndex }

function TJSIDBIndex.count: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.get(aKey: jsValue): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.get(aKey: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAll(aKey: jsValue): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAll(aKey: jsValue; ACount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAll(aKey: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAll(aKey: TJSIDBKeyRange; ACount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAllKeys(aKey: jsValue): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAllKeys(aKey: jsValue; ACount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAllKeys(aKey: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getAllKeys(aKey: TJSIDBKeyRange; ACount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.getKey(aKey: jsValue): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.openCursor: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.openCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.openCursor(aKeyRange: TJSIDBKeyRange; ADirection: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.openKeyCursor: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.openKeyCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBIndex.openKeyCursor(aKeyRange: TJSIDBKeyRange; ADirection: string): TJSIDBRequest;
begin
  Result := nil;
end;

{ TJSIDBCursor }

procedure TJSIDBCursor.advance(aKey: JSValue);
begin
end;

procedure TJSIDBCursor.advance(aCount: NativeInt);
begin
end;

procedure TJSIDBCursor.continue;
begin
end;

procedure TJSIDBCursor.continue(aKey: JSValue);
begin
end;

procedure TJSIDBCursor.continuePrimaryKey(aKey: JSValue);
begin
end;

procedure TJSIDBCursor.continuePrimaryKey(aKey: JSValue; aPrimaryKey: JSValue);
begin
end;

procedure TJSIDBCursor.delete;
begin
end;

procedure TJSIDBCursor.update(aValue: JSValue);
begin
end;

{ TJSIDBObjectStore }

function TJSIDBObjectStore.add(aValue: JSValue): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.add(aValue: JSValue; aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.clear: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.count: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.count(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.count(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.createIndex(aIndexName: string; KeyPath: string): TJSIDBIndex;
begin
  Result := nil;
end;

function TJSIDBObjectStore.createIndex(aIndexName: string; KeyPath: string; Options: TJSIDBIndexParameters): TJSIDBIndex;
begin
  Result := nil;
end;

function TJSIDBObjectStore.createIndex(aIndexName: string; KeyPath: array of string): TJSIDBIndex;
begin
  Result := nil;
end;

function TJSIDBObjectStore.createIndex(aIndexName: string; KeyPath: array of string; Options: TJSIDBIndexParameters): TJSIDBIndex;
begin
  Result := nil;
end;

function TJSIDBObjectStore.delete(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.delete(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

procedure TJSIDBObjectStore.deleteIndex(aIndexName: string);
begin
end;

function TJSIDBObjectStore.get(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.get(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAll: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAll(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAll(aKey: string; aCount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAll(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAll(aKeyRange: TJSIDBKeyRange; aCount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAllKeys(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAllKeys(aKey: string; aCount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAllKeys(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getAllKeys(aKeyRange: TJSIDBKeyRange; aCount: NativeInt): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getKey(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.getKey(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.index(aIndexName: string): TJSIDBIndex;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openCursor: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openCursor(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openCursor(aKey: string; aDirection: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openCursor(aKeyRange: TJSIDBKeyRange; aDirection: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openKeyCursor: TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openKeyCursor(aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openKeyCursor(aKey: string; aDirection: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openKeyCursor(aKeyRange: TJSIDBKeyRange): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.openKeyCursor(aKeyRange: TJSIDBKeyRange; aDirection: string): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.put(aValue: JSValue): TJSIDBRequest;
begin
  Result := nil;
end;

function TJSIDBObjectStore.put(aValue: JSValue; aKey: string): TJSIDBRequest;
begin
  Result := nil;
end;

{ TIDBDatabase }

procedure TIDBDatabase.close;
begin
end;

function TIDBDatabase.createObjectStore(aName: string): TJSIDBObjectStore;
begin
  Result := nil;
end;

function TIDBDatabase.createObjectStore(aName: string; Options: TJSCreateObjectStoreOptions): TJSIDBObjectStore;
begin
  Result := nil;
end;

procedure TIDBDatabase.deleteObjectStore(aName: string);
begin
end;

function TIDBDatabase.transaction(aStoreNames: array of string): TJSIDBTransaction;
begin
  Result := nil;
end;

function TIDBDatabase.transaction(aStoreNames: array of string; aMode: string): TJSIDBTransaction;
begin
  Result := nil;
end;

{ TJSIDBFactory }

function TJSIDBFactory.cmp(a: jsValue; b: jsValue): NativeInt;
begin
  Result := 0;
end;

function TJSIDBFactory.deleteDatabase(aName: string): TJSIDBOpenDBRequest;
begin
  Result := nil;
end;

function TJSIDBFactory.open(aName: string): TJSIDBOpenDBRequest;
begin
  Result := nil;
end;

function TJSIDBFactory.open(aName: string; aVersion: Integer): TJSIDBOpenDBRequest;
begin
  Result := nil;
end;

{ TJSStorage }

procedure TJSStorage.clear;
begin
end;

function TJSStorage.getItem(aKeyName: string): string;
begin
  Result := '';
end;

function TJSStorage.key(aIndex: Integer): string;
begin
  Result := '';
end;

procedure TJSStorage.removeItem(aKeyName: string);
begin
end;

procedure TJSStorage.setItem(aKeyName: string; aValue: string);
begin
end;

{ TJSGeoLocation }

procedure TJSGeoLocation.clearWatch(AID: NativeInt);
begin
end;

procedure TJSGeoLocation.getCurrentPosition(ASuccess: TJSGeoLocationCallback);
begin
end;

procedure TJSGeoLocation.getCurrentPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback);
begin
end;

procedure TJSGeoLocation.getCurrentPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback; AOptions: TJSPositionOptions);
begin
end;

function TJSGeoLocation.watchPosition(ASuccess: TJSGeoLocationCallback): NativeInt;
begin
  Result := 0;
end;

function TJSGeoLocation.watchPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback): NativeInt;
begin
  Result := 0;
end;

function TJSGeoLocation.watchPosition(ASuccess: TJSGeoLocationCallback; aError: TJSGeoLocationErrorCallback; AOptions: TJSPositionOptions): NativeInt;
begin
  Result := 0;
end;

{ TJSWorker }

constructor TJSWorker.new(aURL: string);
begin
end;

procedure TJSWorker.postMessage(aValue: JSValue);
begin
end;

procedure TJSWorker.postMessage(aValue: JSValue; aList: TJSValueDynArray);
begin
end;

procedure TJSWorker.terminate;
begin
end;

{ TJSMessagePort }

procedure TJSMessagePort.close;
begin
end;

procedure TJSMessagePort.postMessage(aValue: JSValue);
begin
end;

procedure TJSMessagePort.postMessage(aValue: JSValue; aList: TJSValueDynArray);
begin
end;

procedure TJSMessagePort.start;
begin
end;

{ TJSSharedWorker }

constructor TJSSharedWorker.new(aURL: string);
begin
end;

constructor TJSSharedWorker.new(aURL: string; aName: string);
begin
end;

{ TJSServiceWorkerRegistration }

function TJSServiceWorkerRegistration.unregister: TJSPromise;
begin
  Result := nil;
end;

procedure TJSServiceWorkerRegistration.update;
begin
end;

{ TJSServiceWorkerContainer }

function TJSServiceWorkerContainer.getRegistration: TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.getRegistration(aURL: string): TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.getRegistrations: TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.register(aURL: string): TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.register(aURL: string; aOptions: TJSServiceWorkerContainerOptions): TJSPromise;
begin
  Result := nil;
end;

{ TJSNavigator }

function TJSNavigator.getBattery: TJSPromise;
begin
  Result := nil;
end;

procedure TJSNavigator.registerContentHandler(aMimeType: string; aURI: string; aTitle: string);
begin
end;

procedure TJSNavigator.registerProtocolHandler(aProtocol: string; aURI: string; aTitle: string);
begin
end;

function TJSNavigator.requestMediaKeySystemAccess(aKeySystem: string; supportedConfigurations: TJSValueDynArray): TJSPromise;
begin
  Result := nil;
end;

procedure TJSNavigator.vibrate(aPattern: array of NativeInt);
begin
end;

procedure TJSNavigator.vibrate(aPattern: NativeInt);
begin
end;

{ TJSWindow }

procedure TJSWindow.alert(const Msg: string);
begin
end;

function TJSWindow.atob(const aValue: string): string;
begin
  Result := '';
end;

procedure TJSWindow.blur;
begin
end;

function TJSWindow.btoa(const aValue: string): string;
begin
  Result := '';
end;

procedure TJSWindow.clearInterval(aID: NativeInt);
begin
end;

procedure TJSWindow.clearTimeout(aID: NativeInt);
begin
end;

procedure TJSWindow.close;
begin
end;

function TJSWindow.confirm(const aMsg: string): Boolean;
begin
  Result := False;
end;

procedure TJSWindow.focus;
begin
end;

function TJSWindow.getComputedStyle(aElement: TJSElement): TJSCSSStyleDeclaration;
begin
  Result := nil;
end;

function TJSWindow.getComputedStyle(aElement: TJSElement; aPseudoElement: TJSElement): TJSCSSStyleDeclaration;
begin
  Result := nil;
end;

function TJSWindow.matchMedia(aQuery: string): TJSMediaQueryList;
begin
  Result := nil;
end;

procedure TJSWindow.moveBy(x: NativeInt; y: NativeInt);
begin
end;

procedure TJSWindow.moveTo(x: NativeInt; y: NativeInt);
begin
end;

function TJSWindow.open: TJSWindow;
begin
  Result := nil;
end;

function TJSWindow.open(const aURL: string): TJSWindow;
begin
  Result := nil;
end;

function TJSWindow.open(const aURL: string; const aTarget: string): TJSWindow;
begin
  Result := nil;
end;

function TJSWindow.open(const aURL: string; const aTarget: string; AOptions: TJSObject): TJSWindow;
begin
  Result := nil;
end;

procedure TJSWindow.print;
begin
end;

function TJSWindow.prompt(const aMessage: string): string;
begin
  Result := '';
end;

function TJSWindow.prompt(const aMessage: string; const aDefault: string): string;
begin
  Result := '';
end;

procedure TJSWindow.resizeBy(aWidth: NativeInt; aHeight: NativeInt);
begin
end;

procedure TJSWindow.resizeTo(aWidth: NativeInt; aHeight: NativeInt);
begin
end;

procedure TJSWindow.scrollBy(x: NativeInt; y: NativeInt);
begin
end;

procedure TJSWindow.scrollTo(x: NativeInt; y: NativeInt);
begin
end;

function TJSWindow.setInterval(ahandler: TJSTimerCallBack; aInterval: NativeUInt): NativeInt;
begin
  Result := 0;
end;

function TJSWindow.setTimeout(ahandler: TJSTimerCallBack; aTimeout: NativeUInt): NativeInt;
begin
  Result := 0;
end;

procedure TJSWindow.stop;
begin
end;

{ TJSCSSStyleDeclaration }

function TJSCSSStyleDeclaration.getPropertyPriority(const aProperty: string): string;
begin
  Result := '';
end;

function TJSCSSStyleDeclaration.getPropertyValue(const aProperty: string): string;
begin
  Result := '';
end;

function TJSCSSStyleDeclaration.item(aIndex: Integer): string;
begin
  Result := '';
end;

function TJSCSSStyleDeclaration.removeProperty(const aProperty: string): string;
begin
  Result := '';
end;

procedure TJSCSSStyleDeclaration.setProperty(const aProperty: string; const aValue: string);
begin
end;

procedure TJSCSSStyleDeclaration.setProperty(const aProperty: string; const aValue: string; const aPriority: string);
begin
end;

{ TJSHTMLElement }

procedure TJSHTMLElement.blur;
begin
end;

procedure TJSHTMLElement.click;
begin
end;

procedure TJSHTMLElement.focus;
begin
end;

{ TJSHTMLFormControlsCollection }

function TJSHTMLFormControlsCollection.namedItem(S: string): TJSElement;
begin
  Result := nil;
end;

{ TJSBlob }

procedure TJSBlob.close;
begin
end;

function TJSBlob.slice: TJSBlob;
begin
  Result := nil;
end;

function TJSBlob.slice(aStart: NativeInt): TJSBlob;
begin
  Result := nil;
end;

function TJSBlob.slice(aStart: NativeInt; aEnd: NativeInt): TJSBlob;
begin
  Result := nil;
end;

function TJSBlob.slice(aStart: NativeInt; aEnd: NativeInt; AContentType: string): TJSBlob;
begin
  Result := nil;
end;

{ TJSHTMLFileList }

function TJSHTMLFileList.item(aIndex: NativeInt): TJSHTMLFile;
begin
  Result := nil;
end;

{ TJSHTMLInputElement }

function TJSHTMLInputElement.checkValidity: Boolean;
begin
  Result := False;
end;

function TJSHTMLInputElement.reportValidity: Boolean;
begin
  Result := False;
end;

procedure TJSHTMLInputElement.select;
begin
end;

procedure TJSHTMLInputElement.setCustomValidity(aText: string);
begin
end;

procedure TJSHTMLInputElement.setRangeText(aText: string; selectionStart: NativeInt; selectionEnd: NativeInt);
begin
end;

procedure TJSHTMLInputElement.setRangeText(aText: string; selectionStart: NativeInt; selectionEnd: NativeInt; Direction: string);
begin
end;

procedure TJSHTMLInputElement.setSelectionRange(selectionStart: NativeInt; selectionEnd: NativeInt);
begin
end;

procedure TJSHTMLInputElement.setSelectionRange(selectionStart: NativeInt; selectionEnd: NativeInt; Direction: string);
begin
end;

procedure TJSHTMLInputElement.stepUp;
begin
end;

procedure TJSHTMLInputElement.stepUp(n: Integer);
begin
end;

{ TJSHTMLOptionElement }

constructor TJSHTMLOptionElement.New;
begin
end;

constructor TJSHTMLOptionElement.New(aText: string);
begin
end;

constructor TJSHTMLOptionElement.New(aText: string; aValue: string);
begin
end;

constructor TJSHTMLOptionElement.New(aText: string; aValue: string; aDefaultSelected: Boolean);
begin
end;

constructor TJSHTMLOptionElement.New(aText: string; aValue: string; aDefaultSelected: Boolean; Selected: Boolean);
begin
end;

{ TJSHTMLSelectElement }

procedure TJSHTMLSelectElement.add(anItem: TJSHTMLOptionElement);
begin
end;

procedure TJSHTMLSelectElement.add(anItem: TJSHTMLOptionElement; before: TJSHTMLOptionElement);
begin
end;

function TJSHTMLSelectElement.item(aIndex: NativeInt): TJSHTMLOptionElement;
begin
  Result := nil;
end;

function TJSHTMLSelectElement.namedItem(aName: string): TJSHTMLOptionElement;
begin
  Result := nil;
end;

procedure TJSHTMLSelectElement.remove(aIndex: NativeInt);
begin
end;

{ TJSHTMLCanvasElement }

function TJSHTMLCanvasElement.getContext(contextType: string): TJSObject;
begin
  Result := nil;
end;

function TJSHTMLCanvasElement.getContext(contextType: string; contextAttributes: TJSObject): TJSObject;
begin
  Result := nil;
end;

function TJSHTMLCanvasElement.getContextAs2DContext(contextType: string): TJSCanvasRenderingContext2D;
begin
  Result := nil;
end;

function TJSHTMLCanvasElement.getContextAs2DContext(contextType: string; contextAttributes: TJSObject): TJSCanvasRenderingContext2D;
begin
  Result := nil;
end;

procedure TJSHTMLCanvasElement.toBlob(aCallBack: THTMLCanvasToBlobCallback; aMimeType: string);
begin
end;

procedure TJSHTMLCanvasElement.toBlob(aCallBack: THTMLCanvasToBlobCallback; aMimeType: string; aQuality: Double);
begin
end;

function TJSHTMLCanvasElement.toDataURL: string;
begin
  Result := '';
end;

function TJSHTMLCanvasElement.toDataURL(aMimeType: string): string;
begin
  Result := '';
end;

function TJSHTMLCanvasElement.toDataURL(aMimeType: string; aQuality: Double): string;
begin
  Result := '';
end;

{ TJSImageData }

constructor TJSImageData.new(awidth: Integer; aheight: Integer);
begin
end;

constructor TJSImageData.new(anArray: TJSUint8ClampedArray; awidth: Integer; aheight: Integer);
begin
end;

{ TJSCanvasRenderingContext2D }

procedure TJSCanvasRenderingContext2D.arc(x: TCanvasCoordType; y: TCanvasCoordType; radius: TCanvasCoordType; startAngle: TCanvasCoordType; endAngle: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.arc(x: TCanvasCoordType; y: TCanvasCoordType; radius: TCanvasCoordType; startAngle: TCanvasCoordType; endAngle: TCanvasCoordType; antiClockWise: Boolean);
begin
end;

procedure TJSCanvasRenderingContext2D.arcTo(x1: TCanvasCoordType; y1: TCanvasCoordType; x2: TCanvasCoordType; y2: TCanvasCoordType; radius: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.beginPath;
begin
end;

procedure TJSCanvasRenderingContext2D.bezierCurveTo(cp1x: TCanvasCoordType; cp1y: TCanvasCoordType; cp2x: TCanvasCoordType; cp2y: TCanvasCoordType; x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.clearRect(x: TCanvasCoordType; y: TCanvasCoordType; width: TCanvasCoordType; height: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.clip;
begin
end;

procedure TJSCanvasRenderingContext2D.clip(aFillRule: string);
begin
end;

procedure TJSCanvasRenderingContext2D.clip(aPath: TJSPath2D);
begin
end;

procedure TJSCanvasRenderingContext2D.closePath;
begin
end;

function TJSCanvasRenderingContext2D.createImageData(aWidth: Integer; aHeight: Integer): TJSImageData;
begin
  Result := nil;
end;

function TJSCanvasRenderingContext2D.createImageData(aImage: TJSImageData): TJSImageData;
begin
  Result := nil;
end;

function TJSCanvasRenderingContext2D.createLinearGradient(x0: TCanvasCoordType; y0: TCanvasCoordType; x1: TCanvasCoordType; y1: TCanvasCoordType): TJSCanvasGradient;
begin
  Result := nil;
end;

function TJSCanvasRenderingContext2D.createPattern(aImage: TJSObject; repetition: string): TJSCanvasPattern;
begin
  Result := nil;
end;

function TJSCanvasRenderingContext2D.createRadialGradient(x0: TCanvasCoordType; y0: TCanvasCoordType; r0: TCanvasCoordType; x1: TCanvasCoordType; y1: TCanvasCoordType; r1: TCanvasCoordType): TJSCanvasGradient;
begin
  Result := nil;
end;

procedure TJSCanvasRenderingContext2D.drawFocusIfNeeded(aElement: TJSElement);
begin
end;

procedure TJSCanvasRenderingContext2D.drawFocusIfNeeded(aPath: TJSPath2D; aElement: TJSElement);
begin
end;

procedure TJSCanvasRenderingContext2D.drawImage(image: TJSObject; dx: TCanvasCoordType; dy: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.drawImage(image: TJSObject; dx: TCanvasCoordType; dy: TCanvasCoordType; dwidth: TCanvasCoordType; dheight: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.drawImage(image: TJSObject; sx: TCanvasCoordType; sy: TCanvasCoordType; sWidth: TCanvasCoordType; sHeight: TCanvasCoordType; dx: TCanvasCoordType; dy: TCanvasCoordType; dwidth: TCanvasCoordType; dheight: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.fill;
begin
end;

procedure TJSCanvasRenderingContext2D.fill(aRule: string);
begin
end;

procedure TJSCanvasRenderingContext2D.fill(aPath: TJSPath2D);
begin
end;

procedure TJSCanvasRenderingContext2D.fill(aPath: TJSPath2D; aRule: string);
begin
end;

procedure TJSCanvasRenderingContext2D.fillRect(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.fillText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.fillText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType; aMaxWidth: TCanvasCoordType);
begin
end;

function TJSCanvasRenderingContext2D.getImageData(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType): TJSImageData;
begin
  Result := nil;
end;

function TJSCanvasRenderingContext2D.getLineDash: TJSArray;
begin
  Result := nil;
end;

function TJSCanvasRenderingContext2D.isPointInPath(x: TCanvasCoordType; y: TCanvasCoordType): Boolean;
begin
  Result := False;
end;

function TJSCanvasRenderingContext2D.isPointInPath(x: TCanvasCoordType; y: TCanvasCoordType; aFillRule: string): Boolean;
begin
  Result := False;
end;

function TJSCanvasRenderingContext2D.isPointInPath(aPath: TJSPath2D; x: TCanvasCoordType; y: TCanvasCoordType): Boolean;
begin
  Result := False;
end;

function TJSCanvasRenderingContext2D.isPointInPath(aPath: TJSPath2D; x: TCanvasCoordType; y: TCanvasCoordType; aFillRule: string): Boolean;
begin
  Result := False;
end;

function TJSCanvasRenderingContext2D.isPointInStroke(x: TCanvasCoordType; y: TCanvasCoordType): Boolean;
begin
  Result := False;
end;

function TJSCanvasRenderingContext2D.isPointInStroke(aPath: TJSPath2D; x: TCanvasCoordType; y: TCanvasCoordType): Boolean;
begin
  Result := False;
end;

procedure TJSCanvasRenderingContext2D.lineTo(x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

function TJSCanvasRenderingContext2D.measureText(S: string): TJSTextMetrics;
begin
end;

procedure TJSCanvasRenderingContext2D.moveTo(x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.putImageData(aData: TJSImageData; x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.putImageData(aData: TJSImageData; x: TCanvasCoordType; y: TCanvasCoordType; dityX: TCanvasCoordType; dirtyY: TCanvasCoordType; dirtyWidth: TCanvasCoordType; dirtyHeight: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.quadraticCurveTo(cpx: TCanvasCoordType; cpy: TCanvasCoordType; x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.rect(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.restore;
begin
end;

procedure TJSCanvasRenderingContext2D.rotate(anAngle: Double);
begin
end;

procedure TJSCanvasRenderingContext2D.save;
begin
end;

procedure TJSCanvasRenderingContext2D.scale(x: Double; y: Double);
begin
end;

procedure TJSCanvasRenderingContext2D.setLineDash(segments: array of Integer);
begin
end;

procedure TJSCanvasRenderingContext2D.setLineDash(segments: TJSArray);
begin
end;

procedure TJSCanvasRenderingContext2D.setTransform(a: Double; b: Double; c: Double; d: Double; e: Double; f: Double);
begin
end;

procedure TJSCanvasRenderingContext2D.stroke;
begin
end;

procedure TJSCanvasRenderingContext2D.stroke(aPath: TJSPath2D);
begin
end;

procedure TJSCanvasRenderingContext2D.strokeRect(x: TCanvasCoordType; y: TCanvasCoordType; awidth: TCanvasCoordType; aheight: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.strokeText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.strokeText(aText: string; x: TCanvasCoordType; y: TCanvasCoordType; aMaxWidth: TCanvasCoordType);
begin
end;

procedure TJSCanvasRenderingContext2D.transform(a: Double; b: Double; c: Double; d: Double; e: Double; f: Double);
begin
end;

procedure TJSCanvasRenderingContext2D.translate(x: TCanvasCoordType; y: TCanvasCoordType);
begin
end;

{ TJSXMLHttpRequest }

constructor TJSXMLHttpRequest.new;
begin
end;

procedure TJSXMLHttpRequest.abort;
begin
end;

function TJSXMLHttpRequest.getAllResponseHeaders: string;
begin
  Result := '';
end;

function TJSXMLHttpRequest.getResponseHeader(aName: string): string;
begin
  Result := '';
end;

procedure TJSXMLHttpRequest.open(aMethod: string; aURL: string);
begin
end;

procedure TJSXMLHttpRequest.open(aMethod: string; aURL: string; Async: Boolean);
begin
end;

procedure TJSXMLHttpRequest.open(aMethod: string; aURL: string; Async: Boolean; AUserame: string);
begin
end;

procedure TJSXMLHttpRequest.open(aMethod: string; aURL: string; Async: Boolean; AUserame: string; APassword: string);
begin
end;

procedure TJSXMLHttpRequest.overrideMimeType(aType: string);
begin
end;

procedure TJSXMLHttpRequest.send;
begin
end;

procedure TJSXMLHttpRequest.send(aBody: jsValue);
begin
end;

procedure TJSXMLHttpRequest.setRequestHeader(aName: string; AValue: string);
begin
end;

{ TJSWheelEvent }

constructor TJSWheelEvent.new(atype: string);
begin
end;

constructor TJSWheelEvent.new(atype: string; aInit: TJSWheelEventInit);
begin
end;

{ TJSKeyboardEvent }

function TJSKeyboardEvent.getModifierState(aKey: string): Boolean;
begin
  Result := False;
end;

{ TJSHTMLTableRowElement }

procedure TJSHTMLTableRowElement.deleteCell(index: Integer);
begin
end;

function TJSHTMLTableRowElement.insertCell(index: Integer): TJSHTMLTableCellElement;
begin
  Result := nil;
end;

{ TJSHTMLTableSectionElement }

procedure TJSHTMLTableSectionElement.deleteRow(index: Integer);
begin
end;

function TJSHTMLTableSectionElement.insertRow(index: Integer): TJSHTMLTableRowElement;
begin
  Result := nil;
end;

{ TJSHTMLTableElement }

function TJSHTMLTableElement.createCaption: TJSHTMLElement;
begin
  Result := nil;
end;

function TJSHTMLTableElement.createTFoot: TJSHTMLTableSectionElement;
begin
  Result := nil;
end;

function TJSHTMLTableElement.createTHead: TJSHTMLTableSectionElement;
begin
  Result := nil;
end;

procedure TJSHTMLTableElement.deleteCaption;
begin
end;

procedure TJSHTMLTableElement.deleteRow(index: Integer);
begin
end;

procedure TJSHTMLTableElement.deleteTFoot;
begin
end;

procedure TJSHTMLTableElement.deleteTHead;
begin
end;

function TJSHTMLTableElement.insertRow(index: Integer): TJSHTMLTableRowElement;
begin
  Result := nil;
end;

end.
