{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2019                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Crypto;

interface

uses
  Classes, JS, Web, Types;

type
  TCryptoEncSignEvent = procedure(ADataAsArrayBuffer: TJSArrayBuffer) of object;
  TCryptoVerifyEvent = procedure(AValid: Boolean) of object;
  TCryptoDecryptStringEvent = procedure(AData: string) of object;
  TCryptoDecryptBinaryEvent = procedure(AData: TJSUint8Array) of object;

  TCryptoKeyUsage = (kuEncrypt, kuDecrypt, kuSign, kuVerify, kuDeriveKey, kuDeriveBits, kuWrapKey, kuUnwrapKey);
  TCryptoKeyUsages = set of TCryptoKeyUsage;

  TAESEncryptionType = (aetCBC, aetGCM);
  TAESEncryptionKeyLength = (akl128, akl192, akl256);
  TCryptoDecryptResultType = (drtString, drtBinary);
  TCryptoExportImportFormat = (efRaw, efPKCS8, efSPKI, efJSON);

  TCryptoRawExportWrapEvent = procedure(ABinary: TJSUint8Array) of object;
  TCryptoJSONExportWrapEvent = procedure(AJSON: string) of object;
  TCryptoStringExportWrapEvent = procedure(AValue: string; ABinary: TJSUint8Array) of object;
  TCryptoKeyUnwrappedEvent = procedure(AUnwrappedKey: TJSCryptoKey) of object;
  TCryptoKeyWrappedEvent = procedure(AWrappedKey: TJSArrayBuffer) of object;
  TCryptoErrorEvent = procedure(AError: string) of object;

  TCryptoBase = class(TPersistent)
  protected
    FExtractableKey: Boolean;
    FUsages: TCryptoKeyUsages;
    function DoErrorCatch(Err: JSValue): JSValue;
    function DoKeyExportedJSON(AResult: JSValue): JSValue; virtual;
    function DoKeyExportedPKCS(AResult: JSValue): JSValue; virtual;
    function DoKeyExportedRaw(AResult: JSValue): JSValue; virtual;
    function DoKeyExportedSPKI(AResult: JSValue): JSValue; virtual;
    function GetUsages(AUsages: TCryptoKeyUsages): TStringDynArray;
    function GetPKCSAndSPKIImportAB(APEMString: string): TJSArrayBuffer;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedPKCS8: TCryptoStringExportWrapEvent read FOnKeyExportedPKCS write FOnKeyExportedPKCS;
    property OnKeyExportedRaw: TCryptoRawExportWrapEvent read FOnKeyExportedRaw write FOnKeyExportedRaw;
    property OnKeyExportedSPKI: TCryptoStringExportWrapEvent read FOnKeyExportedSPKI write FOnKeyExportedSPKI;
  public
    property ExtractableKey: Boolean read FExtractableKey write SetExtractableKey default False;
    property Usages: TCryptoKeyUsages read FUsages write FUsages default [kuEncrypt, kuDecrypt];
    property OnKeyCreated: TNotifyEvent read FOnKeyCreated write FOnKeyCreated;
    property OnError: TCryptoErrorEvent read FOnError write FOnError;
  end;

  TEncryptionBase = class(TCryptoBase)
  protected
    function DoDecryptedBinary(AResult: JSValue): JSValue;
    function DoDecryptedString(AResult: JSValue): JSValue;
    function DoKeyWrapped(AResult: JSValue): JSValue; virtual;
    function DoKeyUnwrapped(AResult: JSValue): JSValue; virtual;
  public
    procedure Encrypt(APlainText: string); overload; virtual;
    procedure Encrypt(ABinary: TJSUint8Array); overload; virtual;
    procedure Decrypt(AEncryptedData: TJSArrayBuffer; AResultType: TCryptoDecryptResultType); virtual;
    property OnDecryptedBinary: TCryptoDecryptBinaryEvent read FOnDecryptedBinary write FOnDecryptedBinary;
    property OnDecryptedString: TCryptoDecryptStringEvent read FOnDecryptedString write FOnDecryptedString;
    property OnEncrypted: TCryptoEncSignEvent read FOnEncrypted write FOnEncrypted;
    property OnKeyUnwrapped: TCryptoKeyUnwrappedEvent read FOnKeyUnwrapped write FOnKeyUnwrapped;
    property OnKeyWrapped: TCryptoKeyWrappedEvent read FOnKeyWrappedEvent write FOnKeyWrappedEvent;
  end;

  TAESEncryption = class(TEncryptionBase)
  protected
    function DoEncrypted(AResult: JSValue): JSValue;
    procedure DoImport(AMode: string; AArrayBuffer: TJSArrayBuffer);
    function DoKeyGenerated(AKey: JSValue): JSValue;
    function DoKeyImported(AKey: JSValue): JSValue;
    function DoKeyWrapped(AResult: JSValue): JSValue; override;
    function GetIVCombinedBuffer(AResultBuffer: JSValue): TJSUint8Array;
  public
    constructor Create(AType: TAESEncryptionType); overload;
    constructor Create(AType: TAESEncryptionType; AKeyLength: TAESEncryptionKeyLength); overload;
    constructor Create(AType: TAESEncryptionType; AKeyLength: TAESEncryptionKeyLength; AExtractable: Boolean); overload;
    constructor Create(AType: TAESEncryptionType; AKeyLength: TAESEncryptionKeyLength; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure AfterConstruction; override;
    procedure Encrypt(APlainText: string); overload; override;
    procedure Encrypt(ABinary: TJSUint8Array); overload; override;
    procedure ExportKey(AFormat: TCryptoExportImportFormat);
    procedure Decrypt(AEncryptedData: TJSArrayBuffer; AResultType: TCryptoDecryptResultType); override;
    procedure GenerateKey;
    procedure ImportKey(AJSON: string); overload;
    procedure ImportKey(ABinary: TJSUint8Array); overload;
    procedure ImportKey(ARaw: TJSArrayBuffer); overload;
    procedure UnwrapKey(AImportFormat: TCryptoExportImportFormat; AKey: TJSArrayBuffer; AKeyAlgorithm: JSValue; AExtractable: Boolean; AKeyUsages: TCryptoKeyUsages); overload;
    procedure WrapKey(AKey: TJSCryptoKey; AExportFormat: TCryptoExportImportFormat);
    property AESType: TAESEncryptionType read FType write SetAESType default aetCBC;
    property CryptoKey: TJSCryptoKey read FCryptoKey write SetCryptoKey;
    property KeyLength: TAESEncryptionKeyLength read FKeyLength write SetKeyLength default akl128;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedRaw: TCryptoRawExportWrapEvent read FOnKeyExportedRaw write FOnKeyExportedRaw;
    property OnKeyImported: TNotifyEvent read FOnKeyImported write FOnKeyImported;
  end;

  TWebAESEncryption = class(TAESEncryption);

  TRSAModulusLength = (rml2048, rml3072, rml4096);
  TCryptoHash = (ehSHA1, ehSHA256, ehSHA384, ehSHA512);
  TCryptoAsymKeyType = (aktPrivate, aktPublic);

  TRSAEncryption = class(TEncryptionBase)
  protected
    function DoEncrypted(AResult: JSValue): JSValue;
    procedure DoImportPrivate(AMode: string; AArrayBuffer: TJSArrayBuffer);
    procedure DoImportPublic(AMode: string; AArrayBuffer: TJSArrayBuffer);
    function DoKeyGenerated(AKey: JSValue): JSValue;
    function DoPrivateKeyImported(AKey: JSValue): JSValue;
    function DoPublicKeyImported(AKey: JSValue): JSValue;
  public
    constructor Create; overload;
    constructor Create(AModulusLength: TRSAModulusLength); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure AfterConstruction; override;
    procedure Encrypt(APlainText: string); overload; override;
    procedure Encrypt(ABinary: TJSUint8Array); overload; override;
    procedure ExportKey(AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat);
    procedure Decrypt(AEncryptedData: TJSArrayBuffer; AResultType: TCryptoDecryptResultType); override;
    procedure GenerateKey;
    procedure ImportKey(AKey: string; AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat); overload;
    procedure ImportKey(ABinary: TJSUint8Array; AKeyType: TCryptoAsymKeyType); overload; //PKCS#8 if private, SPKI if public
    procedure WrapKey(AKey: TJSCryptoKey; AExportFormat: TCryptoExportImportFormat);
    procedure UnwrapKey(AImportFormat: TCryptoExportImportFormat; AKey: TJSArrayBuffer; AKeyAlgorithm: JSValue; AExtractable: Boolean; AKeyUsages: TCryptoKeyUsages);
    property Hash: TCryptoHash read FHash write SetHash;
    property ModulusLength: TRSAModulusLength read FModulusLength write SetModulusLength;
    property PrivateKey: TJSCryptoKey read FPrivateKey write SetPrivateKey;
    property PublicKey: TJSCryptoKey read FPublicKey write SetPublicKey;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedPKCS8: TCryptoStringExportWrapEvent read FOnKeyExportedPKCS write FOnKeyExportedPKCS;
    property OnKeyExportedSPKI: TCryptoStringExportWrapEvent read FOnKeyExportedSPKI write FOnKeyExportedSPKI;
    property OnPrivateKeyImported: TNotifyEvent read FOnPrivateKeyImported write FOnPrivateKeyImported;
    property OnPublicKeyImported: TNotifyEvent read FOnPublicKeyImported write FOnPublicKeyImported;
  end;

  TWebRSAEncryption = class(TRSAEncryption);

  TSignatureBase = class(TCryptoBase)
  protected
    function DoSignatureCreated(AResult: JSValue): JSValue;
    function DoSignatureVerify(AResult: JSValue): JSValue;
  public
    procedure Sign(AText: string); overload; virtual;
    procedure Sign(ABinary: TJSUint8Array); overload; virtual;
    procedure Verify(ASignature: TJSArrayBuffer; AData: TJSArrayBuffer); virtual;
    property OnSigned: TCryptoEncSignEvent read FOnSigned write FOnSigned;
    property OnVerify: TCryptoVerifyEvent read FOnVerify write FOnVerify;
  end;

  TRSASignature = class(TSignatureBase)
  protected
    procedure DoImportPrivate(AMode: string; AArrayBuffer: TJSArrayBuffer);
    procedure DoImportPublic(AMode: string; AArrayBuffer: TJSArrayBuffer);
    function DoKeyGenerated(AKey: JSValue): JSValue;
    function DoPrivateKeyImported(AKey: JSValue): JSValue;
    function DoPublicKeyImported(AKey: JSValue): JSValue;
  public
    constructor Create; overload;
    constructor Create(AModulusLength: TRSAModulusLength); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean); overload;
    constructor Create(AModulusLength: TRSAModulusLength; AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure AfterConstruction; override;
    procedure ExportKey(AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat);
    procedure GenerateKey;
    procedure ImportKey(AKey: string; AKeyType: TCryptoAsymKeyType; AFormat: TCryptoExportImportFormat); overload;
    procedure ImportKey(ABinary: TJSUint8Array; AKeyType: TCryptoAsymKeyType); overload; //PKCS#8 if private, SPKI if public
    procedure Sign(AText: string); overload; override;
    procedure Sign(ABinary: TJSUint8Array); overload; override;
    procedure Verify(ASignature: TJSArrayBuffer; AData: TJSArrayBuffer); override;
    property Hash: TCryptoHash read FHash;
    property ModulusLength: TRSAModulusLength read FModulusLength;
    property PrivateKey: TJSCryptoKey read FPrivateKey write SetPrivateKey;
    property PublicKey: TJSCryptoKey read FPublicKey write SetPublicKey;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedPKCS8: TCryptoStringExportWrapEvent read FOnKeyExportedPKCS write FOnKeyExportedPKCS;
    property OnKeyExportedSPKI: TCryptoStringExportWrapEvent read FOnKeyExportedSPKI write FOnKeyExportedSPKI;
    property OnPrivateKeyImported: TNotifyEvent read FOnPrivateKeyImported write FOnPrivateKeyImported;
    property OnPublicKeyImported: TNotifyEvent read FOnPublicKeyImported write FOnPublicKeyImported;
  end;

  TWebRSASignature = class(TRSASignature);

  THMACSignature = class(TSignatureBase)
  protected
    procedure DoImport(AMode: string; AArrayBuffer: TJSArrayBuffer);
    function DoKeyGenerated(AKey: JSValue): JSValue;
    function DoKeyImported(AKey: JSValue): JSValue;
  public
    constructor Create; overload;
    constructor Create(AHash: TCryptoHash); overload;
    constructor Create(AHash: TCryptoHash; AExtractable: Boolean); overload;
    constructor Create(AHash: TCryptoHash; AExtractable: Boolean; AUsages: TCryptoKeyUsages); overload;
    procedure AfterConstruction; override;
    procedure GenerateKey;
    procedure ExportKey(AFormat: TCryptoExportImportFormat);
    procedure ImportKey(AJSON: string); overload;
    procedure ImportKey(ABinary: TJSUint8Array); overload;
    procedure ImportKey(ARaw: TJSArrayBuffer); overload;
    procedure Sign(AText: string); overload; override;
    procedure Sign(ABinary: TJSUint8Array); overload; override;
    procedure Verify(ASignature: TJSArrayBuffer; AData: TJSArrayBuffer); override;
    property CryptoKey: TJSCryptoKey read FCryptoKey write SetCryptoKey;
    property Hash: TCryptoHash read FHash;
    property OnKeyExportedJSON: TCryptoJSONExportWrapEvent read FOnKeyExportedJSON write FOnKeyExportedJSON;
    property OnKeyExportedRaw: TCryptoRawExportWrapEvent read FOnKeyExportedRaw write FOnKeyExportedRaw;
    property OnKeyImported: TNotifyEvent read FOnKeyImported write FOnKeyImported;
  end;

  TWebHMACSignature = class(THMACSignature);

var
  TCryptoKeyUsageValue: array[TCryptoKeyUsage] of string = ('encrypt', 'decrypt', 'sign', 'verify', 'deriveKey',
    'deriveBits', 'wrapKey', 'unwrapKey');

const
  IVLEN = 16;

function ABToBase(AArrayBuffer: TJSArrayBuffer): string;
function ABToBinaryString(AArrayBuffer: TJSArrayBuffer): string;
function BaseToAB(ABase: string): TJSArrayBuffer;
function BinaryStringToAB(ABinaryString: string): TJSArrayBuffer;
function GetMessageEncoding(APlainText: string): TJSUint8Array;
function GetMessageStringDecoding(AArray: TJSArrayBuffer): string;
function IsTextEncoderSupported: Boolean;

implementation
end.
