{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Electron;

interface

uses
  Classes, SysUtils, StrUtils, libelectron, Web, JS, WEBLib.Menus, WEBLib.Graphics, WEBLib.Forms;

type
  TReceivedValue = record
    AsString: string;
    AsBoolean: Boolean;
    AsInteger: NativeInt;
    AsObject: TJSObject;
    AsArray: TJSArray;
    AsFloat: Double;
    ValueType: TJSValueType;
  end;

  TEBWEventProc = reference to procedure;
  TEBWResizeProc = reference to procedure(Width, Height: NativeInt);
  TEBWMessageProc = reference to procedure(AMessage: JSValue);
  TEShortCutProc = reference to function(ShortCut: TShortCut): string;
  TEMenuClickProc = reference to function(Event: TJSMouseEvent; ItemId: string): Boolean;
  TEDragProc = reference to procedure(AEvent: TIPCEvent; Args: JSValue);

  TElectronStringList = class(TStringList)
  public
    procedure LoadFromFile(const FileName: string); virtual; reintroduce;
    procedure SaveToFile(const FileName: string); override;
  end;

  TElectronBinaryDataStream = class(TPersistent)
  public
    procedure LoadFromFile(const FileName: string); virtual;
    procedure SaveToFile(const FileName: string); virtual;
    property AsArrayBuffer: TJSArrayBuffer read GetArrayBuffer write SetArrayBuffer;
    property Base64: string read GetBase64 write SetBase64;
    property Text: string read FText write FText;
  end;

  TElectronDialogOption = (openFile, openDirectory, multiSelections, showHiddenFiles);
  TElectronDialogOptions = set of TElectronDialogOption;
  TElectronMessageBoxType = (embNone, embInfo, embError, embQuestion, embWarning);

  TElectronOpenDialog = class(TComponent)
  protected
    procedure AddProperty(Opt: TElectronDialogOption);
    property DialogOptions: TOpenDialogOptions read FDialogOptions write FDialogOptions;
  public
    constructor Create(AOwner: TComponent); override;
    function Execute(AProc: TSelectOpenFileCallBack): Boolean;
  published
    property ButtonLabel: string read FButtonLabel write FButtonLabel;
    property DefaultPath: string read FDefaultPath write FDefaultPath;
    property FileName: string read FFileName write FFileName;
    property Filters: string read FFilters write FFilters;
    property Options: TElectronDialogOptions read FOptions write FOptions default [openFile];
    property Title: string read FTitle write FTitle;
  end;

  TElectronSaveDialog = class(TComponent)
  protected
    property Options: TSaveDialogOptions read FOptions write FOptions;
  public
    constructor Create(AOwner: TComponent); override;
    function Execute(AProc: TSelectSaveFileCallBack): Boolean;
  published
    property ButtonLabel: string read FButtonLabel write FButtonLabel;
    property DefaultPath: string read FDefaultPath write FDefaultPath;
    property FileName: string read FFileName write FFileName;
    property Filters: string read FFilters write FFilters;
    property Title: string read FTitle write FTitle;
  end;

  TElectronMessageBox = class(TComponent)
  protected
    property Options: TMessageBoxOptions read FOptions write FOptions;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute(AProc: TSelectMessageBoxCallBack = nil);
  published
    property Buttons: TStringList read FButtons write SetButtons;
    property CancelId: NativeInt read FCancelId write FCancelId;
    property CheckboxChecked: Boolean read FCheckboxChecked write FCheckboxChecked;
    property CheckboxLabel: string read FCheckboxLabel write FCheckboxLabel;
    property DefaultId: NativeInt read FDefaultId write FDefaultId;
    property Detail: string read FDetail write FDetail;
    property DialogType: TElectronMessageBoxType read FType write FType;
    property IconPath: string read FIconPath write FIconPath;
    property IconURL: string read FIconURL write FIconURL;
    property Message: string read FMessage write FMessage;
    property NoLink: Boolean read FNoLink write FNoLink;
    property NormalizeAccessKeys: Boolean read FNormalizeAccessKeys write FNormalizeAccessKeys;
    property Response: NativeInt read FResponse write FResponse;
    property Title: string read FTitle write FTitle;
  end;

  TElectronErrorBox = class(TComponent)
  public
    procedure Execute;
  published
    property Content: string read FContent write FContent;
    property Title: string read FTitle write FTitle;
  end;

  TElectronMenuItem = class(TMenuItem)
  protected
    property Id: string read FId write FId;
  public
    property Role: string read FRole write FRole;
  end;

  TElectronMainMenu = class(TCustomMainMenu)
  protected
    procedure HandleMenuItemClick(event: TIPCEvent; args: JSValue);
    function MenuClick(ItemId: string): Boolean; virtual;
  public
    destructor Destroy; override;
    function CreateElement: TJSElement; override;
    procedure CreateInitialize; override;
    procedure EndUpdate; override;
  end;

  TElectronPopupMenu = class(TCustomMainMenu)
  protected
    procedure DoClosePopup;
    procedure DoPopup;
    procedure GeneratePopupMenu;
    procedure HandleMenuItemClick(event: TIPCEvent; args: JSValue);
    function MenuClick(ItemId: string): Boolean; virtual;
    property UniqueId: string read FUniqueId;
  public
    destructor Destroy; override;
    procedure CloseMenu;
    function CreateElement: TJSElement; override;
    procedure CreateInitialize; override;
    procedure EndUpdate; override;
    procedure Popup(X, Y: Integer);
    property OnPopup: TNotifyEvent read FOnPopup write FOnPopup;
    property OnClosePopup: TNotifyEvent read FOnClosePopup write FOnClosePopup;
  end;

  TElectronClipboard = class(TPersistent)
  public
    procedure Clear;
    function HasFormat(AFormat: string): Boolean;
    property AsHTML: string read GetAsHTML write SetAsHMTL;
    property AsImageURL: string read GetAsImageURL write SetAsImageURL;
    property AsRTF: string read GetAsRTF write SetAsRTF;
    property AsText: string read GetAsText write SetAsText;
    property FormatCount: Integer read GetFormatCount;
    property Formats[Index: Integer]: string read GetFormats;
  end;

  TElectronShell = class(TPersistent)
  public
    procedure Beep;
    function OpenExternal(URL: string): Boolean;
    function OpenItem(FullPath: string): Boolean;
    function MoveItemToTrash(FullPath: string): Boolean;
    function ShowItemInFolder(FullPath: string): Boolean;
  end;

  TElectronBrowserWindowResizeEvent = procedure(Sender: TObject; Width: NativeInt; Height: NativeInt) of object;
  TElectronBrowserWindowMessageEvent = procedure(AMessage: TReceivedValue) of object;

  TElectronBrowserWindow = class(TComponent)
  protected
    procedure GenerateBrowserWindow(Modal: Boolean);
    procedure DoClose; virtual;
    procedure DoShow; virtual;
    procedure DoHide; virtual;
    procedure DoMaximize; virtual;
    procedure DoMessage(AMessage: JSValue); virtual;
    procedure DoMinimize; virtual;
    procedure DoActivate; virtual;
    procedure DoDeactivate; virtual;
    procedure DoResize(Width, Height: NativeInt); virtual;
    procedure DoFullScreen; virtual;
    procedure DoExitFullScreen; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Close;
    procedure ForceClose;
    procedure Hide;
    procedure LoadFromURL(URL: string);
    procedure SendMessage(AMessage: string); overload;
    procedure SendMessage(AMessage: JSValue); overload;
    procedure SendMessage(Channel: string; AMessage: string); overload;
    procedure SendMessage(Channel: string; AMessage: JSValue); overload;
    procedure Show;
    procedure ShowModal;
    property FormClass: TFormClass read FFormClass write FFormClass;
  published
    property FullScreen: Boolean read FFullScreen write SetFullScreen default False;
    property FullScreenable: Boolean read FFullScreenable write SetFullScreenable default True;
    property IconPath: string read FIconPath write FIconPath;
    property IconURL: string read FIconURL write FIconURL;
    property Kiosk: Boolean read FKiosk write SetKiosk default False;
    property Resizable: Boolean read FResizable write SetResizable default True;
    property MinHeight: Integer read FMinHeight write SetMinHeight default 0;
    property MinWidth: Integer read FMinWidth write SetMinWidth default 0;
    property MaxHeight: Integer read FMaxHeight write SetMaxHeight;
    property MaxWidth: Integer read FMaxWidth write SetMaxWidth;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
    property OnHide: TNotifyEvent read FOnHide write FOnHide;
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
    property OnMaximize: TNotifyEvent read FOnMaximize write FOnMaximize;
    property OnMessage: TElectronBrowserWindowMessageEvent read FOnMessage write FOnMessage;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnResize: TElectronBrowserWindowResizeEvent read FOnResize write FOnResize;
    property OnFullScreen: TNotifyEvent read FOnFullScreen write FOnFullScreen;
    property OnExitFullScreen: TNotifyEvent read FOnExitFullScreen write FOnExitFullScreen;
  end;

  TElectronWindow = class(TPersistent)
  public
    procedure CloseApp;
    procedure CloseWindow;
    procedure CloseDevTools;
    procedure Copy;
    procedure CopyImageAt(X, Y: NativeInt);
    procedure Cut;
    procedure Delete;
    procedure DownloadURL(URL: string);
    procedure OpenDevTools;
    procedure Paste;
    procedure Redo;
    procedure Reload;
    procedure Replace(AText: string);
    procedure ReplaceMisspelling(AText: string);
    procedure SelectAll;
    procedure ToggleDevTools;
    procedure Undo;
    procedure Unselect;
  end;

  TElectronIPCRenderer = class(TPersistent)
  public
    procedure Drag(PathToItem: string);
    procedure Listen(Channel: string; Listener: TIPCEventHandler);
    procedure ListenOnce(Channel: string; Listener: TIPCEventHandler);
    procedure RemoveAllListeners(Channel: string);
    procedure Send(Channel: string; JSObject: JSValue); overload;
    procedure Send(Channel: string; AMessage: string); overload;
  end;

  TElectronDragAndDrop = class(TPersistent)
  protected
    procedure DoDragCallback; virtual;
    property UniqueId: string read FUniqueId write FUniqueId;
  public
    destructor Destroy; override;
    procedure StartDrag(PathToItem: string);
    procedure ListenToDrag(IconPath: string);
    procedure RemoveListeners;
    property OnStartDrag: TNotifyEvent read FOnDrag write FOnDrag;
  end;

  TElectronPath = class(TPersistent)
  protected
    procedure InitPaths;
  public
    function GetAppData: string;
    function GetAppPath: string;
    function GetDesktop: string;
    function GetDocuments: string;
    function GetDownloads: string;
    function GetExe: string;
    function GetHome: string;
    function GetMusic: string;
    function GetUserData: string;
    function GetPictures: string;
    function GetTemp: string;
    function GetVideos: string;
  end;

  TElectronIPCCommunicationReceiveEvent = procedure(AMessage: TReceivedValue) of object;

  TElectronIPCCommunication = class(TComponent)
  protected
    procedure DoListener(AEvent: TIPCEvent; Argf: JSValue; Args: JSValue);
    procedure DoReceive(Args: TReceivedValue); virtual;
    procedure RemoveListeners;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure EndUpdate; override;
    procedure Send(Channel: string; AMessage: JSValue); overload;
    procedure Send(Channel: string; AMessage: string); overload;
    procedure SendToParent(AMessage: JSValue);
  published
    property Channel: string read FChannel write SetChannel;
    property OnReceive: TElectronIPCCommunicationReceiveEvent read FOnReceive write FOnReceive;
  end;

  TElectronTrayIcon = class(TComponent)
  protected
    procedure DoClick; virtual;
    procedure DoCreateTrayIcon;
    procedure HandleTrayClick(event: TIPCEvent; args: JSValue);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure EndUpdate; override;
  published
    property IconPath: string read FIconPath write FIconPath;
    property IconURL: string read FIconURL write FIconURL;
    property Menu: TElectronPopupMenu read FMenu write FMenu;
    property ToolTip: string read FToolTip write FToolTip;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  TElectronFileWatch = class(TCollectionItem)
  protected
    procedure DoChange;
    procedure FileListener(Current: TNodeStat; Previous: TNodeStat);
  public
    constructor Create(ACollection: TCollection); override;
    destructor Destroy; override;
  published
    property FileName: string read FFileName write SetFileName;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TElectronFileWatches = class(TOwnedCollection)
  public
    function Add: TElectronFileWatch; reintroduce;
    property Items[Index: Integer]: TElectronFileWatch read GetItem write SetItem;
  end;

  TElectronFileWatcher = class(TComponent)
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Files: TElectronFileWatches read FFiles write FFiles;
  end;

  TElectronGlobalShortcut = class(TComponent)
  protected
    procedure DoShortcut(event: TIPCEvent; argf, args: JSValue);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property ShortCut: TShortCut read FShortCut write SetShortCut;
    property OnShortcut: TNotifyEvent read FOnShortcut write FOnShortcut;
  end;

  TElectronNotificationDirection = (endAuto, endLTR, endRTL);

  TElectronNotification = class(TComponent)
  protected
    procedure HandleDoClick(Event: TJSEvent);
  public
    procedure Show;
  published
    property Body: string read FBody write FBody;
    property Direction: TElectronNotificationDirection read FDirection write FDirection;
    property IconURL: string read FIcon write FIcon;
    property ImageURL: string read FImage write FImage;
    property Title: string read FTitle write FTitle;
    property OnNotificationClick: TNotifyEvent read FOnNotificationClick write FOnNotificationClick;
  end;

  TElectronFormBeforeUnloadEvent = procedure(Sender: TObject; var PreventClose: Boolean) of object;

  TElectronForm = class(TWebForm)
  protected
    procedure BindEvents; override;
    procedure Loaded; override;
    procedure UnbindEvents; override;
    function HandleBeforeElectronUnload(Event: TJSEvent): Boolean; virtual;
  public
    class constructor Create;
  published
    property OnBeforeUnload: TElectronFormBeforeUnloadEvent read FOnBeforeUnload write FOnBeforeUnload;
  end;

function CreateBlobFromFile(FileName: string): TJSBlob;
function CreateUint8ArrayFromFile(FileName: string): TJSUint8Array;
procedure DownloadToFolder(URL, Path: string);
function ElectronDragAndDrop: TElectronDragAndDrop;
function ElectronClipboard: TElectronClipboard;
function ElectronShell: TElectronShell;
function ElectronIPCRenderer: TElectronIPCRenderer;
function ElectronPath: TElectronPath;
function ElectronWindow: TElectronWindow;
function GetElectronFileList(Source: TJSEvent) : TJSElectronFileList;
procedure InitializeOS;
function ShortCutToText(ShortCut: TShortCut): string;
function TextToShortcut(Text: string): TShortcut;

var
  OSType: string;

implementation
initialization
end.
