{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ThreeJsMathChart;


interface

uses classes, Sysutils, WEBLib.Controls, WEBLib.webtools, math, strutils,
   WEBLib.graphics, web, js, libthreejs, WEBLib.ThreeJsChart,
   WEBLib.ThreeJsControls;

type
  TThreeJsMathChartType = (ctScatterChart, ctSurfaceChart);
  TThreeJsMathPointShape = (psSphere, psCube);

  TThreeJsSurfaceCallBack = function(x, y: double): double of object;

  TThreeJsMathChart = class;

  TThreeJsMathChartSeries = class
  protected
    function GetCount: integer;
    function GetContainsData: boolean;
    function ConvertToXUnits(aValue: double): double;
    function ConvertToYUnits(aValue: double): double;
    function ConvertToZUnits(aValue: double): double;
    function ConvertToXValue(aValue: double): double;
    function ConvertToYValue(aValue: double): double;
    function ConvertToZValue(aValue: double): double;

    function GetValueStrForXAxis(aValue: double): string;
    function GetValueStrForYAxis(aValue: double): string;
    function GetValueStrForZAxis(aValue: double): string;
    procedure ObtainSurfaceData;
    procedure SetASurfaceGradient(geometry: TThreeJsGeometry;
            colors: TJSArray;
            axis: string;
            reverse: boolean);
    procedure NormalizeSurfaceData;
    procedure PerformAutoScalingCalculations(aChart: TThreeJsMathChart);
    procedure GetData(anIndex: integer; out x: double; out y: double; out z: double;
                     out aPointShape: TThreeJsMathPointShape; out aPointSize: double; out aPointColor: TColor; out aUserData: JSValue);
    procedure MakeUniformAxesScales;

    property ContainsData: boolean read getContainsData;
    property Count: integer read getCount;
  public
    constructor Create; virtual;

    procedure AddData(x, y, z: double; aPointShape: TThreeJsMathPointShape = psSphere; userdata: JSValue=0; aPointSize: double=0; aPointColor: TColor=$0000ff);
    procedure GetDataFromObject(obj: TJSObject; out x: double; out y: double; out z: double;
                     out aPointShape: TThreeJsMathPointShape; out aPointSize: double; out aPointColor: TColor; out aUserData: JSValue);
    procedure AddParametricSurface(xMin, xMax, yMin, yMax: double; numDivisions: integer; surfaceCallbackFunction: TThreeJsSurfaceCallBack);
    procedure SetSurfaceColors(colors: array of JSValue);
    procedure ResetSurfaceColors;
    function GetValueStr(aValue: double): string;

    property ChartType: TThreeJsMathChartType read FChartType write FChartType;

    // Resolution is nothing but the number of divisions
    property Resolution: integer read FNumDivisionsSurface write FNumDivisionsSurface;

    // Format string for FormatFloat function
    // To display value in all places
    property ValueFormatFloat: string read FValueFormatFloat write FValueFormatFloat;

    // Optional overrides, To display different value format on axis
    property XAxisValueFormat: string read FXAxisValueFormat write FXAxisValueFormat;
    property YAxisValueFormat: string read FYAxisValueFormat write FYAxisValueFormat;
    property ZAxisValueFormat: string read FZAxisValueFormat write FZAxisValueFormat;

    // properties of a Point Item
    property PointTextStyle: TThreeJsTextStyle read FPointTextStyle write FPointTextStyle;
    property DefaultPointSize: double read FDefaultPointSize write FDefaultPointSize;
    property DefaultPointColor: TColor read FDefaultPointColor write FDefaultPointColor;

    // Chart Title
    property Title: string read FTitle write FTitle;
    property TitleStyle: TThreeJsTextStyle read FTitleStyle write FTitleStyle;
    property TitleSpace: double read FTitleSpace write FTitleSpace;

    // Axis properties
    // --  Axis Titles
    property AxisTitleStyle: TThreeJsTextStyle read FAxisTitleStyle write FAxisTitleStyle;
    property XAxisTitle: string read FXAxisTitle write FXAxisTitle;
    property XAxisTitleSpace: double read FXAxisTitleSpace write FXAxisTitleSpace;
    property YAxisTitle: string read FYAxisTitle write FYAxisTitle;
    property YAxisTitleSpace: double read FYAxisTitleSpace write FYAxisTitleSpace;
    property ZAxisTitle: string read FZAxisTitle write FZAxisTitle;
    property ZAxisTitleSpace: double read FZAxisTitleSpace write FZAxisTitleSpace;

    //  -- Axis Lengths
    property XAxisLength: double read FXAxisLength write FXAxisLength;
    property YAxisLength: double read FYAxisLength write FYAxisLength;
    property ZAxisLength: double read FZAxisLength write FZAxisLength;

    // -- Axis Marks (range of values used for labeling axes)
    // If Auto flag is set, the following properties are determined
    // automatically from data
    property AutoScale: boolean read FAutoScale write FAutoScale;
    // Use the following properties only in non-Auto mode
    property XAxisMarkMaximum: double read FXAxisMarkMaximum write FXAxisMarkMaximum;
    property XAxisMarkMinimum: double read FXAxisMarkMinimum write FXAxisMarkMinimum;
    property XAxisMarkStep: double read FXAxisMarkStep write FXAxisMarkStep;
    property YAxisMarkMaximum: double read FYAxisMarkMaximum write FYAxisMarkMaximum;
    property YAxisMarkMinimum: double read FYAxisMarkMinimum write FYAxisMarkMinimum;
    property YAxisMarkStep: double read FYAxisMarkStep write FYAxisMarkStep;
    property ZAxisMarkMaximum: double read FZAxisMarkMaximum write FZAxisMarkMaximum;
    property ZAxisMarkMinimum: double read FZAxisMarkMinimum write FZAxisMarkMinimum;
    property ZAxisMarkStep: double read FZAxisMarkStep write FZAxisMarkStep;

    // -- Axis label styles
    property XAxisLabelStyle: TThreeJsTextStyle read FXAxisLabelStyle write FXAxisLabelStyle;
    property YAxisLabelStyle: TThreeJsTextStyle read FYAxisLabelStyle write FYAxisLabelStyle;
    property ZAxisLabelStyle: TThreeJsTextStyle read FZAxisLabelStyle write FZAxisLabelStyle;
    property AxisLabelSpace: double read FAxisLabelSpace write FAxisLabelSpace;

    // In math charts, displaying both Z and X starting labels make them clash
    // as they are almost at the same position. This flag, by default ON,
    // suppresses the starting Z label to avoid the clash.
    property SuppressZStartingLabel: boolean read FSuppressZStartingLabel write FSuppressZStartingLabel;
  end;

  TThreeJsPointItemEventArgs = class
  public
    property Item: TThreeJsObject3D read FItem write FItem;
    property X: double read FX write FX;
    property Y: double read FY write FY;
    property Z: double read FZ write FZ;
    property ValueStr: string read FValueStr write FValueStr;
  end;

  TThreeJsPointEvent = procedure(Sender: TObject; Args: TThreeJsPointItemEventArgs) of object;
  TThreeJsPointNotifyEvent = procedure(Sender: TObject; aPointItem: TThreeJsObject3D) of object;

  TThreeJsMathChart = class(TThreeJsCustomChart)
  protected
    procedure Initialize; override;
    function GetRaycasterTargetObjects: TJSArray; override;
    function EstimateMaxLabelWidthZAxis: double;
    procedure AddAxisLabels;
    procedure AddTitles;
    procedure EmbedChartItemInformation(theItem: TThreeJsObject3D; x, y, z: double; aPointShape: TThreeJsMathPointShape; aPointSize: double; aUserData: JSValue);
    function HandleObjectClick(chartObject: TThreeJsObject3D; intersection: TThreeJsIntersection): boolean; override;
    procedure HandleObjectExit(chartObject: TThreeJsObject3D); override;
    procedure HandleObjectDblClick(chartObject: TThreeJsObject3D); override;
    procedure HandleObjectMouseMove(chartObject: TThreeJsObject3D; intersection: TThreeJsIntersection); override;
    procedure HandleObjectMouseEnter(chartObject: TThreeJsObject3D); override;
    procedure HandleObjectMouseLeave(chartObject: TThreeJsObject3D); override;
    function GetGlowChartItem(chartItem: TThreeJsObject3D): boolean;
    procedure SetGlowChartItem(chartItem: TThreeJsObject3D; avalue: boolean);
    function GetItemValueLabelObject(x, y, z: double): TThreeJsObject3D;
    function GetMaxXExtent: double;
    function GetMaxZExtent: double;
    procedure TextureLoadFinished;
    function TextureLoaded(Event: TEventListenerEvent): boolean;
    procedure AddSurfacePlot;
  public
    procedure CreateInitialize; override;
    procedure CreateChartFromData; override;
    procedure ClearPointLabels;

    function GetItemObject(x, y, z: double): TThreeJsObject3D;
    function ShowPointValue(x, y, z: double; aValueStr: string): TThreeJsObject3D;
    procedure RemoveChartValue(aChartItem: TThreeJsObject3D);
    function ShowSurfacePointValue(x, y, z: double; aValueStr: string): TThreeJsObject3D;
    procedure RemoveSurfacePointValue;

    property Series: TThreeJsMathChartSeries read FSeries write FSeries;

    property Glow[chartItem: TThreeJsObject3D]: boolean read getGlowChartItem write setGlowChartItem;
    property ShowWireframe: boolean read FShowWireframe write FShowWireframe;
    property IsWireframeTexture: boolean read FIsWireframeTexture write FIsWireframeTexture;
    property CustomTextureUrl: string read FCustomTextureUrl write FCustomTextureUrl;

  published
    property OnClick;
    property OnDblClick;
    property OnItemClick: TThreeJsPointEvent read FOnItemClick write FOnItemClick;
    property OnItemExit: TThreeJsPointNotifyEvent read FOnItemExit write FOnItemExit;
    property OnItemDblClick: TThreeJsPointNotifyEvent read FOnItemDblClick write FOnItemDblClick;
    property OnItemMouseMove: TThreeJsPointEvent read FOnItemMouseMove write FOnItemMouseMove;
    property OnItemMouseEnter: TThreeJsPointNotifyEvent read FOnItemMouseEnter write FOnItemMouseEnter;
    property OnItemMouseLeave: TThreeJsPointNotifyEvent read FOnItemMouseLeave write FOnItemMouseLeave;
  end;

  TWebThreeJsMathChart = class(TThreeJsMathChart);


implementation
end.

